@@ -26,6 +26,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <arpa/inet.h>
 #include <netdb.h>
 #include <netinet/in.h>
+#include <glob.h>
 
 #include <zmq.h>
 #include <glib.h>
@@ -156,7 +157,8 @@ static gboolean
 service_expiration_notifier(struct conscience_srv_s *srv, gpointer u)
 {
 	(void) u;
-	if (srv) GRID_INFO("Service expired [%s]", srv->description);
+	if (srv) GRID_INFO("Service expired [%s] (score=%d)",
+			srv->description, srv->score.value);
 	return TRUE;
 }
 
@@ -168,21 +170,21 @@ timer_expire_services(gpointer u)
 	struct conscience_s *cs = u;
 
 	/* XXX start of critical section */
-	conscience_lock_srvtypes(cs,'r');
-	list_type_names = conscience_get_srvtype_names(cs,NULL);
+	conscience_lock_srvtypes(cs, 'r');
+	list_type_names = conscience_get_srvtype_names(cs, NULL);
 	conscience_unlock_srvtypes(cs);
 	/* XXX end of critical section */
 
 	if (!list_type_names) {
 		if (error_local) {
-			ERROR("[NS=%s] Failed to collect the service types names : %s",
+			ERROR("[NS=%s] Failed to collect the service types names: %s",
 				conscience_get_nsname(cs), gerror_get_message(error_local));
 			g_error_free(error_local);
 		}
 		return;
 	}
 
-	for (GSList *l=list_type_names; l ;l=g_slist_next(l)) {
+	for (GSList *l = list_type_names; l; l = g_slist_next(l)) {
 		if (!l->data)
 			continue;
 
@@ -197,7 +199,8 @@ timer_expire_services(gpointer u)
 				conscience_get_nsname(cs), str_name);
 			continue;
 		}
-		count = conscience_srvtype_remove_expired( srvtype, service_expiration_notifier, NULL);
+		count = conscience_srvtype_zero_expired(srvtype,
+				service_expiration_notifier, NULL);
 		conscience_release_locked_srvtype(srvtype);
 		/* XXX end of critical section */
 
@@ -205,57 +208,12 @@ timer_expire_services(gpointer u)
 			NOTICE("Expired [%u] [%s] services", count, str_name);
 	}
 
-	g_slist_foreach(list_type_names,g_free1,NULL);
+	g_slist_foreach(list_type_names, g_free1, NULL);
 	g_slist_free(list_type_names);
 }
 
-static gboolean
-request_body_matches_namespace(struct request_context_s *req_ctx, GError **err)
-{
-	GSList *list_ns = NULL;
-	int rc;
-
-	gsize body_size = 0;
-	void *body = metautils_message_get_BODY(req_ctx->request, &body_size);
-	if (!body)
-		return TRUE;
-
-	if (!strings_unmarshall(&list_ns, body, body_size, err)) {
-		GSETERROR(err, "Invalid request body");
-		return FALSE;
-	}
-
-	if (1 != g_slist_length(list_ns)) {
-		GSETCODE(err, CODE_BAD_REQUEST, "Too many namespaces, this conscience only manage exactly 1 NS");
-		rc = FALSE;
-	}
-	else {
-		gchar local_ns[LIMIT_LENGTH_NSNAME];
-		g_strlcpy(local_ns, conscience->ns_info.name, sizeof(local_ns));
-		rc = (0 == g_ascii_strcasecmp(local_ns, (gchar*)list_ns->data));
-	}
-
-	g_slist_foreach(list_ns, g_free1, NULL);
-	g_slist_free(list_ns);
-	return rc;
-}
-
 /* ------------------------------------------------------------------------- */
 
-static void
-_conscience_srv_clean_udata(struct conscience_srv_s *srv)
-{
-	if (!srv || srv->app_data_type != SAD_PTR)
-		return;
-	if (!srv->app_data.pointer.value)
-		return;
-	if (srv->app_data.pointer.cleaner)
-		srv->app_data.pointer.cleaner(srv->app_data.pointer.value);
-
-	srv->app_data.pointer.value = NULL;
-	srv->app_data.pointer.cleaner = NULL;
-}
-
 static GByteArray *
 _conscience_srv_serialize(struct conscience_srv_s *srv)
 {
@@ -309,7 +267,7 @@ static void
 _conscience_srv_prepare_cache(struct conscience_srv_s *srv)
 {
 	GByteArray *gba = _conscience_srv_serialize(srv);
-	_conscience_srv_clean_udata(srv);
+	conscience_srv_clean_udata(srv);
 	srv->app_data_type = SAD_PTR;
 	srv->app_data.pointer.value = gba;
 	srv->app_data.pointer.cleaner = metautils_gba_unref;
@@ -605,6 +563,7 @@ _srvinfo_append(struct srvget_s *sg, struct conscience_srv_s *srv)
 {
 	GByteArray *gba;
 
+	/* Serialize the service including its tags and stats */
 	if (sg->full) {
 		gba = _conscience_srv_serialize_full(srv);
 		if (gba) {
@@ -613,6 +572,8 @@ _srvinfo_append(struct srvget_s *sg, struct conscience_srv_s *srv)
 			return TRUE;
 		}
 	}
+	/* Reuse the serialized version of the service
+	 * that was made at registration time (saves CPU) */
 	else if (srv->app_data_type == SAD_PTR
 		&& NULL != (gba = srv->app_data.pointer.value))
 	{
@@ -621,6 +582,7 @@ _srvinfo_append(struct srvget_s *sg, struct conscience_srv_s *srv)
 			return TRUE;
 		}
 	}
+	/* Serialize the service without its tags and stats */
 	else if (NULL != (gba = _conscience_srv_serialize(srv))) {
 		g_byte_array_append(sg->gba_body, gba->data, gba->len);
 		sg->srv_list_size++;
@@ -806,18 +768,6 @@ handler_get_ns_info(struct request_context_s *req_ctx)
 
 	init_reply_ctx_with_request(req_ctx, &ctx);
 
-	if (!request_body_matches_namespace(req_ctx, &(ctx.warning))) {
-		if (!ctx.warning)
-			GSETCODE(&(ctx.warning), CODE_NAMESPACE_NOTMANAGED, "Invalid namespace");
-		reply_context_clear(&ctx, FALSE);
-		reply_context_set_message(&ctx, gerror_get_code(ctx.warning),
-				gerror_get_message(ctx.warning));
-		(void) reply_context_reply(&ctx, &(ctx.warning));
-		reply_context_log_access(&ctx, NULL);
-		reply_context_clear(&ctx, TRUE);
-		return 1;
-	}
-
 	g_rec_mutex_lock(&conscience_nsinfo_mutex);
 	gba = namespace_info_marshall(&(conscience->ns_info), &(ctx.warning));
 	g_rec_mutex_unlock(&conscience_nsinfo_mutex);
@@ -843,30 +793,34 @@ handler_get_services_types(struct request_context_s *req_ctx)
 	struct reply_context_s ctx;
 	GHashTableIter iterator;
 	gpointer k, v;
-	GSList *list_names;
 	GByteArray *gba_names;
 
 	init_reply_ctx_with_request(req_ctx, &(ctx));
-	list_names = NULL;
 
 	/* We avoid calling the similar feature from the conscience because
 	 * it makes a deep copy of the list. We do not perform any blocking
 	 * operation on the list, so we build ourselves a hollow copy and we
 	 * marshal it. */
 
-	/* XXX start of critical section */
+	GSList *namel = NULL;
+	/* start of critical section */
 	conscience_lock_srvtypes(conscience,'r');
 	g_hash_table_iter_init(&iterator, conscience->srvtypes);
 	while (g_hash_table_iter_next(&iterator, &k, &v)) {
-		if (k)
-			list_names = g_slist_prepend(list_names, k);
+		if (k) {
+			/* no copy, this only works because the types of services are
+			 * stable and never destroyed */
+			namel = g_slist_prepend(namel, k);
+		}
 	}
-
-	gba_names = strings_marshall_gba(list_names, &(ctx.warning));
 	conscience_unlock_srvtypes(conscience);
-	/* XXX end of critical section */
+	/* end of critical section */
 
-	g_slist_free(list_names);
+	/* do not free the pointers, they haven't been copied */
+	gchar **namev = (gchar**) metautils_list_to_array(namel);
+	g_slist_free (namel);
+	gba_names = STRV_encode_gba(namev);
+	g_free(namev);
 
 	/* Now we can manage the potential error and reply */
 	if (!gba_names) {
@@ -1011,12 +965,12 @@ plugin_handler(MESSAGE m, gint cnx, void *param, GError ** err)
 
 static gboolean
 module_configure_srvtype(struct conscience_s *cs, GError ** err,
-    const gchar * type, const gchar * what, const gchar * value)
+		const gchar * type, const gchar * what, const gchar * value)
 {
 	struct conscience_srvtype_s *srvtype;
 
 	if (!what || !value) {
-		GSETERROR(err, "Invalid Key/Value pair : %s/%s", what, value);
+		GSETERROR(err, "Invalid Key/Value pair: %s/%s", what, value);
 		return FALSE;
 	}
 
@@ -1026,40 +980,91 @@ module_configure_srvtype(struct conscience_s *cs, GError ** err,
 	else
 		srvtype = conscience_get_srvtype(cs, err, type, MODE_AUTOCREATE);
 	if (!srvtype) {
-		GSETERROR(err, "Failled to init a ServiceType");
+		GSETERROR(err, "Failed to init a ServiceType");
 		return FALSE;
 	}
 
 	/*adjust the parameter */
 	if (0 == g_ascii_strcasecmp(what, KEY_SCORE_TIMEOUT)) {
 		srvtype->score_expiration = g_ascii_strtoll(value, NULL, 10);
-		INFO("[NS=%s][SRVTYPE=%s] score expiration interval set to [%ld] seconds", cs->ns_info.name,
-		    srvtype->type_name, srvtype->score_expiration);
+		INFO("[NS=%s][SRVTYPE=%s] score expiration set to [%ld] seconds",
+				cs->ns_info.name, srvtype->type_name,
+				srvtype->score_expiration);
 		return TRUE;
 	}
 	else if (0 == g_ascii_strcasecmp(what, KEY_SCORE_VARBOUND)) {
 		srvtype->score_variation_bound = g_ascii_strtoll(value, NULL, 10);
-		INFO("[NS=%s][SRVTYPE=%s] score variation bound set to [%d]", cs->ns_info.name, srvtype->type_name, srvtype->score_variation_bound);
+		INFO("[NS=%s][SRVTYPE=%s] score variation bound set to [%d]",
+				cs->ns_info.name, srvtype->type_name,
+				srvtype->score_variation_bound);
 		return TRUE;
 	}
 	else if (0 == g_ascii_strcasecmp(what, KEY_SCORE_EXPR)) {
 		if (conscience_srvtype_set_type_expression(srvtype, err, value)) {
-			INFO("[NS=%s][SRVTYPE=%s] score expression set to [%s]", cs->ns_info.name, srvtype->type_name,
-			    value);
+			INFO("[NS=%s][SRVTYPE=%s] score expression set to [%s]",
+					cs->ns_info.name, srvtype->type_name, value);
 			return TRUE;
 		}
 		return FALSE;
 	}
+	else if (0 == g_ascii_strcasecmp(what, KEY_SCORE_LOCK)) {
+		srvtype->lock_at_first_register = oio_str_parse_bool(value, TRUE);
+		INFO("[NS=%s][SRVTYPE=%s] lock at first register: %s",
+				cs->ns_info.name, srvtype->type_name,
+				srvtype->lock_at_first_register? "yes":"no");
+		return TRUE;
+	}
 	else if (0 == g_ascii_strcasecmp(what, KEY_ALERT_LIMIT)) {
 		srvtype->alert_frequency_limit = g_ascii_strtoll(value, NULL, 10);
 		INFO("[NS=%s][SRVTYPE=%s] Alert limit set to %ld", cs->ns_info.name,
-		    srvtype->type_name, srvtype->alert_frequency_limit);
+				srvtype->type_name, srvtype->alert_frequency_limit);
 	}
 
-	WARN("[NS=%s][SRVTYPE=%s] parameter not recognized [%s] (ignored!)", cs->ns_info.name, srvtype->type_name, what);
+	WARN("[NS=%s][SRVTYPE=%s] parameter not recognized [%s] (ignored!)",
+			cs->ns_info.name, srvtype->type_name, what);
 	return TRUE;
 }
 
+static void
+module_configure_srvpool(struct conscience_s *cs, GError ** err,
+		const gchar *pool, const gchar * what, const gchar * value)
+{
+	EXTRA_ASSERT(pool != NULL);
+	EXTRA_ASSERT(what != NULL && *what != '\0');
+
+	if (!value || !*value) {
+		GSETERROR(err, "value is empty");
+		return;
+	}
+
+	GHashTable *svc_pools = cs->ns_info.service_pools;
+	GByteArray *gba = g_hash_table_lookup(svc_pools, pool);
+	if (!strcmp(what, KEY_POOL_TARGETS)) {
+		/* Targets are passed without key, but must start with a digit */
+		if (*value < '0' || *value > '9')
+			return;
+		if (!gba) {
+			gba = metautils_gba_from_string(value);
+			g_hash_table_insert(svc_pools, g_strdup(pool), gba);
+		} else {
+			g_byte_array_append(gba, (guint8*)";", 1);
+			g_byte_array_append(gba, (guint8*)value, strlen(value));
+		}
+	} else {
+		if (!gba) {
+			gchar buf[256] = {0};
+			g_snprintf(buf, sizeof(buf), "%s=%s", what, value);
+			gba = metautils_gba_from_string(buf);
+			g_hash_table_insert(svc_pools, g_strdup(pool), gba);
+		} else {
+			g_byte_array_append(gba, (guint8*)";", 1);
+			g_byte_array_append(gba, (guint8*)what, strlen(what));
+			g_byte_array_append(gba, (guint8*)"=", 1);
+			g_byte_array_append(gba, (guint8*)value, strlen(value));
+		}
+	}
+}
+
 static GHashTable*
 module_init_valuelist(GHashTable * params, GError ** err, GRegex *value_regex)
 {
@@ -1142,6 +1147,7 @@ fill_hashtable_with_group(GHashTable *ht, GKeyFile *conf_file, const gchar *grou
 				WARN("Duplicate key [%s][%s], new value [%s]", group_name, keys[i], v);
 			}
 			g_hash_table_insert (ht, g_strdup(keys[i]), metautils_gba_from_string(v));
+			g_free(v);
 		}
 		g_strfreev(keys);
 		return e;
@@ -1205,34 +1211,150 @@ module_init_storage_conf(struct conscience_s *cs, const gchar *stg_pol_in_option
 		WARN("Data security rules not correctly loaded from file [%s] : %s", filepath, e->message);
 		return e;
 	}
-	g_hash_table_foreach(cs->ns_info.data_security,
-			(GHFunc)_check_for_keyword, (gchar*[2]){DATA_SECURITY_NONE, "data security"});
-
-	// XXX TREATMENTS
-	cs->ns_info.data_treatments = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, metautils_gba_unref);
-	e = fill_hashtable_with_group(cs->ns_info.data_treatments, stg_conf_file, NAME_GROUPNAME_DATA_TREATMENTS);
-	if( NULL != e) {
-		WARN("Data treatments rules not correctly loaded from file [%s] : %s", filepath, e->message);
-		return e;
-	}
-	g_hash_table_foreach(cs->ns_info.data_treatments,
-			(GHFunc)_check_for_keyword, (gchar*[2]){DATA_TREATMENT_NONE, "data treatment"});
-
-	// XXX CLASSES
-	cs->ns_info.storage_class = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, metautils_gba_unref);
-	e = fill_hashtable_with_group(cs->ns_info.storage_class, stg_conf_file, NAME_GROUPNAME_STORAGE_CLASS);
-	if( NULL != e) {
-		WARN("Storage class rules not correctly loaded from file [%s] : %s", filepath, e->message);
-		return e;
-	}
-	g_hash_table_foreach(cs->ns_info.storage_class,
-			(GHFunc)_check_for_keyword, (gchar*[2]){STORAGE_CLASS_NONE, "storage class"});
+	g_hash_table_foreach(cs->ns_info.data_security, (GHFunc)_check_for_keyword,
+			(gchar*[2]){DATA_SECURITY_NONE, "data security"});
 
 	INFO("[NS=%s] storage conf loaded successfully from file [%s]",
 			cs->ns_info.name, filepath);
+	g_key_file_free(stg_conf_file);
 	return NULL;
 }
 
+static GError *
+_load_service_type_section(struct conscience_s *cs, GKeyFile *svc_conf_file,
+		const gchar *section)
+{
+	/*** sample **************************************************************
+	[type:meta0]
+	score_expr = root(3,((num stat.cpu)*(num stat.space)*(num stat.io)))
+	score_variation_bound = 5
+	score_timeout = 300
+	lock_at_first_register = false
+	*************************************************************************/
+	const char *svc = section + strlen(GROUP_PREFIX_TYPE);
+	GHashTable *content = g_hash_table_new_full(
+			g_str_hash, g_str_equal, g_free, metautils_gba_unref);
+	GError *err = fill_hashtable_with_group(content, svc_conf_file, section);
+	if (err) {
+		g_hash_table_destroy(content);
+		return err;
+	}
+	void _configure_section(gchar *key, GByteArray *gba, gpointer u UNUSED) {
+		GError *local_err = NULL;
+		module_configure_srvtype(cs, &local_err, svc, key,
+				(const char*)gba->data);  // FIXME: no '\0'
+		if (local_err) {
+			GRID_WARN("Failed to set %s for %s: %s",
+					key, svc, local_err->message);
+			g_clear_error(&local_err);
+		}
+	}
+	g_hash_table_foreach(content, (GHFunc)_configure_section, NULL);
+	g_hash_table_destroy(content);
+	return err;
+}
+
+static GError *
+_load_service_pool_section(struct conscience_s *cs, GKeyFile *svc_conf_file,
+		const gchar *section)
+{
+	/*** sample **************************************************************
+	[pool:rawx3]
+	targets = 1,rawx-even,rawx;1,rawx-odd,rawx;1,rawx
+	mask = FFFFFFFFFFFF0000
+	mask_max_shift = 16
+	nearby_mode=false
+	*************************************************************************/
+	const char *pool = section + strlen(GROUP_PREFIX_POOL);
+	GHashTable *content = g_hash_table_new_full(
+			g_str_hash, g_str_equal, g_free, metautils_gba_unref);
+	GError *err = fill_hashtable_with_group(content, svc_conf_file, section);
+	if (err) {
+		g_hash_table_destroy(content);
+		return err;
+	}
+	void _configure_section(gchar *key, GByteArray *gba, gpointer u UNUSED) {
+		GError *local_err = NULL;
+		module_configure_srvpool(cs, &local_err, pool, key,
+				(const char*)gba->data);  // FIXME: no '\0'
+		if (local_err) {
+			GRID_WARN("Failed to set %s for %s: %s",
+					key, pool, local_err->message);
+			g_clear_error(&local_err);
+		}
+	}
+	g_hash_table_foreach(content, (GHFunc)_configure_section, NULL);
+	g_hash_table_destroy(content);
+	return NULL;
+}
+
+static GError *
+_module_init_service_conf(struct conscience_s *cs, const gchar *filepath)
+{
+	GError *err = NULL;
+	GKeyFile *svc_conf_file = g_key_file_new();
+
+	if (!g_key_file_load_from_file(svc_conf_file, filepath,
+			G_KEY_FILE_NONE, &err)) {
+		GRID_WARN("[NS=%s] service configuration from %s failed: %s",
+				cs->ns_info.name, filepath, err->message);
+		g_key_file_free(svc_conf_file);
+		return err;
+	} else {
+		GRID_INFO("Loading service configuration from %s", filepath);
+	}
+
+	gchar **groups = g_key_file_get_groups(svc_conf_file, NULL);
+	for (gchar **group = groups; groups && *group; group++) {
+		if (g_str_has_prefix(*group, GROUP_PREFIX_POOL)) {
+			err = _load_service_pool_section(cs, svc_conf_file, *group);
+		} else if (g_str_has_prefix(*group, GROUP_PREFIX_TYPE)) {
+			err = _load_service_type_section(cs, svc_conf_file, *group);
+		} else {
+			GRID_WARN("Unknown configuration group: [%s] in file %s",
+					*group, filepath);
+		}
+		if (err) {
+			GRID_WARN("%s", err->message);
+			g_clear_error(&err);
+		}
+	}
+	g_strfreev(groups);
+	g_key_file_free(svc_conf_file);
+	return err;
+}
+
+static GError *
+module_init_service_conf_glob(struct conscience_s *cs, const gchar *pattern)
+{
+	GError *err = NULL;
+	cs->ns_info.service_pools = g_hash_table_new_full(g_str_hash, g_str_equal,
+			g_free, metautils_gba_unref);
+
+	glob_t globbuf;
+	int rc = glob(pattern, GLOB_MARK|GLOB_NOSORT|GLOB_BRACE, NULL, &globbuf);
+	switch (rc) {
+	case 0:
+		break;
+	case GLOB_NOMATCH:
+		GSETERROR(&err, "No file matched [%s]", pattern);
+		break;
+	default:
+		GSETERROR(&err, "Failed to do pattern matching with [%s] (code %d)",
+				pattern, rc);
+		break;
+	}
+
+	for (char **path = globbuf.gl_pathv; !err && path && *path; path++) {
+		if (g_str_has_suffix(*path, "/"))
+			continue;
+		err = _module_init_service_conf(cs, *path);
+	}
+
+	globfree(&globbuf);
+	return err;
+}
+
 static gboolean
 module_init_srvtype_from_cfg(struct conscience_s *cs, GHashTable * params, GError ** err)
 {
@@ -1241,10 +1363,11 @@ module_init_srvtype_from_cfg(struct conscience_s *cs, GHashTable * params, GErro
 	GRegex *service_param_regex = NULL;
 	GMatchInfo *match_info = NULL;
 
-	service_param_regex = g_regex_new("service\\.([^.]+)\\.(.+)", G_REGEX_CASELESS, G_REGEX_MATCH_NOTEMPTY, NULL);
+	service_param_regex = g_regex_new("service\\.([^.]+)\\.(.+)",
+			G_REGEX_CASELESS, G_REGEX_MATCH_NOTEMPTY, NULL);
 	if (!service_param_regex) {
-		GSETERROR(err,
-		    "Invalid regex for parameters parsing. Contact the development team, or Jethro Gibbs (NCIS).");
+		GSETERROR(err, "Invalid regex for parameters parsing. "
+				"Contact the development team, or Jethro Gibbs (NCIS).");
 		return FALSE;
 	}
 
@@ -1255,10 +1378,12 @@ module_init_srvtype_from_cfg(struct conscience_s *cs, GHashTable * params, GErro
 			g_match_info_free(match_info);
 			match_info = NULL;
 		}
-		if (!g_regex_match(service_param_regex, (gchar *) k, G_REGEX_MATCH_NOTEMPTY, &match_info))
-			TRACE("Parameter skipped : [%s] (does not match %s)", (gchar *) k, g_regex_get_pattern(service_param_regex));
+		if (!g_regex_match(service_param_regex, (gchar *) k,
+				G_REGEX_MATCH_NOTEMPTY, &match_info))
+			TRACE("Parameter skipped: [%s] (does not match %s)", (gchar *) k,
+					g_regex_get_pattern(service_param_regex));
 		else if (g_match_info_get_match_count(match_info) != 3)
-			WARN("Ignored parameter (invalid key) : %s", (gchar *) k);
+			WARN("Ignored parameter (invalid key): %s", (gchar *) k);
 		else {
 			gboolean rc_local;
 			gchar *type_str, *what;
@@ -1414,78 +1539,90 @@ _init_hub (GHashTable *params)
 static gint
 plugin_init(GHashTable * params, GError ** err)
 {
-	gchar *str;
+	gchar *str, *ns_name = NULL;
 
 	g_rec_mutex_init(&counters_mutex);
 	g_rec_mutex_init(&conscience_nsinfo_mutex);
 
 	/*NAMEPSACE name */
-	if (!(str = g_hash_table_lookup(params, KEY_NAMESPACE))) {
+	if (!(ns_name = g_hash_table_lookup(params, KEY_NAMESPACE))) {
 		GSETERROR(err, "The configuration must contain a '%s' key with the namespace name", KEY_NAMESPACE);
 		return -1;
 	}
-	if (!(conscience = conscience_create_named(str, err))) {
+	if (!(conscience = conscience_create_named(ns_name, err))) {
 		GSETERROR(err, "Conscience allocation failure");
 		return -1;
 	}
-	NOTICE("[NS=%s] Configuring a new conscience", conscience->ns_info.name);
+	NOTICE("[NS=%s] Configuring a new conscience", ns_name);
 
 	/*CHUNK SIZE */
 	if (!(str = g_hash_table_lookup(params, KEY_CHUNK_SIZE))) {
 		GSETERROR(err, "[NS=%s] Missing key '%s' (chunk size in bytes)",
-		    conscience->ns_info.name, KEY_CHUNK_SIZE);
+				ns_name, KEY_CHUNK_SIZE);
 		goto error;
 	}
 	conscience->ns_info.chunk_size = g_ascii_strtoll(str, NULL, 10);
-	NOTICE("[NS=%s] Chunk size set to %"G_GINT64_FORMAT, conscience->ns_info.name, conscience->ns_info.chunk_size);
+	NOTICE("[NS=%s] Chunk size set to %"G_GINT64_FORMAT, ns_name, conscience->ns_info.chunk_size);
 
 	/* Serialization optimizations */
 	str = g_hash_table_lookup(params, KEY_SERIALIZE_SRVINFO_TAGS);
 	if (NULL != str)
-		flag_serialize_srvinfo_tags = metautils_cfg_get_bool(str, DEF_SERIALIZE_SRVINFO_TAGS);
-	NOTICE("[NS=%s] Tags in serialized service_info  [%s]", conscience->ns_info.name,
+		flag_serialize_srvinfo_tags = oio_str_parse_bool(str, DEF_SERIALIZE_SRVINFO_TAGS);
+	NOTICE("[NS=%s] Tags in serialized service_info  [%s]", ns_name,
 			(flag_serialize_srvinfo_tags ? "ENABLED" : "DISABLED"));
 
 	str = g_hash_table_lookup(params, KEY_SERIALIZE_SRVINFO_STATS);
 	if (NULL != str)
-		flag_serialize_srvinfo_stats = metautils_cfg_get_bool(str, DEF_SERIALIZE_SRVINFO_STATS);
-	NOTICE("[NS=%s] Stats in serialized service_info  [%s]", conscience->ns_info.name,
+		flag_serialize_srvinfo_stats = oio_str_parse_bool(str, DEF_SERIALIZE_SRVINFO_STATS);
+	NOTICE("[NS=%s] Stats in serialized service_info  [%s]", ns_name,
 			(flag_serialize_srvinfo_stats ? "ENABLED" : "DISABLED"));
 
 	/*Overall alerting maximum per-service frequency*/
 	if (!(str = g_hash_table_lookup(params, KEY_ALERT_LIMIT)))
 		NOTICE("[NS=%s] No overall alert_frequency_limit set, default kept to [%ld] seconds",
-			conscience->ns_info.name, time_default_alert_frequency);
+			ns_name, time_default_alert_frequency);
 	else {
 		time_default_alert_frequency = g_ascii_strtoll(str,NULL,10);
-		NOTICE("[NS=%s] Overall alert_frequency_limit set to [%ld] seconds", conscience->ns_info.name, time_default_alert_frequency);
+		NOTICE("[NS=%s] Overall alert_frequency_limit set to [%ld] seconds", ns_name, time_default_alert_frequency);
 	}
 
 	/* OPTIONS initialization */
 	if (!module_init_options(conscience, params, err)) {
-		GSETERROR(err, "[NS=%s] options init failed", conscience->ns_info.name);
+		GSETERROR(err, "[NS=%s] options init failed", ns_name);
 		goto error;
 	}
 
 	/* storage conf initialization */
-	*err = module_init_storage_conf(conscience, namespace_storage_policy(&conscience->ns_info, conscience->ns_info.name),
+	*err = module_init_storage_conf(conscience,
+			namespace_storage_policy(&conscience->ns_info, ns_name),
 			g_hash_table_lookup(params, KEY_STG_CONF));
 	if( NULL != *err ) {
-		g_prefix_error(err, "[NS=%s] storage conf init failed", conscience->ns_info.name);
+		g_prefix_error(err, "[NS=%s] storage conf init failed", ns_name);
 		goto error;
 	}
 
 	/*SERVICES initiation*/
 	if (!module_init_known_service_types(conscience, params, err)) {
-		GSETERROR(err, "[NS=%s] known service types init failed", conscience->ns_info.name);
+		GSETERROR(err, "[NS=%s] known service types init failed", ns_name);
 		goto error;
 	}
 
+	/* TODO: remove this!*/
+	/* service conf initialization (old style) */
 	if (!module_init_srvtype_from_cfg(conscience, params, err)) {
 		GSETERROR(err, "Configuration error");
 		goto error;
 	}
 
+	/* service conf initialization (new style) */
+	*err = module_init_service_conf_glob(conscience,
+			g_hash_table_lookup(params, KEY_SVC_CONF));
+	if (*err) {
+		g_prefix_error(err, "[NS=%s] service conf init failed",
+				ns_name);
+		goto error;
+	}
+
 	if (!_init_hub (params)) {
 		GSETERROR(err, "Failed to prepare the Conscience's HUB");
 		goto error;
@@ -1507,6 +1644,8 @@ plugin_init(GHashTable * params, GError ** err)
 
 	return 1;
 error:
+	GRID_ERROR("Conscience loading failed: %s",
+			*err?(*err)->message:"unknown error");
 	conscience_destroy(conscience);
 	conscience = NULL;
 	return -1;
@@ -1530,7 +1669,7 @@ plugin_reload(GHashTable * params, GError ** err)
 	/*CHUNK SIZE */
 	if (!(str = g_hash_table_lookup(params, KEY_CHUNK_SIZE))) {
 		GSETERROR(err, "[NS=%s] Missing key '%s' (chunk size in bytes)",
-		    conscience->ns_info.name, KEY_CHUNK_SIZE);
+				conscience->ns_info.name, KEY_CHUNK_SIZE);
 		goto error;
 	}
 	conscience->ns_info.chunk_size = g_ascii_strtoll(str, NULL, 10);
@@ -1550,8 +1689,6 @@ plugin_reload(GHashTable * params, GError ** err)
 	/* storage conf reload */
 	g_hash_table_destroy(conscience->ns_info.storage_policy);
 	g_hash_table_destroy(conscience->ns_info.data_security);
-	g_hash_table_destroy(conscience->ns_info.data_treatments);
-	g_hash_table_destroy(conscience->ns_info.storage_class);
 	*err = module_init_storage_conf(conscience, namespace_storage_policy(&conscience->ns_info, conscience->ns_info.name),
 			g_hash_table_lookup(params, KEY_STG_CONF));
 	if( NULL != *err ) {
