@@ -170,10 +170,10 @@ http_put_create(gint64 content_length, gint64 soft_length)
 struct http_put_dest_s *
 http_put_add_dest(struct http_put_s *p, const char *url, gpointer u)
 {
-	g_assert(p != NULL);
-	g_assert(p->state == HTTP_WHOLE_BEGIN);
-	g_assert(url != NULL);
-	g_assert(u != NULL);
+	EXTRA_ASSERT(p != NULL);
+	EXTRA_ASSERT(p->state == HTTP_WHOLE_BEGIN);
+	EXTRA_ASSERT(url != NULL);
+	EXTRA_ASSERT(u != NULL);
 
 	struct http_put_dest_s *dest = g_try_malloc0(sizeof(struct http_put_dest_s));
 	dest->http_put = p;
@@ -199,9 +199,9 @@ http_put_dest_add_header(struct http_put_dest_s *dest,
 {
 	gchar *val = NULL;
 
-	g_assert(dest != NULL);
-	g_assert(key != NULL);
-	g_assert(val_fmt != NULL);
+	EXTRA_ASSERT(dest != NULL);
+	EXTRA_ASSERT(key != NULL);
+	EXTRA_ASSERT(val_fmt != NULL);
 
 	va_list ap;
 	va_start(ap, val_fmt);
@@ -220,14 +220,15 @@ http_put_dest_destroy(gpointer destination)
 {
 	struct http_put_dest_s *dest = destination;
 
-	g_assert(dest != NULL);
+	EXTRA_ASSERT(dest != NULL);
 
 	if (dest->url)
 		g_free(dest->url);
 	if (dest->handle) {
 		CURLMcode rc;
 		rc = curl_multi_remove_handle(dest->http_put->mhandle, dest->handle);
-		g_assert(rc == CURLM_OK);
+		EXTRA_ASSERT(rc == CURLM_OK);
+		(void)rc;
 		curl_easy_cleanup(dest->handle);
 	}
 	if (dest->headers)
@@ -237,6 +238,11 @@ http_put_dest_destroy(gpointer destination)
 	if (dest->response_headers)
 		g_hash_table_destroy(dest->response_headers);
 
+	if (dest->buffer) {
+		g_bytes_unref(dest->buffer);
+		dest->buffer = NULL;
+	}
+
 	g_free(dest);
 }
 
@@ -259,7 +265,7 @@ http_put_destroy(struct http_put_s *p)
 gint64
 http_put_expected_bytes (struct http_put_s *p)
 {
-	g_assert(p != NULL);
+	EXTRA_ASSERT(p != NULL);
 	if (http_put_done (p))
 		return 0;
 	return p->remaining_length;
@@ -268,11 +274,11 @@ http_put_expected_bytes (struct http_put_s *p)
 void
 http_put_feed (struct http_put_s *p, GBytes *b)
 {
-	g_assert (p != NULL);
-	g_assert (b != NULL);
+	EXTRA_ASSERT (p != NULL);
+	EXTRA_ASSERT (b != NULL);
 	gssize len = g_bytes_get_size (b);
 	GRID_TRACE("%s (%p) <- %"G_GSIZE_FORMAT, __FUNCTION__, p, len);
-	g_assert (len <= 0 || p->remaining_length < 0 || len <= p->remaining_length);
+	EXTRA_ASSERT (len <= 0 || p->remaining_length < 0 || len <= p->remaining_length);
 
 	g_queue_push_tail (p->buffer_tail, b);
 
@@ -286,14 +292,14 @@ http_put_feed (struct http_put_s *p, GBytes *b)
 gboolean
 http_put_done (struct http_put_s *p)
 {
-	g_assert (p != NULL);
+	EXTRA_ASSERT (p != NULL);
 	return p->state == HTTP_WHOLE_FINISHED;
 }
 
 guint
 http_put_get_failure_number(struct http_put_s *p)
 {
-	g_assert(p != NULL);
+	EXTRA_ASSERT(p != NULL);
 	guint ret = 0;
 	for (GSList *l = p->dests ; NULL != l ; l = l->next) {
 		struct http_put_dest_s *d = l->data;
@@ -306,9 +312,9 @@ http_put_get_failure_number(struct http_put_s *p)
 const char *
 http_put_get_header(struct http_put_s *p, gpointer k, const char *h)
 {
-	g_assert(p != NULL);
-	g_assert(k != NULL);
-	g_assert(h != NULL);
+	EXTRA_ASSERT(p != NULL);
+	EXTRA_ASSERT(k != NULL);
+	EXTRA_ASSERT(h != NULL);
 	for (GSList *l=p->dests; l ;l=l->next) {
 		struct http_put_dest_s *dest = l->data;
 		if (dest && dest->user_data == k)
@@ -320,8 +326,8 @@ http_put_get_header(struct http_put_s *p, gpointer k, const char *h)
 guint
 http_put_get_http_code(struct http_put_s *p, gpointer k)
 {
-	g_assert(p != NULL);
-	g_assert(k != NULL);
+	EXTRA_ASSERT(p != NULL);
+	EXTRA_ASSERT(k != NULL);
 	for (GSList *l=p->dests; l ;l=l->next) {
 		struct http_put_dest_s *dest = l->data;
 		if (dest && dest->user_data == k)
@@ -333,18 +339,18 @@ http_put_get_http_code(struct http_put_s *p, gpointer k)
 void
 http_put_get_md5(struct http_put_s *p, guint8 *buffer, gsize size)
 {
-	g_assert (p != NULL);
-	g_assert (buffer != NULL);
+	EXTRA_ASSERT (p != NULL);
+	EXTRA_ASSERT (buffer != NULL);
 	memset (buffer, 0, size);
-	/* XXX TODO */
+	/* TODO FIXME */
+	(void)p;
 }
 
 /* -------------------------------------------------------------------------- */
 
 static size_t
-_done_reading (struct http_put_dest_s *dest, const char *why)
+_done_reading (struct http_put_dest_s *dest, const char *why UNUSED)
 {
-	(void) why;
 	if (dest->buffer) {
 		g_bytes_unref (dest->buffer);
 		dest->buffer = NULL;
@@ -355,9 +361,9 @@ _done_reading (struct http_put_dest_s *dest, const char *why)
 }
 
 static size_t
-cb_read(void *data, size_t s, size_t n, struct http_put_dest_s *dest)
+cb_read(char *data, size_t s, size_t n, struct http_put_dest_s *dest)
 {
-	g_assert (dest->state == HTTP_SINGLE_BEGIN || dest->state == HTTP_SINGLE_REQUEST);
+	EXTRA_ASSERT (dest->state == HTTP_SINGLE_BEGIN || dest->state == HTTP_SINGLE_REQUEST);
 	dest->state = HTTP_SINGLE_REQUEST;
 
 	if (dest->http_put->content_length == 0)
@@ -408,18 +414,18 @@ cb_read(void *data, size_t s, size_t n, struct http_put_dest_s *dest)
 }
 
 static size_t
-cb_write(void *data, size_t size, size_t nmemb, gpointer nothing)
+cb_write(char *data UNUSED, size_t size, size_t nmemb, gpointer u UNUSED)
 {
-	(void)data, (void)nothing;
 	return size * nmemb;
 }
 
 static size_t
-cb_header(void *ptr, size_t size, size_t nmemb, struct http_put_dest_s *dest)
+cb_header(char *ptr, size_t size, size_t nmemb, void *raw_dest)
 {
-	g_assert(ptr != NULL);
-	g_assert(dest != NULL);
+	EXTRA_ASSERT(ptr != NULL);
+	EXTRA_ASSERT(raw_dest != NULL);
 
+	struct http_put_dest_s *dest = raw_dest;
 	int len = size * nmemb;
 	gchar *header = ptr; /* /!\ not nul-terminated */
 	gchar *tmp = g_strstr_len(header, len, ":");
@@ -440,13 +446,13 @@ _start_upload(struct http_put_s *p)
 	for (GSList *l = p->dests ; NULL != l ; l = l->next) {
 		struct http_put_dest_s *dest = l->data;
 
-		g_assert (dest->state == HTTP_SINGLE_BEGIN);
-		g_assert (dest->http_code == 0);
-		g_assert (dest->bytes_sent == 0);
-		g_assert (dest->handle == NULL);
+		EXTRA_ASSERT (dest->state == HTTP_SINGLE_BEGIN);
+		EXTRA_ASSERT (dest->http_code == 0);
+		EXTRA_ASSERT (dest->bytes_sent == 0);
+		EXTRA_ASSERT (dest->handle == NULL);
 
 		dest->handle = _curl_get_handle_blob();
-		g_assert(dest->handle != NULL);
+		EXTRA_ASSERT(dest->handle != NULL);
 
 		curl_easy_setopt(dest->handle, CURLOPT_CONNECTTIMEOUT, p->timeout_cnx);
 		curl_easy_setopt(dest->handle, CURLOPT_TIMEOUT, p->timeout_op);
@@ -461,15 +467,18 @@ _start_upload(struct http_put_s *p)
 			http_put_dest_add_header(dest, "Transfer-Encoding", "chunked");
 		http_put_dest_add_header(dest, "Expect", " ");
 
-		curl_easy_setopt(dest->handle, CURLOPT_READFUNCTION, cb_read);
+		curl_easy_setopt(dest->handle, CURLOPT_READFUNCTION,
+				(curl_read_callback)cb_read);
 		curl_easy_setopt(dest->handle, CURLOPT_READDATA, dest);
-		curl_easy_setopt(dest->handle, CURLOPT_WRITEFUNCTION, cb_write);
+		curl_easy_setopt(dest->handle, CURLOPT_WRITEFUNCTION,
+				(curl_write_callback)cb_write);
 		curl_easy_setopt(dest->handle, CURLOPT_HTTPHEADER, dest->curl_headers);
 		curl_easy_setopt(dest->handle, CURLOPT_HEADERFUNCTION, cb_header);
 		curl_easy_setopt(dest->handle, CURLOPT_HEADERDATA, dest);
 
 		CURLMcode rc = curl_multi_add_handle(p->mhandle, dest->handle);
-		g_assert(rc == CURLM_OK);
+		EXTRA_ASSERT(rc == CURLM_OK);
+		(void)rc;
 	}
 }
 
@@ -487,9 +496,9 @@ _manage_curl_events (struct http_put_s *p)
 			CURLcode curl_ret = msg->data.result;
 			struct http_put_dest_s *dest = NULL;
 
-			curl_easy_getinfo(easy, CURLINFO_PRIVATE, &dest);
-			g_assert (easy == dest->handle);
-			g_assert (dest->state != HTTP_SINGLE_FINISHED);
+			curl_easy_getinfo(easy, CURLINFO_PRIVATE, (char**)&dest);
+			EXTRA_ASSERT (easy == dest->handle);
+			EXTRA_ASSERT (dest->state != HTTP_SINGLE_FINISHED);
 			dest->state = HTTP_SINGLE_FINISHED;
 
 			long http_ret;
@@ -498,11 +507,17 @@ _manage_curl_events (struct http_put_s *p)
 			if (curl_ret == CURLE_OK)
 				dest->http_code = http_ret;
 
-			GRID_TRACE("DONE [%s] code=%ld strerror=%s",
-					dest->url, http_ret, curl_easy_strerror(curl_ret));
+			if (http_ret / 100 == 2) {
+				GRID_TRACE("DONE [%s] code=%ld strerror=%s",
+						dest->url, http_ret, curl_easy_strerror(curl_ret));
+			} else {
+				GRID_INFO("ERROR [%s] code=%ld strerror=%s",
+						dest->url, http_ret, curl_easy_strerror(curl_ret));
+			}
 
 			CURLMcode rc = curl_multi_remove_handle(p->mhandle, dest->handle);
-			g_assert(rc == CURLM_OK);
+			EXTRA_ASSERT(rc == CURLM_OK);
+			(void)rc;
 			curl_easy_cleanup(dest->handle);
 			dest->handle = NULL;
 			g_bytes_unref(dest->buffer);
@@ -527,9 +542,9 @@ GError *
 http_put_step (struct http_put_s *p)
 {
 	int rc;
-	guint count_dests = 0, count_up = 0, count_waiting_for_data = 0;
+	guint count_up = 0, count_waiting_for_data = 0;
 
-	g_assert (p != NULL);
+	EXTRA_ASSERT (p != NULL);
 
 	if (!p->dests) {
 		GRID_TRACE("%s Empty upload detected", __FUNCTION__);
@@ -541,7 +556,7 @@ http_put_step (struct http_put_s *p)
 		return NULL;
 	}
 
-	count_dests = g_slist_length(p->dests);
+	register guint count_dests = g_slist_length(p->dests);
 	GRID_TRACE("%s STEP on %u destinations", __FUNCTION__, count_dests);
 
 	/* consume the CURL notifications for terminated actions */
@@ -552,14 +567,20 @@ http_put_step (struct http_put_s *p)
 		struct http_put_dest_s *d = l->data;
 		if (d->state == HTTP_SINGLE_FINISHED)
 			continue;
+		count_up ++;
 		if (!d->buffer && d->state < HTTP_SINGLE_FINISHED)
 			count_waiting_for_data ++;
 	}
-	if (count_waiting_for_data >= count_dests) {
+	EXTRA_ASSERT(count_waiting_for_data <= count_up);
+	if (count_waiting_for_data == count_up) {
 		GBytes *buf = g_queue_pop_head (p->buffer_tail);
 		if (buf) {
 			for (GSList *l=p->dests; l ;l=l->next) {
 				struct http_put_dest_s *d = l->data;
+				if (d->buffer) {
+					g_bytes_unref(d->buffer);
+					d->buffer = NULL;
+				}
 				d->buffer = g_bytes_ref (buf);
 			}
 			g_bytes_unref (buf);
@@ -567,7 +588,7 @@ http_put_step (struct http_put_s *p)
 	}
 
 	if (p->state == HTTP_WHOLE_BEGIN) {
-		GRID_TRACE("%s Starting %u uploads", __FUNCTION__, count_dests);
+		GRID_DEBUG("%s Starting %u uploads", __FUNCTION__, count_dests);
 		_start_upload(p);
 		p->state = HTTP_WHOLE_READY;
 	}
@@ -633,29 +654,45 @@ retry:
 
 /* -------------------------------------------------------------------------- */
 
-static int
-_trace(CURL *h, curl_infotype t, char *data, size_t size, void *u)
+static const char *
+_trace_tag (const curl_infotype t)
 {
-	(void) h, (void) u, (void) data, (void) size;
 	switch (t) {
 		case CURLINFO_TEXT:
-			GRID_TRACE2("CURL: %.*s", (int)size, data);
-			return 0;
+			return "CURL:";
 		case CURLINFO_HEADER_IN:
-			GRID_TRACE2("CURL< %.*s", (int)size, data);
-			return 0;
+			return "CURL<";
 		case CURLINFO_HEADER_OUT:
-			GRID_TRACE2("CURL> %.*s", (int)size, data);
-			return 0;
+			return "CURL>";
 		default:
-			return 0;
+			return NULL;
 	}
 }
 
 static int
-_curl_set_sockopt_common (void *u, curl_socket_t fd, curlsocktype event)
+_trace(CURL *h UNUSED, curl_infotype t, char *data UNUSED, size_t size UNUSED,
+		void *u UNUSED)
+{
+	GString *tmp = g_string_new("");
+	for (; size>0 ;++data,--size) {
+		if (g_ascii_isprint(*data) && !g_ascii_isspace(*data)) {
+			g_string_append_c (tmp, *data);
+		} else {
+			g_string_append_c (tmp, ' ');
+		}
+	}
+
+	const char *tag = _trace_tag(t);
+	if (tag) {
+		GRID_TRACE2("%s %.*s", tag, (int)tmp->len, tmp->str);
+	}
+	g_string_free (tmp, TRUE);
+	return 0;
+}
+
+static int
+_curl_set_sockopt_common (void *u UNUSED, curl_socket_t fd, curlsocktype event)
 {
-	g_assert_null (u);
 	if (event == CURLSOCKTYPE_IPCXN) {
 		int opt = 1;
 		setsockopt (fd, SOL_SOCKET, SO_REUSEADDR, (void*)&opt, sizeof(opt));
@@ -697,11 +734,18 @@ CURL *
 _curl_get_handle_blob (void)
 {
 	CURL *h = curl_easy_init ();
+	curl_easy_setopt (h, CURLOPT_FORBID_REUSE, 1L);
+	curl_easy_setopt (h, CURLOPT_NOSIGNAL, 1L);
+	curl_easy_setopt (h, CURLOPT_FRESH_CONNECT, 1L);
 	curl_easy_setopt (h, CURLOPT_USERAGENT, OIOSDS_http_agent);
 	curl_easy_setopt (h, CURLOPT_NOPROGRESS, 1L);
-	curl_easy_setopt (h, CURLOPT_PROXY, NULL);
+	curl_easy_setopt (h, CURLOPT_PROXY, "");
 	curl_easy_setopt (h, CURLOPT_SOCKOPTDATA, NULL);
 	curl_easy_setopt (h, CURLOPT_SOCKOPTFUNCTION, _curl_set_sockopt_blob);
+	if (GRID_TRACE2_ENABLED()) {
+		curl_easy_setopt (h, CURLOPT_DEBUGFUNCTION, _trace);
+		curl_easy_setopt (h, CURLOPT_VERBOSE, 1L);
+	}
 	return h;
 }
 
@@ -710,13 +754,15 @@ _curl_get_handle_proxy (void)
 {
 	CURL *h = curl_easy_init ();
 	curl_easy_setopt (h, CURLOPT_USERAGENT, OIOSDS_http_agent);
+	curl_easy_setopt (h, CURLOPT_NOSIGNAL, 1L);
+	curl_easy_setopt (h, CURLOPT_TCP_NODELAY, 1L);
 	curl_easy_setopt (h, CURLOPT_NOPROGRESS, 1L);
-	curl_easy_setopt (h, CURLOPT_PROXY, NULL);
+	curl_easy_setopt (h, CURLOPT_PROXY, "");
 	curl_easy_setopt (h, CURLOPT_FORBID_REUSE, 0L);
 	curl_easy_setopt (h, CURLOPT_FRESH_CONNECT, 0L);
 	curl_easy_setopt (h, CURLOPT_SOCKOPTDATA, NULL);
 	curl_easy_setopt (h, CURLOPT_SOCKOPTFUNCTION, _curl_set_sockopt_proxy);
-	if (GRID_TRACE_ENABLED()) {
+	if (GRID_TRACE2_ENABLED()) {
 		curl_easy_setopt (h, CURLOPT_DEBUGFUNCTION, _trace);
 		curl_easy_setopt (h, CURLOPT_VERBOSE, 1L);
 	}
