@@ -1,6 +1,6 @@
 /*
 OpenIO SDS core library
-Copyright (C) 2015 OpenIO, original work as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2016 OpenIO, work as part of OpenIO Software Defined Storage
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -30,6 +30,23 @@ License along with this library.
 
 /* -------------------------------------------------------------------------- */
 
+
+static GString *
+_build_json (const char* const *src, GString *json)
+{
+	if (!json)
+		json = g_string_new ("");
+	g_string_append_c(json, '{');
+	for (int i = 0; src [i] && src [i+1] ; i +=2 ) {
+		if (i != 0)
+			g_string_append (json,",");
+		oio_str_gstring_append_json_pair (json, src [i], src [i+1]);
+	}
+	g_string_append (json, "}");
+	return json;
+}
+
+
 static void
 _ptrv_free_content (gchar **tab)
 {
@@ -42,33 +59,29 @@ enum _prefix_e { PREFIX_CONSCIENCE, PREFIX_REFERENCE, PREFIX_CONTAINER };
 static GString *
 _curl_url_prefix (const char *ns, enum _prefix_e which)
 {
-	GString *hu = g_string_new("http://");
-
 	if (!ns) {
 		GRID_WARN ("BUG No namespace configured!");
-		g_string_append (hu, "proxy");
-	} else {
-		gchar *s = NULL;
-
-		if (which == PREFIX_CONSCIENCE)
-			s = oio_cfg_get_proxy_conscience (ns);
-		else if (which == PREFIX_CONTAINER)
-			s = oio_cfg_get_proxy_containers (ns);
-		else if (which == PREFIX_REFERENCE)
-			s = oio_cfg_get_proxy_directory (ns);
-
-		if (!s)
-			s = oio_cfg_get_proxy (ns);
-
-		if (!s) {
-			GRID_WARN ("No proxy configured!");
-			g_string_append (hu, "proxy");
-		} else {
-			g_string_append (hu, s);
-			g_free (s);
-		}
+		return NULL;
 	}
 
+	gchar *s = NULL;
+
+	if (which == PREFIX_CONSCIENCE)
+		s = oio_cfg_get_proxy_conscience (ns);
+	else if (which == PREFIX_CONTAINER)
+		s = oio_cfg_get_proxy_containers (ns);
+	else if (which == PREFIX_REFERENCE)
+		s = oio_cfg_get_proxy_directory (ns);
+	if (!s)
+		s = oio_cfg_get_proxy (ns);
+	if (!s) {
+		GRID_WARN ("No proxy configured!");
+		return NULL;
+	}
+
+	GString *hu = g_string_new("http://");
+	g_string_append (hu, s);
+	g_free (s);
 	return hu;
 }
 
@@ -97,6 +110,8 @@ static GString *
 _curl_reference_url (struct oio_url_s *u, const char *action)
 {
 	GString *hu = _curl_url_prefix_reference (u);
+	if (!hu) return NULL;
+
 	g_string_append_printf (hu, "/%s/%s/reference/%s", PROXYD_PREFIX,
 			oio_url_get(u, OIOURL_NS), action);
 	_append (hu, '?', "acct", oio_url_get (u, OIOURL_ACCOUNT));
@@ -108,6 +123,8 @@ static GString *
 _curl_conscience_url (const char *ns, const char *action)
 {
 	GString *hu = _curl_url_prefix (ns, PREFIX_CONSCIENCE);
+	if (!hu) return NULL;
+
 	g_string_append_printf (hu, "/%s/%s/conscience/%s", PROXYD_PREFIX,
 			ns, action);
 	return hu;
@@ -126,6 +143,8 @@ static GString *
 _curl_container_url (struct oio_url_s *u, const char *action)
 {
 	GString *hu = _curl_url_prefix_containers (u);
+	if (!hu) return NULL;
+
 	g_string_append_printf (hu, "/%s/%s/container/%s", PROXYD_PREFIX,
 			oio_url_get(u, OIOURL_NS), action);
 	_append (hu, '?', "acct", oio_url_get (u, OIOURL_ACCOUNT));
@@ -138,6 +157,8 @@ static GString *
 _curl_content_url (struct oio_url_s *u, const char *action)
 {
 	GString *hu = _curl_url_prefix_containers (u);
+	if (!hu) return NULL;
+
 	g_string_append_printf (hu, "/%s/%s/content/%s", PROXYD_PREFIX,
 			oio_url_get(u, OIOURL_NS), action);
 	_append (hu, '?', "acct", oio_url_get (u, OIOURL_ACCOUNT));
@@ -152,25 +173,32 @@ _curl_content_url (struct oio_url_s *u, const char *action)
 static GError *
 _body_parse_error (GString *b)
 {
-	g_assert (b != NULL);
+	EXTRA_ASSERT (b != NULL);
 	struct json_tokener *tok = json_tokener_new ();
 	struct json_object *jbody = json_tokener_parse_ex (tok, b->str, b->len);
+	enum json_tokener_error parsing_error = json_tokener_get_error (tok);
 	json_tokener_free (tok);
 	tok = NULL;
 
+	if (json_tokener_success != parsing_error) {
+		if (jbody) json_object_put(jbody);
+		return NEWERROR(0, "unknown error (invalid json: %s)",
+				json_tokener_error_desc(parsing_error));
+	}
+
 	if (!jbody)
-		return NEWERROR(0, "No error explained");
+		return NEWERROR(0, "unknown error (empty body or null json)");
 
 	struct json_object *jcode, *jmsg;
 	struct oio_ext_json_mapping_s map[] = {
-		{"status", &jcode, json_type_int,    0},
+		{"status", &jcode, json_type_int, 0},
 		{"message",  &jmsg,  json_type_string, 0},
 		{NULL, NULL, 0, 0}
 	};
 	GError *err =  oio_ext_extract_json(jbody, map);
 	if (!err) {
 		int code = 0;
-		const char *msg = "Unknown error";
+		const char *msg = "Unknown error (unexpected json)";
 		if (jcode) code = json_object_get_int64 (jcode);
 		if (jmsg) msg = json_object_get_string (jmsg);
 		err = NEWERROR(code, "(code=%d) %s", code, msg);
@@ -180,14 +208,13 @@ _body_parse_error (GString *b)
 }
 
 static size_t
-_write_NOOP (void *data, size_t s, size_t n, void *ignored)
+_write_NOOP (char *d UNUSED, size_t s, size_t n, void *i UNUSED)
 {
-	(void) data, (void) ignored;
 	return s*n;
 }
 
 static size_t
-_write_GString (void *b, size_t s, size_t n, GString *out)
+_write_GString (char *b, size_t s, size_t n, GString *out)
 {
 	g_string_append_len (out, (gchar*)b, s*n);
 	return s*n;
@@ -200,7 +227,7 @@ struct view_GString_s
 };
 
 static size_t
-_read_GString (void *b, size_t s, size_t n, struct view_GString_s *in)
+_read_GString (char *b, size_t s, size_t n, struct view_GString_s *in)
 {
 	size_t remaining = in->data->len - in->done;
 	size_t available = s * n;
@@ -236,16 +263,14 @@ _has_prefix_len (char **pb, size_t *plen, const char *prefix)
 {
 	char *b = *pb;
 	size_t blen = *plen;
-	if (!b)
-		return FALSE;
-	while (blen && !g_ascii_isalnum(b[blen-1]))
-		blen --;
-	if (!blen)
+	if (!b || !blen)
 		return FALSE;
+
 	while (*prefix) {
-		if (!(blen--) || g_ascii_tolower(*(b++)) != *(prefix++))
+		if (!(blen--) || g_ascii_tolower(*(b++)) != g_ascii_tolower(*(prefix++)))
 			return FALSE;
 	}
+
 	*pb = b;
 	*plen = blen;
 	return TRUE;
@@ -256,7 +281,12 @@ _header_callback(char *b, size_t s, size_t n, void *u)
 {
 	struct http_ctx_s *o = u;
 	size_t total = n*s;
-	if (!o || !o->headers || !_has_prefix_len (&b, &total, "x-oio-"))
+
+	if (!o || !o->headers) /* caller not interested in headers */
+		return total;
+	if (!_has_prefix_len (&b, &total, "x-oio-"))
+		return total;
+	if (total > 8192) /* header too big */
 		return total;
 
 	gchar tmp[total+1];
@@ -281,9 +311,9 @@ static GError *
 _proxy_call_notime (CURL *h, const char *method, const char *url,
 		struct http_ctx_s *in, struct http_ctx_s *out)
 {
-	g_assert (h != NULL);
-	g_assert (method != NULL);
-	g_assert (url != NULL);
+	EXTRA_ASSERT (h != NULL);
+	EXTRA_ASSERT (method != NULL);
+	EXTRA_ASSERT (url != NULL);
 	struct view_GString_s view_input = {.data=NULL, .done=0};
 
 	GError *err = NULL;
@@ -311,7 +341,8 @@ _proxy_call_notime (CURL *h, const char *method, const char *url,
 	if (in && in->body) {
 		view_input.data = in->body;
 		gint64 len = in->body->len;
-		curl_easy_setopt (h, CURLOPT_READFUNCTION, _read_GString);
+		curl_easy_setopt (h, CURLOPT_READFUNCTION,
+				(curl_read_callback)_read_GString);
 		curl_easy_setopt (h, CURLOPT_READDATA, &view_input);
 		curl_easy_setopt (h, CURLOPT_UPLOAD, 1L);
 		curl_easy_setopt (h, CURLOPT_INFILESIZE_LARGE, len);
@@ -323,10 +354,12 @@ _proxy_call_notime (CURL *h, const char *method, const char *url,
 	}
 
 	if (out && out->body) {
-		curl_easy_setopt (h, CURLOPT_WRITEFUNCTION, _write_GString);
+		curl_easy_setopt (h, CURLOPT_WRITEFUNCTION,
+				(curl_write_callback)_write_GString);
 		curl_easy_setopt (h, CURLOPT_WRITEDATA, out->body);
 	} else {
-		curl_easy_setopt (h, CURLOPT_WRITEFUNCTION, _write_NOOP);
+		curl_easy_setopt (h, CURLOPT_WRITEFUNCTION,
+				(curl_write_callback)_write_NOOP);
 	}
 
 	CURLcode rc = curl_easy_perform (h);
@@ -338,7 +371,7 @@ _proxy_call_notime (CURL *h, const char *method, const char *url,
 		if (2 != (code/100)) {
 			if (out && out->body) {
 				err = _body_parse_error (out->body);
-				g_prefix_error (&err, "Request error (%ld): ", code);
+				g_prefix_error (&err, "Proxy error (%ld): ", code);
 			} else {
 				err = NEWERROR(code, "Request error (%ld)", code);
 			}
@@ -362,33 +395,18 @@ _proxy_call (CURL *h, const char *method, const char *url,
 	return err;
 }
 
-/* -------------------------------------------------------------------------- */
-
-GError *
-oio_proxy_call_container_create (CURL *h, struct oio_url_s *u)
-{
-	GString *http_url = _curl_container_url (u, "create");
-	gchar *hdrin[] = {PROXYD_HEADER_MODE, "autocreate", NULL};
-	struct http_ctx_s i = { .headers = hdrin, .body = NULL };
-	GError *err = _proxy_call (h, "POST", http_url->str, &i, NULL);
-	g_string_free(http_url, TRUE);
-	return err;
-}
-
-GError *
-oio_proxy_call_container_get_properties (CURL *h, struct oio_url_s *u,
+static GError *
+_get_properties(CURL *h, GString *http_url,
 		GString **props_str)
 {
-	GString *http_url = _curl_container_url (u, "get_properties");
 	struct http_ctx_s out_ctx = {0};
 	if (props_str && *props_str)
 		out_ctx.body = *props_str;
 	else
 		out_ctx.body = g_string_sized_new(512);
 
-	GError *err = _proxy_call (h, "POST", http_url->str, NULL, &out_ctx);
+	GError *err = _proxy_call(h, "POST", http_url->str, NULL, &out_ctx);
 
-	g_string_free(http_url, TRUE);
 	if (props_str && !*props_str && !err)
 		*props_str = out_ctx.body;
 	else
@@ -396,13 +414,114 @@ oio_proxy_call_container_get_properties (CURL *h, struct oio_url_s *u,
 	return err;
 }
 
+static GError *
+_set_properties(CURL *h, GString *http_url, const char* const *values)
+{
+	GError *err = NULL;
+	GString *json = g_string_new("{\"properties\":");
+	json = _build_json(values, json);
+	g_string_append_c(json, '}');
+	struct http_ctx_s i = {
+		.body = json
+	};
+	err = _proxy_call (h, "POST", http_url->str, &i, NULL);
+	g_string_free(i.body, TRUE);
+	return err;
+}
+
+/* -------------------------------------------------------------------------- */
+
 GError *
-oio_proxy_call_content_show (CURL *h, struct oio_url_s *u, GString *out)
+oio_proxy_call_container_create (CURL *h, struct oio_url_s *u)
+{
+	GString *http_url = _curl_container_url (u, "create");
+	if (!http_url) return BADNS();
+
+	gchar *hdrin[] = {PROXYD_HEADER_MODE, "autocreate", NULL};
+	GString *body = g_string_new("{}");
+	struct http_ctx_s i = { .headers = hdrin, .body = body };
+	GError *err = _proxy_call (h, "POST", http_url->str, &i, NULL);
+	g_string_free(body, TRUE);
+	g_string_free(http_url, TRUE);
+	return err;
+}
+
+GError *
+oio_proxy_call_container_delete (CURL *h, struct oio_url_s *u)
+{
+	GString *http_url = _curl_container_url (u, "destroy");
+	GError *err = _proxy_call (h, "POST", http_url->str, NULL, NULL);
+	g_string_free(http_url, TRUE);
+	return err;
+}
+
+GError *
+oio_proxy_call_container_get_properties(CURL *h, struct oio_url_s *u,
+		GString **props_str)
+{
+	GString *http_url = _curl_container_url(u, "get_properties");
+	if (!http_url)
+		return BADNS();
+
+	GError *err = _get_properties(h, http_url, props_str);
+
+	g_string_free(http_url, TRUE);
+	return err;
+}
+
+GError *
+oio_proxy_call_container_set_properties (CURL *h, struct oio_url_s *u,
+		const char* const *values)
+{
+	GString *http_url = _curl_container_url (u, "set_properties");
+	if (!http_url) return BADNS();
+
+	GError *err = _set_properties(h, http_url, values);
+	g_string_free (http_url, TRUE);
+	return err;
+}
+
+GError *
+oio_proxy_call_content_get_properties(CURL *h, struct oio_url_s *u,
+		GString **props_str)
+{
+	GString *http_url = _curl_content_url(u, "get_properties");
+	if (!http_url)
+		return BADNS();
+
+	GError *err = _get_properties(h, http_url, props_str);
+
+	g_string_free(http_url, TRUE);
+	return err;
+}
+
+GError *
+oio_proxy_call_content_set_properties (CURL *h, struct oio_url_s *u,
+		const char* const *values)
+{
+	GString *http_url = _curl_content_url (u, "set_properties");
+	if (!http_url) return BADNS();
+
+	GError *err = _set_properties(h, http_url, values);
+	g_string_free (http_url, TRUE);
+	return err;
+}
+
+GError *
+oio_proxy_call_content_show (CURL *h, struct oio_url_s *u, GString *out,
+		gchar ***hout)
 {
 	GString *http_url = _curl_content_url (u, "show");
-	struct http_ctx_s o = { .headers = NULL, .body = out };
+	if (!http_url) return BADNS();
+
+	struct http_ctx_s o = {
+			.headers = hout ? g_malloc0(sizeof(gchar*)) : NULL,
+			.body = out
+	};
 	GError *err = _proxy_call (h, "GET", http_url->str, NULL, &o);
 	g_string_free (http_url, TRUE);
+	if (hout)
+		*hout = o.headers;
 	return err;
 }
 
@@ -410,6 +529,8 @@ GError *
 oio_proxy_call_content_delete (CURL *h, struct oio_url_s *u)
 {
 	GString *http_url = _curl_content_url (u, "delete");
+	if (!http_url) return BADNS();
+
 	GError *err = _proxy_call (h, "POST", http_url->str, NULL, NULL);
 	g_string_free (http_url, TRUE);
 	return err;
@@ -418,19 +539,42 @@ oio_proxy_call_content_delete (CURL *h, struct oio_url_s *u)
 GError *
 oio_proxy_call_content_link (CURL *h, struct oio_url_s *u, const char *id)
 {
-	struct http_ctx_s i = { .headers = NULL, .body = _gs_vprintf("{\"id\":\"%s\"}", id) };
 	GString *http_url = _curl_content_url (u, "link");
+	if (!http_url) return BADNS();
+
+	struct http_ctx_s i = { .headers = NULL, .body = _gs_vprintf("{\"id\":\"%s\"}", id) };
 	GError *err = _proxy_call (h, "POST", http_url->str, &i, NULL);
 	g_string_free (http_url, TRUE);
 	g_string_free (i.body, TRUE);
 	return err;
 }
 
+GError *
+oio_proxy_call_content_truncate (CURL *h, struct oio_url_s *u, gint64 size)
+{
+	GString *http_url = _curl_content_url(u, "truncate");
+	if (!http_url)
+		return BADNS();
+
+	gchar size_str[16] = {0};
+	g_snprintf(size_str, sizeof(size_str), "%"G_GINT64_FORMAT, size);
+
+	_append(http_url, '&', "content", oio_url_get(u, OIOURL_CONTENTID));
+	_append(http_url, '&', "size", size_str);
+
+	GError *err = _proxy_call(h, "POST", http_url->str, NULL, NULL);
+	g_string_free(http_url, TRUE);
+	return err;
+}
+
 GError *
 oio_proxy_call_content_prepare (CURL *h, struct oio_url_s *u,
 		gsize size, gboolean autocreate,
 		struct oio_proxy_content_prepare_out_s *out)
 {
+	GString *http_url = _curl_content_url (u, "prepare");
+	if (!http_url) return BADNS();
+
 	gchar *hdrin[] = {
 		PROXYD_HEADER_MODE, autocreate ? "autocreate" : NULL,
 		NULL
@@ -444,7 +588,7 @@ oio_proxy_call_content_prepare (CURL *h, struct oio_url_s *u,
 		.headers = g_malloc0(sizeof(void*)),
 		.body = out ? out->body : NULL
 	};
-	GString *http_url = _curl_content_url (u, "prepare");
+
 	GError *err = _proxy_call (h, "POST", http_url->str, &i, &o);
 	if (!err && out && o.headers) {
 		for (gchar **p=o.headers; *p && *(p+1) ;p+=2) {
@@ -473,11 +617,18 @@ GError *
 oio_proxy_call_content_create (CURL *h, struct oio_url_s *u,
 		struct oio_proxy_content_create_in_s *in, GString *out)
 {
-	GString *http_url = _curl_content_url (u, "create");
+	GString *http_url = NULL;
+	if (in->update)
+		http_url = _curl_content_url (u, "update");
+	else
+		http_url = _curl_content_url (u, "create");
+	if (!http_url) return BADNS();
+
 	if (in->content) {
 		g_string_append (http_url, "&id=");
 		g_string_append_uri_escaped (http_url, in->content, NULL, TRUE);
 	}
+
 	gchar *hdrin[] = {
 		g_strdup(PROXYD_HEADER_PREFIX "content-meta-id"),
 		g_strdup_printf("%s", in->content),
@@ -488,28 +639,57 @@ oio_proxy_call_content_create (CURL *h, struct oio_url_s *u,
 		g_strdup(PROXYD_HEADER_PREFIX "content-meta-hash"),
 		g_strdup_printf("%s", in->hash),
 		g_strdup(PROXYD_HEADER_PREFIX "content-meta-policy"),
-		g_strdup_printf("%s", "NONE"),
+		g_strdup_printf("%s", in->stgpol?: "NONE"),
+		g_strdup(PROXYD_HEADER_PREFIX "content-meta-chunk-method"),
+		g_strdup_printf("%s", in->chunk_method?: "plain"),
+		NULL,
+		NULL,
 		NULL
 	};
-	struct http_ctx_s i = { .headers = hdrin, .body = in ? in->chunks : NULL };
+
+	if (in->append) {
+		gsize len = oio_ptrv_length(hdrin);
+		hdrin[len] = g_strdup(PROXYD_HEADER_MODE);
+		hdrin[len+1] = g_strdup("append");
+	}
+	GString *body = in? in->chunks : NULL;
+	if (in && !in->update && in->properties) {
+		GString *val = g_string_new("{'properties':");
+		val = _build_json(in->properties, val);
+		g_string_append(val, ",'chunks':");
+		g_string_append(val, in->chunks->str);
+		g_string_append_c(val, '}');
+		body = val;
+	}
+	struct http_ctx_s i = { .headers = hdrin, .body = body };
 	struct http_ctx_s o = { .headers = NULL, .body = out };
 	GError *err = _proxy_call (h, "POST", http_url->str, &i, &o);
 	_ptrv_free_content (i.headers);
+	if (in && !in->update && in->properties)
+		g_string_free (body, TRUE);
 	g_string_free (http_url, TRUE);
 	return err;
 }
 
 GError *
-oio_proxy_call_content_list (CURL *h, struct oio_url_s *u, GString *out,
-		const char *prefix, const char *marker, const char *end,
-		guint max, char delim)
+oio_proxy_call_content_list(CURL *h, struct oio_sds_list_param_s *params,
+		GString *out)
 {
-	GString *http_url = _curl_container_url (u, "list");
-	if (prefix) _append (http_url, '&', "prefix", prefix);
-	if (marker) _append (http_url, '&', "marker", marker);
-	if (end) _append (http_url, '&', "end", end);
-	if (max) g_string_append_printf (http_url, "&max=%u", max);
-	if (delim) g_string_append_printf (http_url, "&delimiter=%c", delim);
+	GString *http_url = _curl_container_url(params->url, "list");
+	if (!http_url) return BADNS();
+
+	if (params->prefix)
+		_append(http_url, '&', "prefix", params->prefix);
+	if (params->marker)
+		_append(http_url, '&', "marker", params->marker);
+	if (params->end)
+		_append(http_url, '&', "end", params->end);
+	if (params->max_items)
+		g_string_append_printf(http_url, "&max=%zu", params->max_items);
+	if (params->delimiter)
+		g_string_append_printf(http_url, "&delimiter=%c", params->delimiter);
+	if (params->flag_properties)
+		g_string_append_printf(http_url, "&properties=1");
 
 	struct http_ctx_s o = { .headers = NULL, .body = out };
 	GError *err = _proxy_call (h, "GET", http_url->str, NULL, &o);
@@ -524,6 +704,8 @@ oio_proxy_call_reference_show (CURL *h, struct oio_url_s *u,
 		const char *t, GString *out)
 {
 	GString *http_url = _curl_reference_url (u, "show");
+	if (!http_url) return BADNS();
+
 	if (t) _append(http_url, '&', "type", t);
 
 	struct http_ctx_s o = { .headers = NULL, .body = out };
@@ -538,6 +720,8 @@ GError *
 oio_proxy_call_reference_create (CURL *h, struct oio_url_s *u)
 {
 	GString *http_url = _curl_reference_url (u, "create");
+	if (!http_url) return BADNS();
+
 	GError *err = _proxy_call (h, "POST", http_url->str, NULL, NULL);
 	g_string_free(http_url, TRUE);
 	return err;
@@ -548,6 +732,8 @@ oio_proxy_call_reference_link (CURL *h, struct oio_url_s *u,
 		const char *srvtype, gboolean autocreate, GString *out)
 {
 	GString *http_url = _curl_reference_url (u, "link");
+	if (!http_url) return BADNS();
+
 	_append (http_url, '&', "type", srvtype);
 	gchar *hdrin[] = {
 		g_strdup(PROXYD_HEADER_MODE),
@@ -565,12 +751,42 @@ oio_proxy_call_reference_link (CURL *h, struct oio_url_s *u,
 	return err;
 }
 
+GError *
+oio_proxy_call_reference_get_properties(CURL *h, struct oio_url_s *u,
+		GString **props_str)
+{
+	GString *http_url = _curl_reference_url(u, "get_properties");
+	if (!http_url)
+		return BADNS();
+
+	GError *err = _get_properties(h, http_url, props_str);
+
+	g_string_free(http_url, TRUE);
+	return err;
+}
+
+GError *
+oio_proxy_call_reference_set_properties(CURL *h, struct oio_url_s *u,
+		const char* const *values)
+{
+	GString *http_url = _curl_reference_url(u, "set_properties");
+	if (!http_url)
+		return BADNS();
+
+	GError *err = _set_properties(h, http_url, values);
+
+	g_string_free (http_url, TRUE);
+	return err;
+}
+
 /* -------------------------------------------------------------------------- */
 
 GError *
 oio_proxy_call_conscience_register (CURL *h, const char *ns, GString *in)
 {
 	GString *http_url = _curl_conscience_url (ns, "register");
+	if (!http_url) return BADNS();
+
 	struct http_ctx_s i = { .headers = NULL, .body = in };
 	GError *err = _proxy_call (h, "POST", http_url->str, &i, NULL);
 	g_string_free(http_url, TRUE);
@@ -581,6 +797,8 @@ GError *
 oio_proxy_call_conscience_lock (CURL *h, const char *ns, GString *in)
 {
 	GString *http_url = _curl_conscience_url (ns, "lock");
+	if (!http_url) return BADNS();
+
 	struct http_ctx_s i = { .headers = NULL, .body = in };
 	GError *err = _proxy_call (h, "POST", http_url->str, &i, NULL);
 	g_string_free(http_url, TRUE);
@@ -591,6 +809,8 @@ GError *
 oio_proxy_call_conscience_deregister (CURL *h, const char *ns, GString *in)
 {
 	GString *http_url = _curl_conscience_url (ns, "deregister");
+	if (!http_url) return BADNS();
+
 	struct http_ctx_s i = { .headers = NULL, .body = in };
 	GError *err = _proxy_call (h, "POST", http_url->str, &i, NULL);
 	g_string_free(http_url, TRUE);
@@ -601,6 +821,8 @@ GError *
 oio_proxy_call_conscience_flush (CURL *h, const char *ns, const char *srvtype)
 {
 	GString *http_url = _curl_conscience_url (ns, "flush");
+	if (!http_url) return BADNS();
+
 	_append (http_url, '?', "type", srvtype);
 	GError *err = _proxy_call (h, "POST", http_url->str, NULL, NULL);
 	g_string_free(http_url, TRUE);
@@ -611,6 +833,8 @@ GError *
 oio_proxy_call_conscience_unlock (CURL *h, const char *ns, GString *in)
 {
 	GString *http_url = _curl_conscience_url (ns, "unlock");
+	if (!http_url) return BADNS();
+
 	struct http_ctx_s i = { .headers = NULL, .body = in };
 	GError *err = _proxy_call (h, "POST", http_url->str, &i, NULL);
 	g_string_free(http_url, TRUE);
@@ -622,6 +846,8 @@ oio_proxy_call_conscience_list (CURL *h, const char *ns,
 		const char *srvtype, gboolean full, GString *out)
 {
 	GString *http_url = _curl_conscience_url (ns, "list");
+	if (!http_url) return BADNS();
+
 	_append (http_url, '?', "type", srvtype);
 	gchar *hdrin[] = {
 		g_strdup(PROXYD_HEADER_MODE),
@@ -644,6 +870,8 @@ oio_proxy_call_conscience_list_types (CURL *h, const char *ns,
 		GString *out)
 {
 	GString *http_url = _curl_conscience_url (ns, "info");
+	if (!http_url) return BADNS();
+
 	_append (http_url, '?', "what", "types");
 	struct http_ctx_s o = { .headers = NULL, .body = out };
 	GError *err = _proxy_call (h, "GET", http_url->str, NULL, &o);
@@ -656,6 +884,8 @@ GError *
 oio_proxy_call_conscience_info (CURL *h, const char *ns, GString *out)
 {
 	GString *http_url = _curl_conscience_url (ns, "info");
+	if (!http_url) return BADNS();
+
 	struct http_ctx_s o = { .headers = NULL, .body = out };
 	GError *err = _proxy_call (h, "GET", http_url->str, NULL, &o);
 	g_strfreev (o.headers);
