@@ -28,15 +28,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <strings.h>
 #include <stdlib.h>
-#include <sys/socket.h>
-#include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/time.h>
 #include <unistd.h>
-#include <arpa/inet.h>
-#include <netdb.h>
-#include <netinet/in.h>
 #include <signal.h>
+#include <malloc.h>
 
 #include <metautils/lib/metautils.h>
 #include <metautils/lib/metacomm.h>
@@ -76,16 +72,11 @@ guint32 gridd_flags = GRIDD_FLAG_NOLINGER | GRIDD_FLAG_SHUTDOWN
 
 /* NEW WAY INFORMATIONS ----------------------------------------------------- */
 
-gboolean old_style = FALSE;
-gchar* service_type = NULL;
-gboolean rec_service = FALSE;
-gboolean load_ns_info = FALSE;
-namespace_info_t *ns_info = NULL;
-addr_info_t *serv_addr = NULL;
-GPtrArray *serv_tags = NULL;
 gboolean first = TRUE;
 gchar* ns_name = NULL;
 
+static GAsyncQueue *threads_to_be_joined = NULL;
+
 /* -------------------------------------------------------------------------- */
 
 volatile gboolean may_continue = TRUE;
@@ -122,7 +113,7 @@ GET_FLAG(GKeyFile *gkf, const gchar *section, const gchar *k, gboolean def)
 	if (!g_key_file_has_key(gkf, section, buf, NULL))
 		return def;
 	gchar *v = g_key_file_get_value(gkf, section, buf, NULL);
-	gboolean flag = metautils_cfg_get_bool(v, def);
+	gboolean flag = oio_str_parse_bool(v, def);
 	g_free(v);
 	return flag;
 }
@@ -159,75 +150,25 @@ srv_inner_gauges_update (gpointer d)
 	srvstat_set_u64("gauge cnx.client", tmpTotalStats.total);
 }
 
-static gboolean 
-self_register_in_cluster(GError **err)
-{
-	/* Init the service header */
-	service_info_t *si = g_malloc0(sizeof(service_info_t));
-	memcpy(&(si->addr), serv_addr, sizeof(addr_info_t));
-	g_strlcpy(si->ns_name, ns_info->name, sizeof(si->ns_name));
-	g_strlcpy(si->type, service_type, sizeof(si->type));
-	si->tags = g_ptr_array_new();
-
-	if (first) 
-		service_tag_set_value_boolean(service_info_ensure_tag(si->tags, NAME_TAGNAME_RAWX_FIRST), first);
-
-	/* Copy the service tags */
-	if (!serv_tags)
-		DEBUG("No tag found in gridd to set in service info");
-	else {
-		gsize i;
-		for (i=0; i<serv_tags->len ;i++) {
-			struct service_tag_s *tag = g_ptr_array_index(serv_tags, i);
-			g_ptr_array_add(si->tags, service_tag_dup(tag));
-		}
-	}
-
-	/* Register nw in the conscience */
-	GError *e = register_namespace_service(si);
-	if (NULL != e) {
-		ERROR("Failed to register service in cluster : %s", gerror_get_message(*err));
-		g_clear_error(&e);
-	}
-
-	service_info_clean(si);
-	first = FALSE;
-	return TRUE;
-}
-
 static void
-srv_periodic_register (gpointer d)
+srv_periodic_joiner (gpointer d UNUSED)
 {
-	(void)d;
-	GError *err = NULL;
-	if(!self_register_in_cluster(&err))
-		NOTICE("Failed to register service in cluster : %s", gerror_get_message(err));
-	if(err)
-		g_clear_error(&err);
-}
+	g_assert(threads_to_be_joined != NULL);
 
-static void
-srv_periodic_refresh_ns_info (gpointer d)
-{
-	/* free current ns_info and load newer */
-	namespace_info_t *nsinfo = NULL;
-	namespace_info_t *tmp = NULL;
-	GError *error = NULL;
+	guint nb = 0;
+	GThread *th;
+	while (NULL != (th = g_async_queue_try_pop(threads_to_be_joined))) {
+		g_thread_join(th);
+		nb++;
+	}
 
-	(void) d;
+	static gint64 last = 0;
+	gint64 now = oio_ext_monotonic_time();
 
-	if (!(error = conscience_get_namespace(ns_name, &ns_info))) {
-		NOTICE("Failed to refresh the Namespace info from the gridagent");
-		if(error)
-			g_clear_error(&error);
-		return;
+	if (now > last + G_TIME_SPAN_MINUTE) {
+		last = now;
+		malloc_trim(0);
 	}
-	g_rw_lock_writer_lock (&ns_info_lock);
-	tmp = ns_info;
-	ns_info = nsinfo;
-	namespace_info_free(tmp);
-	g_rw_lock_writer_unlock(&ns_info_lock);
-	nsinfo = NULL;
 }
 
 static gpointer main_thread (gpointer arg);
@@ -238,11 +179,11 @@ static void
 server_notify_alert( struct server_s *srv )
 {
 	register time_t now;
-	
+
 	if (!srv) return;
 
 	now = time(0);
-	
+
 	g_rec_mutex_lock (&(srv->recMutex));
 	srv->alert_cfg.last_sent = now;
 	g_rec_mutex_unlock (&(srv->recMutex));
@@ -252,7 +193,7 @@ static gboolean
 server_alert_possible( struct server_s *srv )
 {
 	register time_t last, now, freq;
-	
+
 	if (!srv)
 		return FALSE;
 
@@ -262,7 +203,7 @@ server_alert_possible( struct server_s *srv )
 	g_rec_mutex_unlock (&(srv->recMutex));
 
 	now = time(0);
-	
+
 	return (last<=0 || now-last>freq);
 }
 
@@ -369,12 +310,12 @@ thread_monitoring_release (struct server_s *srv)
 {
 	gboolean rc = TRUE;
 	gchar *str_dbg = NULL;
-	
+
 	/* XXX start of locked section */
 	g_rec_mutex_lock (&(srv->recMutex));
-	
+
 	srv->mon.used_workers --;
-	
+
 	gint spare_workers = srv->mon.nb_workers - srv->mon.used_workers;
 	gboolean too_few_workers = srv->mon.nb_workers < srv->mon.min_workers;
 	gboolean too_many_workers = srv->mon.nb_workers > srv->mon.max_workers;
@@ -390,7 +331,7 @@ thread_monitoring_release (struct server_s *srv)
 			}
 		}
 	}
-	
+
 	g_rec_mutex_unlock (&(srv->recMutex));
 	/* XXX end of locked section */
 
@@ -408,7 +349,7 @@ thread_monitoring_periodic_debug (struct server_s *srv)
 	struct thread_monitoring_s mon, mon0;
 	gchar str_pool[512];
 	gsize str_pool_size;
-	
+
 	/* XXX locked section */
 	g_rec_mutex_lock (&(srv->recMutex));
 	memcpy(&mon, &(srv->mon), sizeof(struct thread_monitoring_s));
@@ -566,7 +507,7 @@ main_thread (gpointer arg)
 
 	/*init variables*/
 	if (!(srv = arg))
-		return NULL;
+		goto exit;
 
 	/*about to block unwanted signals in the worker threads*/
 	metautils_ignore_signals();
@@ -656,10 +597,12 @@ main_thread (gpointer arg)
 		STATS_CNX_DOWN();
 
 		if (!thread_monitoring_release(srv)) /* in excess */
-			return 0;
+			goto exit;
 	}
 
 	thread_monitoring_remove(srv,FALSE);
+exit:
+	g_async_queue_push(threads_to_be_joined, g_thread_self());
 	return 0;
 }
 
@@ -669,6 +612,7 @@ extract_parameters (GKeyFile *kf, const char *s, const char *p, GError **err)
 	gchar **all_keys=NULL, **current_key=NULL;
 	gsize size=0;
 	GHashTable *ht=NULL;
+	size_t pref_len = p? strlen(p) : 0;
 
 	ht = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
@@ -677,9 +621,9 @@ extract_parameters (GKeyFile *kf, const char *s, const char *p, GError **err)
 	{
 		GSETERROR(err, "cannot get the keys of the section '%s'", s);
 	}
-	for (current_key=all_keys; current_key && *current_key ;current_key++)
+	for (current_key = all_keys; all_keys && *current_key; current_key++)
 	{
-		if (!g_ascii_strncasecmp(*current_key,p,strlen(p)))
+		if (g_str_has_prefix(*current_key, p))
 		{
 			gchar *value = NULL;
 			value = g_key_file_get_value (kf, s, *current_key, err);
@@ -688,7 +632,7 @@ extract_parameters (GKeyFile *kf, const char *s, const char *p, GError **err)
 				GSETERROR (err, "Cannot get the value");
 				goto error;
 			}
-			g_hash_table_insert (ht, g_strdup(*current_key + strlen(p)), value);
+			g_hash_table_insert(ht, g_strdup(*current_key + pref_len), value);
 		}
 	}
 
@@ -702,106 +646,6 @@ error:
 	return NULL;
 }
 
-#if 0
-static int
-prepare_plugins_reload (GKeyFile *cfgFile, GError **err)
-{
-	GHashTable *params=NULL;
-	gchar **groups=NULL;
-	gsize nbgroups=0, i;
-	gchar *fileName=NULL;
-
-	if (!cfgFile)
-	{
-		GSETERROR(err,"Invalid Parameter");
-		goto errorLabel;
-	}
-
-	DEBUG ("Start loading all the plugins found in the configuration");
-	
-	/*run the key's list and keep those mathing Plugin~*/
-	groups = g_key_file_get_groups (cfgFile, &nbgroups);
-	if (!groups || nbgroups<=0)
-	{
-		GSETERROR(err,"Cannot retrieve the list of the configuration groups");
-		goto errorLabel;
-	}
-
-	for (i=0; i<nbgroups ;i++)
-	{
-		gchar *group;
-		group = groups[i];
-
-		if (!group)
-		{
-			GSETERROR (err, "Invalid group name");
-			goto errorLabel;
-		}
-
-		if (0 == fnmatch("Plugin.*", group, 0))
-		{
-			GModule *mod = NULL;
-
-			/*get the filename of the plugin*/
-			if (    !g_key_file_has_key (cfgFile, group, NAME_PATH, err)
-				||	!(fileName = g_key_file_get_string (cfgFile, group, NAME_PATH, err)))
-			{
-				GSETERROR(err, "The group %s must contain the path of the plugin in the key %s", group, NAME_PATH);
-				goto errorLabel;
-			}
-
-			/*get the parameter string*/
-			params = extract_parameters (cfgFile, group, NAME_PARAM"_", err);
-			if (!params)
-			{
-				GSETERROR(err,"Cannot load the parameters hash for the plugin (group %s in the configuration file)", group);
-				goto errorLabel;
-			}
-
-			/*load the path*/
-			mod = g_module_open (fileName, 0);
-			if (!mod)
-			{
-				GSETERROR(err, "Cannot load the plug-in from file %s (%s)", fileName, g_module_error());
-				goto errorLabel;
-			}
-
-			/*load the main exported symbol*/
-			if (plugin_holder_update_config (mod, params, err))
-				DEBUG ("updated %s", fileName);
-			else {
-				GSETERROR(err, "cannot update %s", fileName);
-				goto errorLabel;
-			}
-			
-			if (fileName) {
-				g_free(fileName);
-				fileName=NULL;
-			}
-			
-			/*the hash table has not been copied, just its pointers*/
-			params=NULL;
-		}
-	}
-
-	g_strfreev(groups);
-
-	return 1;
-
-errorLabel:
-
-	if (groups)
-		g_strfreev (groups);
-
-	if (fileName)
-		g_free (fileName);
-	
-	if (params)
-		g_hash_table_destroy (params);
-	return 0;
-}
-#endif
-
 static int
 preload_plugins (GKeyFile *cfgFile, GError **err)
 {
@@ -817,7 +661,7 @@ preload_plugins (GKeyFile *cfgFile, GError **err)
 	}
 
 	DEBUG ("Start loading all the plugins found in the configuration");
-	
+
 	/*run the key's list and keep those mathing Plugin~*/
 	groups = g_key_file_get_groups (cfgFile, &nbgroups);
 	if (!groups || nbgroups<=0)
@@ -875,12 +719,12 @@ preload_plugins (GKeyFile *cfgFile, GError **err)
 				GSETERROR(err, "cannot load %s", fileName);
 				goto errorLabel;
 			}
-			
+
 			if (fileName) {
 				g_free(fileName);
 				fileName=NULL;
 			}
-			
+
 			/*the hash table has not been copied, just its pointers*/
 			params=NULL;
 		}
@@ -897,13 +741,13 @@ errorLabel:
 
 	if (fileName)
 		g_free (fileName);
-	
+
 	if (params)
 		g_hash_table_destroy (params);
 	return 0;
 }
 
-static void 
+static void
 set_srv_addr(const gchar* url)
 {
 	gchar str_addr[STRLEN_ADDRINFO];
@@ -920,14 +764,6 @@ set_srv_addr(const gchar* url)
 
 	grid_addrinfo_to_string(newaddr, str_addr, sizeof(str_addr));
 	NOTICE("Saving [%s] as main server address", str_addr);
-
-	if (serv_addr != NULL) {
-		grid_addrinfo_to_string(serv_addr, str_addr, sizeof(str_addr));
-		WARN("Disarding anoter server address [%s], this won't be registered in the conscience", str_addr);
-		addr_info_gclean(serv_addr, NULL);
-	}
-
-	serv_addr = newaddr;
 }
 
 static int
@@ -944,7 +780,7 @@ load_servers (GKeyFile *cfgFile, GError **err)
 	}
 
 	DEBUG ("Start loading all the servers found in the configuration");
-	
+
 	/*run the key's list and keep those mathing Plugin~*/
 	groups = g_key_file_get_groups (cfgFile, &nbgroups);
 	if (!groups || nbgroups<=0)
@@ -986,7 +822,7 @@ load_servers (GKeyFile *cfgFile, GError **err)
 				goto errorLabel;
 			}
 
-			/*prepare a new server structure, unshift it in the list*/	
+			/* prepare a new server structure, unshift it in the list */
 			if (!(srv = g_try_malloc0(sizeof(struct server_s))))
 			{
 				GSETERROR(err, "Memory allocation error");
@@ -1037,9 +873,8 @@ load_servers (GKeyFile *cfgFile, GError **err)
 				goto errorLabel;
 			}
 
-			for (url=srvList; url && *url && *(*url); url++)
-			{
-				set_srv_addr(*url);		
+			for (url = srvList; srvList && *url && *(*url); url++) {
+				set_srv_addr(*url);
 				if (!accept_add(srv->ap, *url, err))
 					goto errorLabel;
 			}
@@ -1112,10 +947,10 @@ errorLabel:
 
 	if (srvList)
 		g_strfreev(srvList);
-		
+
 	if (pluginList)
 		g_strfreev(pluginList);
-		
+
 	return 0;
 }
 
@@ -1162,72 +997,6 @@ load_defaults (GKeyFile *cfgFile, GError **err)
 	return 1;
 }
 
-static void
-load_service_tags(GKeyFile *cfgFile, GError **err)
-{
-	gchar **tags, **cur_tag;
-	gchar *tag_name, *tag_value;
-	gsize number_of_tags;
-	struct service_tag_s tag_s;
-
-	if (NULL == serv_tags)
-		serv_tags = g_ptr_array_new();
-
-	if (g_key_file_has_group (cfgFile, NAME_SERVICETAGS)) {
-		tags = g_key_file_get_keys(cfgFile, NAME_SERVICETAGS, &number_of_tags, err);
-		for (cur_tag = tags; *cur_tag; cur_tag++) {
-			tag_name = g_strconcat("tag.", *cur_tag, NULL);
-			tag_value = g_key_file_get_value(cfgFile, NAME_SERVICETAGS, *cur_tag, err);
-			g_strlcpy(tag_s.name, tag_name, sizeof(tag_s.name));
-			service_tag_set_value_string(&tag_s, tag_value);
-			g_ptr_array_add(serv_tags, service_tag_dup(&tag_s));
-			g_free(tag_name);
-		}
-		g_strfreev(tags);
-	}
-}
-
-/**
- * @warning This function will block until gridagent is available
- */
-static int
-load_service_info (GKeyFile *cfgFile, GError **err)
-{
-	if (!g_key_file_has_group (cfgFile, NAME_SERVICE)) {
-		DEBUG("No '%s' group in configuration, Old style service declaration", NAME_SERVICE);
-		old_style = TRUE;
-		return 1;
-	}
-
-	
-	ns_name = g_key_file_get_string (cfgFile, NAME_SERVICE, NAME_NAMESPACE, err);
-	
-	service_type = g_key_file_get_string (cfgFile, NAME_SERVICE, NAME_SRV_TYPE, err);
-	/* TODO: service type ok */
-
-	rec_service = g_key_file_get_boolean (cfgFile, NAME_SERVICE, NAME_REGISTER, err);
-	if (!rec_service)
-		load_ns_info = g_key_file_get_boolean (cfgFile, NAME_SERVICE, NAME_LOAD_NS_INFO, err);
-	else
-		load_ns_info = TRUE;
-
-	if(load_ns_info) {
-		GError *e = conscience_get_namespace (ns_name, &ns_info);
-		/* We really want these informations, so loop until we get them. */
-		while (e) {
-			g_clear_error(&e);
-			WARN("Failed to get namespace info (Retrying in %d seconds): %s",
-					GET_NS_INFO_RETRY_DELAY, (*err)->message);
-			sleep(GET_NS_INFO_RETRY_DELAY);
-			e = conscience_get_namespace (ns_name, &ns_info);
-		}
-	}
-
-	load_service_tags(cfgFile, err);
-
-	return 1;
-}
-
 static int
 load_configuration (const char *cfg_path, GError **err)
 {
@@ -1250,12 +1019,6 @@ load_configuration (const char *cfg_path, GError **err)
 		goto errorLabel;
 	}
 
-	/* load info about service we are and our namespace */
-	if (!load_service_info(cfgFile,err)) {
-		GSETERROR (err, "Failed to get all informations about the service");
-		goto errorLabel;
-	}
-
 	/*preloads the plugins*/
 	if (!preload_plugins (cfgFile,err)) {
 		GSETERROR (err, "Cannot preload the plugins from the configuration");
@@ -1267,17 +1030,17 @@ load_configuration (const char *cfg_path, GError **err)
 		GSETERROR (err, "an error occured during the plugins initialization");
 		goto errorLabel;
 	}
-	
+
 	INFO ("Plug-in's loaded!");
 
 	/*then load the servers from the configuration file*/
 	if (!load_servers (cfgFile, err)) {
-		GSETERROR (err, "Cannot servers from the configuration");
+		GSETERROR(err, "Cannot load servers");
 		goto errorLabel;
 	}
 
 	INFO ("Server threads loaded!");
-	
+
 	g_key_file_free (cfgFile);
 	return 1;
 errorLabel:
@@ -1300,23 +1063,15 @@ server_has_thread(struct server_s *srv)
 static void
 wait_for_workers(void)
 {
-	struct server_s *srv;
-	gboolean rc;
-	gint64 i64;
-
-	do {
-		rc = FALSE;
-		for (srv=BEACON_SRV.next; !rc && srv ;srv=srv->next) {
-			if (0 < (i64 = server_has_thread(srv))) {
-				rc = TRUE;
-				INFO("Waiting for workers : still %"G_GINT64_FORMAT, i64);
-			}
+retry:
+	for (struct server_s *srv=BEACON_SRV.next ; srv ; srv=srv->next) {
+		gint64 i64;
+		if (0 < (i64 = server_has_thread(srv))) {
+			INFO("Waiting for workers : still %"G_GINT64_FORMAT, i64);
+			g_usleep(G_TIME_SPAN_SECOND);
+			goto retry;
 		}
-		if (rc)
-			sleep(1);
-	} while (rc);
-
-	NOTICE("No more workers detected");
+	}
 }
 
 static gboolean
@@ -1369,61 +1124,6 @@ start_threads (void)
 
 /* ------------------------------------------------------------------------- */
 
-#if 0
-static int
-reload_config(void)
-{
-	GKeyFile *cfgFile = NULL;
-	cfgFile = g_key_file_new ();
-	int status = 0;
-	GError *err= NULL;
-
-	/*Parse the configuration*/
-	g_key_file_set_list_separator (cfgFile, ',');
-
-	if (!g_key_file_load_from_file (cfgFile, config_file, G_KEY_FILE_NONE, &err)) {
-		if(err && err->message)
-			ERROR("Cannot parse the configuration : %s",err->message);
-		else
-			ERROR("Cannot parse the configuration : no error");
-		goto errorLabel;
-	}
-
-	/*loads the default values*/
-	if (!reload_defaults (cfgFile, &err)) {
-		if(err && err->message)
-			ERROR("Cannot set the default value from configuration: %s",err->message);
-		else
-			ERROR("Cannot set the default value from configuration: no error");
-		goto errorLabel;
-	}
-
-	if(!prepare_plugins_reload(cfgFile, &err)){
-		if(err && err->message)
-			ERROR("Cannot set the default value from configuration: %s",err->message);
-		else
-			ERROR("Cannot set the default value from configuration: no error");
-		goto errorLabel;
-	}
-
-	/*init all the loaded plugins*/
-	if (!plugin_holder_reload_all(&err)) {
-		if(err && err->message)
-			ERROR("Cannot update plugins configuration: %s",err->message);
-		else
-			ERROR("Cannot update plugins configuration: no error");
-		goto errorLabel;
-	}
-
-	status = 1;
-
-errorLabel:
-
-	g_key_file_free (cfgFile);
-	return status;
-}
-#endif
-
 static void
 main_action (void)
 {
@@ -1436,23 +1136,9 @@ main_action (void)
 		return;
 	}
 
-	/* Register service if needed */
-	if (rec_service) {
-		GError *err = NULL;
-		if (!self_register_in_cluster(&err)) {
-			ERROR("Failed to register service in cluster : (%d) %s", err->code, err->message);
-			g_clear_error(&err);
-		}
-		/* Periodic register in cluster */
-		srvtimer_register_regular(service_type, srv_periodic_register, NULL, NULL, PERIOD_REGISTER); 
-	}	
-	if(load_ns_info)
-		srvtimer_register_regular(service_type, srv_periodic_refresh_ns_info, NULL, NULL, PERIOD_REFRESH_NS_INFO);
-	/*Threads started, we start monitoring them*/
+	srvtimer_register_regular("joiner", srv_periodic_joiner, NULL, NULL, 1LLU);
 
-	struct server_s *s = NULL;
-
-	for (s=BEACON_SRV.next; s && s!=&BEACON_SRV ;s=s->next) {
+	for (struct server_s *s=BEACON_SRV.next; s && s!=&BEACON_SRV ;s=s->next) {
 		srvtimer_register_regular(s->name, (srvtimer_f)thread_monitoring_periodic_debug, NULL, s, PERIOD_DEBUG);
 		srvtimer_register_regular(s->name, (srvtimer_f)thread_monitoring_periodic_stats, NULL, s, PERIOD_STATS);
 	}
@@ -1486,14 +1172,14 @@ main_configure(int argc, char **argv)
 		ERROR("Missing argument\n");
 		return FALSE;
 	}
-	
-	/*loads the configuration*/	
+
 	GError *err = NULL;
 	if (!load_configuration (config_file, &err)) {
 		ERROR("Failed to load the configuration : (%d) %s", err->code, err->message);
 		return FALSE;
 	}
 
+	threads_to_be_joined = g_async_queue_new();
 	return TRUE;
 }
 
@@ -1501,7 +1187,13 @@ static void
 main_specific_fini (void)
 {
 	may_continue = FALSE;
+
 	wait_for_workers();
+	if (threads_to_be_joined) {
+		srv_periodic_joiner(NULL);  // final call
+		g_async_queue_unref(threads_to_be_joined);
+		threads_to_be_joined = NULL;
+	}
 
 	DEBUG("Closing the servers");
 	for (SERVER srv=BEACON_SRV.next; srv ;srv=srv->next) {
@@ -1533,7 +1225,6 @@ main_specific_fini (void)
 			s->handlers = NULL;
 		}
 		g_rec_mutex_clear(&(s->recMutex));
-		
 	}
 
 	DEBUG("Cleaning the servers");
@@ -1595,7 +1286,7 @@ main (int argc, char **argv)
 		g_error("GLib MODULES are not supported on this platform!");
 		return 1;
 	}
-	
+
 	return grid_main(argc, argv, &callbacks);
 }
 
