@@ -22,6 +22,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <errno.h>
 
+#include <json-c/json.h>
+
 #include <metautils/lib/metacomm.h>
 #include <sqliterepo/sqliterepo.h>
 #include <sqliterepo/version.h>
@@ -65,29 +67,6 @@ m0_to_sqlx(enum m0v2_open_type_e t)
 	return SQLX_OPEN_LOCAL;
 }
 
-static GError*
-_array_check(GPtrArray *gpa)
-{
-	guint16 prefix;
-	guint i;
-	gchar **v;
-
-	if (gpa->len != 65536)
-		return NEWERROR(EINVAL, "Invalid cache size");
-
-	for (i=0; i<gpa->len ;i++) {
-		prefix = i;
-		if (!(v = gpa->pdata[i])) {
-			/* cache not loaded */
-			continue;
-		}
-		if (!*v)
-			return NEWERROR(EINVAL, "Prefix not managed (2) : %04X", prefix);
-	}
-
-	return NULL;
-}
-
 struct meta0_backend_s *
 meta0_backend_init(const gchar *ns, const gchar *id,
 		struct sqlx_repository_s *repo)
@@ -119,22 +98,6 @@ meta0_backend_clean(struct meta0_backend_s *m0)
 	g_free(m0);
 }
 
-GError*
-meta0_backend_check(struct meta0_backend_s *m0)
-{
-	EXTRA_ASSERT(m0 != NULL);
-	EXTRA_ASSERT(m0->array_by_prefix != NULL);
-
-	return _array_check(m0->array_by_prefix);
-}
-
-struct sqlx_repository_s*
-meta0_backend_get_repository(struct meta0_backend_s *m0)
-{
-	EXTRA_ASSERT(m0 != NULL);
-	return m0->repository;
-}
-
 void
 meta0_backend_reload_requested(struct meta0_backend_s *m0)
 {
@@ -144,69 +107,6 @@ meta0_backend_reload_requested(struct meta0_backend_s *m0)
 
 /* ------------------------------------------------------------------------- */
 
-void
-meta0_backend_migrate(struct meta0_backend_s *m0)
-{
-	GError *err = NULL;
-	struct sqlx_sqlite3_s *handle=NULL;
-	struct sqlx_sqlite3_s *oldhandle=NULL;
-
-	struct sqlx_name_s n;
-	n.base = m0->ns;
-	n.type = NAME_SRVTYPE_META0;
-	n.ns = m0->ns;
-	err = sqlx_repository_open_and_lock(m0->repository, &n, SQLX_OPEN_LOCAL, &handle, NULL);
-	if ( err )
-	{
-		// check if the erreur message is the ENOENT message ; DB doesn't exist
-		if  (  (strstr(err->message, strerror(ENOENT)) != NULL) ) {
-			g_clear_error(&err);
-			err = NULL;
-			err = sqlx_repository_open_and_lock(m0->repository, &n, SQLX_OPEN_LOCAL|SQLX_OPEN_CREATE,&handle, NULL);
-			if( !err ) {
-				// Migration (1.7 -> 1.8) from old meta0 database
-
-				n.base = m0->id;
-				err = sqlx_repository_open_and_lock(m0->repository, &n, SQLX_OPEN_LOCAL, &oldhandle,NULL);
-				if ( ! err ) {
-					GRID_INFO("Start Migrate meta0 database");
-					err = sqlx_repository_backup_base(oldhandle,handle);
-					if ( err ) {
-						GRID_ERROR("Failed to migrate meta0 database : (%d) %s",err->code,err->message);
-					} else {
-						sqlx_repository_unlock_and_close_noerror(oldhandle);
-						// APPLY schema
-						gint rc;
-						char *errmsg = NULL;
-						rc = sqlite3_exec(handle->db, META0_SCHEMA, NULL, NULL, &errmsg);
-						if (rc != SQLITE_OK && rc != SQLITE_DONE)  {
-							GRID_WARN("Failed to apply schema (%d) %s %s",rc,sqlite3_errmsg(handle->db), errmsg);
-						}
-						if(errmsg != NULL)
-							g_free(errmsg);
-
-						// set table version
-						GRID_INFO("Update version in database");
-						sqlx_admin_inc_all_versions(handle, 2);
-					}
-
-				} else {
-					// database 1.7  doesn't existe ; new grid
-					g_clear_error(&err);
-					err = NULL;
-				}
-			} else {
-				GRID_ERROR("Failed to create meta0 database :(%d) %s",err->code,err->message);
-			}
-		}
-	}
-	if ( handle)
-		sqlx_repository_unlock_and_close_noerror(handle);
-
-}
-
-/* ------------------------------------------------------------------------- */
-
 static GError*
 _load_from_base(struct sqlx_sqlite3_s *sq3, GPtrArray **result)
 {
@@ -239,7 +139,6 @@ _load_from_base(struct sqlx_sqlite3_s *sq3, GPtrArray **result)
 				GRID_WARN("Invalid prefix for URL [%s] ROWID %"G_GINT64_FORMAT,
 						url, rowid);
 			else {
-				meta0_utils_check_url_from_base((gchar**)&url);
 				meta0_utils_array_add(array, prefix, (gchar*)url);
 				count ++;
 			}
@@ -345,63 +244,63 @@ _load(struct meta0_backend_s *m0)
 	return err;
 }
 
-static GError*
-_fill_in__transaction (sqlite3 *db, const char * const *urls, guint max, guint shift)
+static GError *
+_json_to_meta0_mapping(const char *json_mapping, GPtrArray **result)
 {
-	gint rc;
-	guint idx;
-	sqlite3_stmt *stmt = NULL;
 	GError *err = NULL;
+	GPtrArray *urls_by_pfx = NULL;
+	json_object *jbody = NULL;
+	json_tokener *parser = json_tokener_new();
 
-	sqlite3_prepare_debug(rc, db, "INSERT INTO meta1"
-			" (prefix,addr) VALUES (?,?)", -1, &stmt, NULL);
-	if (rc != SQLITE_OK && rc != SQLITE_DONE)
-		return SQLITE_GERROR(db, rc);
+	jbody = json_tokener_parse_ex(parser, json_mapping, strlen(json_mapping));
 
-	/* One partition for each url */
-	for (idx=0; idx<65536 ;idx++) {
-
-		const char * const *purl = urls + ((idx+shift) % max);
-		guint16 index16 = idx;
-
-		sqlite3_reset(stmt);
-		sqlite3_clear_bindings(stmt);
-		sqlite3_bind_blob(stmt, 1, &index16, 2, NULL);
-		sqlite3_bind_text(stmt, 2, *purl, -1, NULL);
-		while (!err) {
-			rc = sqlite3_step(stmt);
-			if (rc == SQLITE_OK || rc == SQLITE_DONE)
-				break;
-			if (rc == SQLITE_BUSY)
-				sleep(1);
-			else
-				err = SQLITE_GERROR(db, rc);
+	if (json_tokener_get_error(parser) != json_tokener_success) {
+		err = NEWERROR(CODE_BAD_REQUEST, "Invalid JSON");
+	} else if (!json_object_is_type(jbody, json_type_object)) {
+		err = NEWERROR(CODE_BAD_REQUEST,
+				"Invalid JSON object: must be a hash");
+	} else {
+		urls_by_pfx = meta0_utils_array_create();
+		json_object_object_foreach(jbody, pfx_str, urls_obj) {
+			guint8 pfx[2] = {0, 0};
+			int url_count = json_object_array_length(urls_obj);
+			oio_str_hex2bin(pfx_str, pfx, 2);
+			for (int i = 0; i < url_count; i++) {
+				const char *url = json_object_get_string(
+						json_object_array_get_idx(urls_obj, i));
+				meta0_utils_array_add(urls_by_pfx, pfx, url);
+			}
 		}
 	}
-	sqlite3_finalize_debug(rc, stmt);
 
+	if (jbody)
+		json_object_put(jbody);
+	json_tokener_free(parser);
+
+	if (err)
+		meta0_utils_array_clean(urls_by_pfx);
+	else
+		*result = urls_by_pfx;
 	return err;
 }
 
 static GError*
-_fill(struct sqlx_sqlite3_s *sq3, guint replicas, const char * const *m1urls)
+_fill_mapping_holes(struct meta0_backend_s *m0, GPtrArray *mapping)
 {
-	const guint max = oio_strv_length(m1urls);
-	EXTRA_ASSERT(max > 0 && max < 65536);
+	GError *err = NULL;
 
-	struct sqlx_repctx_s *repctx = NULL;
-	GError *err = sqlx_transaction_begin(sq3, &repctx);
-	if (NULL != err)
-		return err;
+	for (int idx = 0; idx < CID_PREFIX_COUNT && !err; idx++) {
+		gchar **url = mapping->pdata[idx];
+		guint16 index16 = idx;
 
-	while (replicas--) {
-		err = _fill_in__transaction(sq3->db, m1urls, max, replicas);
-		if (err)
-			 break;
+		if (url && *url)
+			continue;
+
+		err = meta0_backend_get_one(m0,
+				(guint8*)&index16, (gchar***)&mapping->pdata[idx]);
 	}
-	if (!err)
-		sqlx_transaction_notify_huge_changes(repctx);
-	return sqlx_transaction_end(repctx, err);
+
+	return err;
 }
 
 static GError *
@@ -477,7 +376,7 @@ _assign_prefixes(sqlite3 *db, const GPtrArray *new_assign_prefixes,
 	guint idx;
 	sqlite3_stmt *stmt = NULL;
 
-	if ( !init ) {
+	if (!init) {
 		sqlite3_prepare_debug(rc, db, "DELETE FROM meta1", -1, &stmt, NULL);
 		if (rc != SQLITE_OK && rc != SQLITE_DONE)
 			return SQLITE_GERROR(db, rc);
@@ -494,21 +393,21 @@ _assign_prefixes(sqlite3 *db, const GPtrArray *new_assign_prefixes,
 		sqlite3_finalize_debug(rc, stmt);
 	}
 
-	sqlite3_prepare_debug(rc, db, "INSERT  INTO meta1"
+	sqlite3_prepare_debug(rc, db, "INSERT INTO meta1"
 			" (prefix,addr) VALUES (?,?)", -1, &stmt, NULL);
 
 	if (rc != SQLITE_OK && rc != SQLITE_DONE) {
 		return SQLITE_GERROR(db, rc);
 	}
 
-	for (idx=0; idx<65536 ;idx++) {
-
+	for (idx = 0; idx < CID_PREFIX_COUNT; idx++) {
 		gchar **url = new_assign_prefixes->pdata[idx];
 		guint16 index16 = idx;
 
-		if (!url || ! *url )
+		if (!url || !*url)
 			continue;
-		for (; *url ;url++) {
+
+		for (; *url; url++) {
 			sqlite3_reset(stmt);
 			sqlite3_clear_bindings(stmt);
 			sqlite3_bind_blob(stmt, 1, &index16, 2, NULL);
@@ -528,7 +427,6 @@ _assign_prefixes(sqlite3 *db, const GPtrArray *new_assign_prefixes,
 	sqlite3_finalize_debug(rc, stmt);
 
 	return err;
-
 }
 
 static GError *
@@ -552,7 +450,7 @@ _record_meta1ref(sqlite3 *db, const GPtrArray *new_assign_meta1ref)
 	for (idx=0; idx < new_assign_meta1ref->len; idx++) {
 		gchar *m1ref = new_assign_meta1ref->pdata[idx];
 		gchar *addr, *ref, *nb;
-		if ( ! meta0_utils_unpack_meta1ref(m1ref,&addr,&ref,&nb) )
+		if (!meta0_utils_unpack_meta1ref(m1ref,&addr,&ref,&nb))
 			continue;
 		sqlite3_reset(stmt);
 		sqlite3_clear_bindings(stmt);
@@ -605,19 +503,39 @@ _delete_meta1_ref(sqlite3 *db, gchar *meta1_ref)
 /* ------------------------------------------------------------------------- */
 
 GError*
-meta0_backend_fill(struct meta0_backend_s *m0, guint replicas, const char * const *m1urls)
+meta0_backend_fill_from_json(struct meta0_backend_s *m0,
+		const char *json_mapping)
 {
 	EXTRA_ASSERT(m0 != NULL);
-	EXTRA_ASSERT(replicas < 65535);
-	EXTRA_ASSERT(m1urls != 0);
+	EXTRA_ASSERT(json_mapping != NULL);
 
+	GError *err = NULL;
+	GPtrArray *mapping = NULL;
 	struct sqlx_sqlite3_s *sq3 = NULL;
-	GError *err = _open_and_lock(m0, M0V2_OPENBASE_MASTERONLY, &sq3);
-	if (!err) {
-		err = _fill(sq3, replicas, m1urls);
-		_unlock_and_close(sq3);
-	}
+	struct sqlx_repctx_s *repctx = NULL;
 
+	err = _json_to_meta0_mapping(json_mapping, &mapping);
+	if (err)
+		goto cleanup;
+
+	err = _fill_mapping_holes(m0, mapping);
+	if (err)
+		goto cleanup;
+
+	err = _open_and_lock(m0, M0V2_OPENBASE_MASTERONLY, &sq3);
+	if (err)
+		goto cleanup;
+
+	if (!(err = sqlx_transaction_begin(sq3, &repctx))) {
+		err = _assign_prefixes(sq3->db, mapping, 0);
+		if (!err)
+			sqlx_transaction_notify_huge_changes(repctx);
+		err = sqlx_transaction_end(repctx, err);
+	}
+	_unlock_and_close(sq3);
+
+cleanup:
+	meta0_utils_array_clean(mapping);
 	return err;
 }
 
@@ -658,7 +576,7 @@ meta0_backend_reset(struct meta0_backend_s *m0, gboolean flag_local)
 	}
 
 	_unlock_and_close (sq3);
-	return NULL;
+	return err;
 }
 
 GError*
