@@ -1,7 +1,7 @@
 /*
 OpenIO SDS metautils
 Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO, modified as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -36,6 +36,10 @@ License along with this library.
 # define EVENT_BUFFER_SIZE 2048
 #endif
 
+#ifdef HAVE_ENBUG
+gint32 oio_client_timeout_threshold = 10;
+#endif
+
 enum client_step_e
 {
 	NONE = 0,
@@ -75,14 +79,90 @@ struct gridd_client_s
 	int fd;
 	guint sent_bytes;
 
-	enum client_step_e step : 16;
-	gboolean keepalive : 8;
-	gboolean forbid_redirect : 8;
+	enum client_step_e step : 4;
+	guint8 keepalive : 1;
+	guint8 forbid_redirect : 1;
+	guint8 avoid_on_error : 1;
 
 	gchar orig_url[URL_MAXLEN];
 	gchar url[URL_MAXLEN];
 };
 
+/* Cache of network errors ------------------------------------------------- */
+
+guint64 oio_client_max_net_errors = 15;
+time_t oio_client_period_net_errors = 15;
+gboolean oio_cache_avoid_on_error = FALSE;
+
+static GMutex lock_errors;
+static GTree *tree_errors = NULL;
+
+void  _oio_cache_of_errors_contructor (void);
+
+void __attribute__ ((constructor))
+_oio_cache_of_errors_contructor (void)
+{
+	static volatile guint lazy_init = 1;
+	if (lazy_init) {
+		if (g_atomic_int_compare_and_exchange(&lazy_init, 1, 0)) {
+			g_mutex_init (&lock_errors);
+			tree_errors = g_tree_new_full(metautils_strcmp3, NULL,
+					g_free, (GDestroyNotify) grid_single_rrd_destroy);
+		}
+	}
+}
+
+static gboolean
+_has_too_many_errors (const char *url)
+{
+	if (!url || !*url)
+		return FALSE;
+
+	guint64 delta = 0;
+	g_mutex_lock(&lock_errors);
+
+	struct grid_single_rrd_s *rrd = g_tree_lookup(tree_errors, url);
+	if (rrd) {
+		const gint64 now = oio_ext_monotonic_seconds();
+		delta = grid_single_rrd_get_delta(rrd, now, oio_client_period_net_errors);
+	}
+	g_mutex_unlock(&lock_errors);
+
+	gboolean rc = (oio_client_max_net_errors <= delta);
+	if (rc) {
+		GRID_DEBUG("[%s] seems DOWN (%"G_GUINT64_FORMAT" fails in %lds)",
+				url, delta, oio_client_period_net_errors);
+	}
+	return rc;
+}
+
+static void
+_count_network_error (const char *url, const GError *err)
+{
+	if (!url || !*url || !err)
+		return;
+	if (!CODE_IS_NETWORK_ERROR(err->code) || err->code == CODE_AVOIDED)
+		return;
+
+	/* all the network errors are considered, with the notable exception on
+	 * CODE_AVOIDED that is precisely generated by the current feature.
+	 * Using it to detect a failed service would be false (we don't know,
+	 * in facts) and it would self-feed the feature. */
+
+	g_mutex_lock(&lock_errors);
+	struct grid_single_rrd_s *rrd = g_tree_lookup(tree_errors, url);
+	if (!rrd) {
+		rrd = grid_single_rrd_create(oio_ext_monotonic_seconds(),
+				oio_client_period_net_errors + 1);
+		g_tree_replace(tree_errors, g_strdup(url), rrd);
+	}
+	const gint64 now = oio_ext_monotonic_seconds();
+	grid_single_rrd_add(rrd, now, 1);
+	g_mutex_unlock(&lock_errors);
+}
+
+/* ------------------------------------------------------------------------- */
+
 static void _client_free(struct gridd_client_s *client);
 static GError* _client_connect_url(struct gridd_client_s *client, const gchar *url);
 static GError* _client_request(struct gridd_client_s *client, GByteArray *req,
@@ -96,6 +176,7 @@ static GError* _client_error(struct gridd_client_s *client);
 static gboolean _client_start(struct gridd_client_s *client);
 static GError* _client_set_fd(struct gridd_client_s *client, int fd);
 static void _client_set_timeout(struct gridd_client_s *client, gdouble seconds);
+static void _client_set_timeout_cnx(struct gridd_client_s *client, gdouble sec);
 static void _client_set_keepalive(struct gridd_client_s *client, gboolean on);
 static void _client_react(struct gridd_client_s *client);
 static gboolean _client_expire(struct gridd_client_s *client, gint64 now);
@@ -123,6 +204,7 @@ struct gridd_client_vtable_s VTABLE_CLIENT =
 	_client_set_fd,
 	_client_set_keepalive,
 	_client_set_timeout,
+	_client_set_timeout_cnx,
 	_client_expired,
 	_client_finished,
 	_client_start,
@@ -187,16 +269,17 @@ _client_reset_target(struct gridd_client_s *client)
 }
 
 static void
-_client_reset_error(struct gridd_client_s *client)
+_client_replace_error(struct gridd_client_s *client, GError *e)
 {
 	if (client->error)
 		g_clear_error(&(client->error));
+	client->error = e;
 }
 
 static GError *
 _client_manage_reply(struct gridd_client_s *client, MESSAGE reply)
 {
-	GError *err;
+	GError *err = NULL;
 	guint status = 0;
 	gchar *message = NULL;
 
@@ -237,9 +320,22 @@ _client_manage_reply(struct gridd_client_s *client, MESSAGE reply)
 		_client_reset_cnx(client);
 		client->sent_bytes = 0;
 
-		if ((++ client->nb_redirects) > 3)
+		++ client->nb_redirects;
+
+		/* Allow 4 redirections (i.e. 5 attemps) */
+		if (client->nb_redirects > 4)
 			return NEWERROR(CODE_TOOMANY_REDIRECT, "Too many redirections");
 
+		/* The first redirection is legit, but subsequent tell us there is
+		 * something happening with the election. Let the services get a
+		 * final status and wait for it a little bit.
+		 * We will wait 200, 400, 800 ms. */
+		if (client->nb_redirects > 1) {
+			guint backoff = client->nb_redirects - 1;
+			gulong delay = (1<<backoff) * 100 * G_TIME_SPAN_MILLISECOND;
+			g_usleep(delay);
+		}
+
 		/* Replace the URL */
 		g_strlcpy(client->url, message, URL_MAXLEN);
 		if (NULL != (err = _client_connect(client)))
@@ -247,16 +343,11 @@ _client_manage_reply(struct gridd_client_s *client, MESSAGE reply)
 		return err;
 	}
 
-	/* all other are considered errors */
-	if (status != CODE_REDIRECT)
-		err = NEWERROR(status, "Request error: %s", message);
-	else
-		err = NEWERROR(status, "%s", message);
-
 	if (!client->keepalive)
 		_client_reset_cnx(client);
 	_client_reset_reply(client);
-	return err;
+
+	return NEWERROR(status, "%s", message);
 }
 
 static GError *
@@ -295,9 +386,10 @@ _client_manage_event_in_buffer(struct gridd_client_s *client, guint8 *d, gsize d
 			_client_reset_reply(client);
 
 			/* Continue to send the request */
-			rc = metautils_syscall_write(client->fd,
+			rc = metautils_syscall_send(client->fd,
 					client->request->data + client->sent_bytes,
-					client->request->len - client->sent_bytes);
+					client->request->len - client->sent_bytes,
+					MSG_NOSIGNAL);
 
 			if (rc < 0)
 				return (errno == EINTR || errno == EAGAIN) ? NULL :
@@ -409,12 +501,11 @@ retry:
 		if (client->step == REP_READING_SIZE && client->reply
 				&& client->reply->len >= 4)
 				goto retry;
-	}
-	else {
+	} else {
 		_client_reset_request(client);
 		_client_reset_reply(client);
 		_client_reset_cnx(client);
-		client->error = err;
+		_client_replace_error(client, err);
 		client->step = STATUS_FAILED;
 	}
 }
@@ -488,11 +579,14 @@ _client_free(struct gridd_client_s *client)
 	EXTRA_ASSERT(client != NULL);
 	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
 
+	if (client->avoid_on_error)
+		_count_network_error(client->url, client->error);
+
 	_client_reset_reply(client);
 	_client_reset_request(client);
 	_client_reset_cnx(client);
 	_client_reset_target(client);
-	_client_reset_error(client);
+	_client_replace_error(client, NULL);
 	if (client->reply)
 		g_byte_array_free(client->reply, TRUE);
 	client->fd = -1;
@@ -514,11 +608,19 @@ _client_set_timeout(struct gridd_client_s *client, gdouble seconds)
 	EXTRA_ASSERT(client != NULL);
 	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
 
-	client->delay_connect = COMMON_CNX_TIMEOUT;
 	client->delay_single = seconds * (gdouble) G_TIME_SPAN_SECOND;
 	client->delay_overall = seconds * (gdouble) G_TIME_SPAN_SECOND;
 }
 
+static void
+_client_set_timeout_cnx(struct gridd_client_s *client, gdouble seconds)
+{
+	EXTRA_ASSERT(client != NULL);
+	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
+
+	client->delay_connect = seconds * (gdouble) G_TIME_SPAN_SECOND;
+}
+
 static GError*
 _client_set_fd(struct gridd_client_s *client, int fd)
 {
@@ -617,7 +719,7 @@ _client_request(struct gridd_client_s *client, GByteArray *req,
 	/* if any, reset the last reply */
 	_client_reset_reply(client);
 	_client_reset_request(client);
-	_client_reset_error(client);
+	_client_replace_error(client, NULL);
 
 	/* Now set the new request components */
 	client->ctx = ctx;
@@ -665,6 +767,15 @@ _client_expired(struct gridd_client_s *client, gint64 now)
 	return FALSE;
 }
 
+static void
+_client_react_timeout(struct gridd_client_s *client)
+{
+	_client_reset_cnx(client);
+	_client_replace_error(client, NEWERROR(client->step == CONNECTING
+			? ERRCODE_CONN_TIMEOUT : ERRCODE_READ_TIMEOUT, "Timeout"));
+	client->step = STATUS_FAILED;
+}
+
 static gboolean
 _client_expire(struct gridd_client_s *client, gint64 now)
 {
@@ -673,12 +784,19 @@ _client_expire(struct gridd_client_s *client, gint64 now)
 
 	if (_client_finished(client))
 		return FALSE;
-	if (!_client_expired(client, now))
+
+	if (!_client_expired(client, now)) {
+#ifdef HAVE_ENBUG
+		if (oio_client_timeout_threshold >= oio_ext_rand_int_range(1,100)) {
+			_client_react_timeout(client);
+			return TRUE;
+		}
+#endif
 		return FALSE;
-	_client_reset_cnx(client);
-	client->error = NEWERROR(ERRCODE_READ_TIMEOUT, "Timeout");
-	client->step = STATUS_FAILED;
-	return FALSE;
+	}
+
+	_client_react_timeout(client);
+	return TRUE;
 }
 
 static gboolean
@@ -723,17 +841,24 @@ _client_start(struct gridd_client_s *client)
 		return FALSE;
 
 	if (!client->url[0]) {
-		_client_reset_error(client);
-		client->error = NEWERROR(EINVAL, "No target");
+		_client_replace_error(client, NEWERROR(EINVAL, "No target"));
 		return FALSE;
 	}
 
+	if (oio_cache_avoid_on_error && client->avoid_on_error) {
+		if (_has_too_many_errors(client->url)) {
+			_client_replace_error(client, NEWERROR(CODE_AVOIDED,
+						"Request avoided, service probably down"));
+			return FALSE;
+		}
+	}
+
 	GError *err = _client_connect(client);
 	if (NULL == err)
 		return TRUE;
 
 	client->step = STATUS_FAILED;
-	client->error = err;
+	_client_replace_error(client, err);
 	return FALSE;
 }
 
@@ -742,9 +867,7 @@ _client_fail(struct gridd_client_s *client, GError *why)
 {
 	EXTRA_ASSERT(client != NULL);
 	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
-	if (client->error != NULL)
-		g_clear_error(&(client->error));
-	client->error = NEWERROR(why->code, "%s", why->message);
+	_client_replace_error(client, why);
 }
 
 static void
@@ -761,11 +884,7 @@ _factory_create_client (struct gridd_client_factory_s *factory)
 	EXTRA_ASSERT(factory != NULL);
 	EXTRA_ASSERT(factory->abstract.vtable == &VTABLE_FACTORY);
 	(void) factory;
-	struct gridd_client_s *client = gridd_client_create_empty();
-	if (!client)
-		return NULL;
-	gridd_client_set_timeout(client, COMMON_CLIENT_TIMEOUT);
-	return client;
+	return gridd_client_create_empty();
 }
 
 /* ------------------------------------------------------------------------- */
@@ -780,9 +899,11 @@ gridd_client_create_empty(void)
 	client->abstract.vtable = &VTABLE_CLIENT;
 	client->fd = -1;
 	client->step = NONE;
-	client->delay_overall = GRIDC_DEFAULT_TIMEOUT_OVERALL * (gdouble)G_TIME_SPAN_SECOND;
-	client->delay_single = GRIDC_DEFAULT_TIMEOUT_STEP * (gdouble)G_TIME_SPAN_SECOND;
+	client->delay_overall = COMMON_CLIENT_TIMEOUT * (gdouble)G_TIME_SPAN_SECOND;
+	client->delay_single = COMMON_CLIENT_TIMEOUT * (gdouble)G_TIME_SPAN_SECOND;
+	client->delay_connect = COMMON_CNX_TIMEOUT * (gdouble)G_TIME_SPAN_SECOND;
 	client->tv_start = client->tv_connect = oio_ext_monotonic_time ();
+	client->avoid_on_error = BOOL(oio_cache_avoid_on_error);
 
 	return client;
 }
@@ -866,6 +987,12 @@ gridd_client_set_timeout (struct gridd_client_s *self, gdouble seconds)
 	GRIDD_CALL(self,set_timeout)(self,seconds);
 }
 
+void
+gridd_client_set_timeout_cnx (struct gridd_client_s *self, gdouble seconds)
+{
+	GRIDD_CALL(self,set_timeout_cnx)(self,seconds);
+}
+
 gboolean
 gridd_client_expired(struct gridd_client_s *self, gint64 now)
 {
@@ -887,7 +1014,6 @@ gridd_client_start (struct gridd_client_s *self)
 gboolean
 gridd_client_expire (struct gridd_client_s *self, gint64 now)
 {
-
 	GRIDD_CALL(self,expire)(self,now);
 }
 
