@@ -1,7 +1,7 @@
 /*
 OpenIO SDS proxy
 Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -34,10 +34,12 @@ static GThread *downstream_thread = NULL;
 guint csurl_refresh_delay = PROXYD_PERIOD_RELOAD_CSURL;
 guint srvtypes_refresh_delay = PROXYD_PERIOD_RELOAD_SRVTYPES;
 guint nsinfo_refresh_delay = PROXYD_PERIOD_RELOAD_NSINFO;
+time_t nsinfo_refresh_m0 = PROXYD_PERIOD_RELOAD_M0INFO;
 
-gint64 cs_expire_local_services = PROXYD_TTL_DEAD_LOCAL_SERVICES;
-gint64 cs_down_services = PROXYD_TTL_DOWN_SERVICES;
-gint64 cs_known_services = PROXYD_TTL_KNOWN_SERVICES;
+gint64 ttl_expire_local_services = PROXYD_TTL_DEAD_LOCAL_SERVICES;
+gint64 ttl_down_services = PROXYD_TTL_DOWN_SERVICES;
+gint64 ttl_known_services = PROXYD_TTL_KNOWN_SERVICES;
+gint64 ttl_expire_master_services = PROXYD_TTL_MASTER_SERVICES;
 
 // Configuration
 
@@ -50,19 +52,26 @@ static guint dir_low_max = PROXYD_DEFAULT_MAX_SERVICES;
 static guint dir_high_ttl = PROXYD_DEFAULT_TTL_CSM0 / G_TIME_SPAN_SECOND;
 static guint dir_high_max = PROXYD_DEFAULT_MAX_CSM0;
 
-gchar *nsname = NULL;
+gchar *ns_name = NULL;
 gboolean flag_cache_enabled = TRUE;
 gboolean flag_local_scores = FALSE;
+gboolean flag_prefer_master_for_write = TRUE;
+gboolean flag_prefer_master_for_read = FALSE;
+gboolean flag_prefer_slave_for_read = FALSE;
+gboolean flag_force_master = FALSE;
 
-struct grid_lbpool_s *lbpool = NULL;
+struct oio_lb_world_s *lb_world = NULL;
+struct oio_lb_s *lb = NULL;
 struct hc_resolver_s *resolver = NULL;
 
 GRWLock csurl_rwlock = {0};
 gchar **csurl = NULL;
 gsize csurl_count = 0;
 
-GMutex push_mutex = {0};
+GRWLock push_rwlock = {0};
 struct lru_tree_s *push_queue = NULL;
+
+GRWLock reg_rwlock = {0};
 struct lru_tree_s *srv_registered = NULL;
 
 GRWLock nsinfo_rwlock = {0};
@@ -70,6 +79,9 @@ struct namespace_info_s nsinfo = {{0}};
 gchar **srvtypes = NULL;
 gint64 ns_chunk_size = 10*1024*1024;
 
+GRWLock master_rwlock = {0};
+struct lru_tree_s *srv_master = NULL;
+
 GRWLock srv_rwlock = {0};
 struct lru_tree_s *srv_down = NULL;
 struct lru_tree_s *srv_known = NULL;
@@ -201,11 +213,6 @@ _metacd_load_url (struct req_args_s *args)
 static enum http_rc_e
 handler_action (struct http_request_s *rq, struct http_reply_ctx_s *rp)
 {
-	gboolean _boolhdr (const gchar * n) {
-		return metautils_cfg_get_bool (
-			(gchar *) g_tree_lookup (rq->tree_headers, n), FALSE);
-	}
-
 	// Get a request id for the current request
 	const gchar *reqid = g_tree_lookup (rq->tree_headers, PROXYD_HEADER_REQID);
 	if (reqid)
@@ -213,6 +220,9 @@ handler_action (struct http_request_s *rq, struct http_reply_ctx_s *rp)
 	else
 		oio_ext_set_random_reqid();
 
+	const gchar *admin = g_tree_lookup (rq->tree_headers, PROXYD_HEADER_ADMIN);
+	oio_ext_set_admin(oio_str_parse_bool(admin, FALSE));
+
 	// Then parse the request to find a handler
 	struct oio_url_s *url = NULL;
 	struct oio_requri_s ruri = {NULL, NULL, NULL, NULL};
@@ -235,15 +245,12 @@ handler_action (struct http_request_s *rq, struct http_reply_ctx_s *rp)
 		rp->finalize ();
 		rc = HTTPRC_DONE;
 	} else {
-		struct req_args_s args = {NULL,NULL,NULL, NULL,NULL, 0};
+		struct req_args_s args = {0};
 		args.req_uri = &ruri;
 		args.matchings = matchings;
 		args.rq = rq;
 		args.rp = rp;
 
-		if (_boolhdr (PROXYD_HEADER_NOEMPTY))
-			args.flags |= FLAG_NOEMPTY;
-
 		args.url = url = _metacd_load_url (&args);
 		rp->subject(oio_url_get(url, OIOURL_HEXID));
 		gq_count = (*matchings)->last->gq_count;
@@ -276,37 +283,53 @@ _push_queue_create (void)
 // Administrative tasks --------------------------------------------------------
 
 static guint
-_expire_services (struct lru_tree_s *lru, const gint64 delay)
+_lru_tree_expire (GRWLock *rw, struct lru_tree_s *lru, const gint64 delay)
 {
 	if (delay <= 0) return 0;
 	const gint64 now = oio_ext_monotonic_time();
-	guint count = 0;
-	SRV_WRITE(count = lru_tree_remove_older(lru, OLDEST(now,delay)));
+	g_rw_lock_writer_lock (rw);
+	guint count = lru_tree_remove_older (lru, OLDEST(now,delay));
+	g_rw_lock_writer_unlock (rw);
 	return count;
 }
 
 static void
-_task_expire_services_known (gpointer p)
+_task_expire_services_master (gpointer p UNUSED)
 {
-	(void) p;
-	guint count = _expire_services (srv_known, cs_known_services);
+	guint count = _lru_tree_expire (&master_rwlock, srv_master,
+			ttl_expire_master_services);
+	if (count)
+		GRID_DEBUG("Expired %u masters", count);
+}
+
+static void
+_task_expire_services_known (gpointer p UNUSED)
+{
+	guint count = _lru_tree_expire (&srv_rwlock, srv_known, ttl_known_services);
 	if (count)
 		GRID_INFO("Forgot %u services", count);
 }
 
 static void
-_task_expire_services_down (gpointer p)
+_task_expire_services_down (gpointer p UNUSED)
 {
-	(void) p;
-	guint count = _expire_services (srv_down, cs_down_services);
+	guint count = _lru_tree_expire (&srv_rwlock, srv_down, ttl_down_services);
 	if (count)
-		GRID_INFO("re-enabled %u services", count);
+		GRID_INFO("Re-enabled %u services", count);
 }
 
 static void
-_task_expire_resolver (gpointer p)
+_task_expire_local (gpointer p UNUSED)
+{
+	guint count = _lru_tree_expire (&reg_rwlock, srv_registered,
+			ttl_expire_local_services);
+	if (count)
+		GRID_INFO("Expired %u local services", count);
+}
+
+static void
+_task_expire_resolver (gpointer p UNUSED)
 {
-	(void) p;
 	guint count_expire = hc_resolver_expire (resolver);
 	guint count_purge = hc_resolver_purge (resolver);
 	if (count_expire || count_purge) {
@@ -316,7 +339,7 @@ _task_expire_resolver (gpointer p)
 }
 
 static void
-_local_score_update (const struct service_info_s *si0)
+_NOLOCK_local_score_update (const struct service_info_s *si0)
 {
 	gchar *k = service_info_key (si0);
 	STRING_STACKIFY(k);
@@ -360,23 +383,8 @@ _encode_wanted_services (const char *type, GSList *list)
 }
 
 static void
-_reload_srvtype(const char *type)
+_reload_srvtype(const char *type, GSList *list)
 {
-	CSURL(cs);
-
-	GSList *list = NULL;
-	GError *err = conscience_remote_get_services (cs, type, FALSE, &list);
-	if (err) {
-		GRID_WARN("Services listing error for type[%s]: code=%d %s",
-				type, err->code, err->message);
-		g_clear_error(&err);
-		return;
-	}
-
-	if (GRID_TRACE_ENABLED()) {
-		GRID_TRACE ("SRV loaded %u [%s]", g_slist_length(list), type);
-	}
-
 	/* reloads the known services */
 	gulong now = oio_ext_monotonic_seconds ();
 	SRV_WRITE(for (GSList *l=list; l ;l=l->next) {
@@ -387,7 +395,7 @@ _reload_srvtype(const char *type)
 	/* updates the score of the local services */
 	if (flag_local_scores && NULL != list) {
 		for (GSList *l=list; l ;l=l->next)
-			PUSH_DO(_local_score_update(l->data));
+			REG_WRITE(_NOLOCK_local_score_update(l->data));
 	}
 
 	/* prepares a cache of services wanted by the clients */
@@ -397,56 +405,130 @@ _reload_srvtype(const char *type)
 		g_bytes_unref (encoded);
 	}
 
-	/* reload the LB */
-	if (NULL != list) {
-		GSList *l = list;
-		gboolean provide(struct service_info_s **p_si) {
-			if (!l) return FALSE;
-			*p_si = l->data;
-			l->data = NULL;
-			l = l->next;
-			return TRUE;
-		}
-		grid_lbpool_reload(lbpool, type, provide);
-		g_slist_free(list);
-	}
+	/* reload the LB world */
+	if (list)
+		oio_lb_world__feed_service_info_list(lb_world, list);
 }
 
 static void
-_task_reload_lbpool (gpointer p)
+_reload_lb_service_types(
+		struct oio_lb_world_s *lbw, struct oio_lb_s *lb_,
+		struct namespace_info_s *nsi,
+		gchar **tabtypes, GPtrArray *tabsrv, GPtrArray *taberr)
+{
+	struct service_update_policies_s *pols = service_update_policies_create();
+	gchar *pols_cfg = gridcluster_get_service_update_policy(nsi);
+	service_update_reconfigure(pols, pols_cfg);
+	g_free(pols_cfg);
+
+	for (guint i=0; tabtypes[i] ;++i) {
+		const char * srvtype = tabtypes[i];
+		if (!oio_lb__has_pool(lb_, srvtype)) {
+			GRID_DEBUG("Creating pool for service type [%s]", srvtype);
+			oio_lb__force_pool(lb_,
+					oio_lb_pool__from_service_policy(lbw, srvtype, pols));
+		}
+
+		if (!taberr->pdata[i])
+			_reload_srvtype(srvtype, tabsrv->pdata[i]);
+	}
+
+	service_update_policies_destroy(pols);
+}
+
+static void
+_free_list_of_services(gpointer p)
+{
+	if (!p)
+		return;
+	g_slist_free_full((GSList*)p, (GDestroyNotify)service_info_clean);
+}
+
+static void
+_free_error(gpointer p)
+{
+	if (!p)
+		return;
+	g_error_free((GError*)p);
+}
+
+/* If you ever plan to factorize this code with the similar part in
+ * sqlx/sqlx_service.c be carefull that a lot of context is expected on both
+ * sides, and that even the function used to fetch the services cannot be the
+ * same (sqlx talks to the proxy, the proxy talks to the conscience). */
+gboolean
+lb_cache_reload (void)
+{
+	struct namespace_info_s *nsi = NULL;
+	gchar **tabtypes = NULL;
+	GPtrArray *tabsrv = NULL, *taberr = NULL;
+	gboolean any_loading_error = FALSE;
+
+	CSURL(cs);
+	NSINFO_READ(
+		tabtypes = g_strdupv_inline(srvtypes);
+		nsi = namespace_info_dup(&nsinfo);
+	);
+
+	if (!tabtypes || !*tabtypes) {
+		GRID_WARN("proxy not ready to reload the LB");
+		any_loading_error = TRUE;
+		goto out;
+	}
+
+	/* preload all the services */
+	tabsrv = g_ptr_array_new_full(8, _free_list_of_services);
+	taberr = g_ptr_array_new_full(8, _free_error);
+	for (char **pt=tabtypes; *pt ;++pt) {
+		GSList *srv = NULL;
+		GError *e = conscience_remote_get_services(cs, *pt, FALSE, &srv);
+		if (e) {
+			GRID_WARN("Failed to load the list of [%s] in NS=%s", *pt, ns_name);
+			any_loading_error = TRUE;
+		}
+		g_ptr_array_add(tabsrv, srv);
+		g_ptr_array_add(taberr, e);
+	}
+
+	/* refresh the load-balancing world */
+	if (!any_loading_error)
+		oio_lb_world__increment_generation(lb_world);
+	oio_lb_world__reload_pools(lb_world, lb, nsi);
+	_reload_lb_service_types(lb_world, lb, nsi, tabtypes, tabsrv, taberr);
+	oio_lb_world__reload_storage_policies(lb_world, lb, nsi);
+	if (!any_loading_error)
+		oio_lb_world__purge_old_generations(lb_world);
+out:
+	if (tabtypes) g_free0 (tabtypes);
+	if (nsi) namespace_info_free(nsi);
+	if (tabsrv) g_ptr_array_free(tabsrv, TRUE);
+	if (taberr) g_ptr_array_free(taberr, TRUE);
+	return !any_loading_error;
+}
+
+static void
+_task_reload_lb(gpointer p UNUSED)
 {
 	ADAPTIVE_PERIOD_DECLARE();
-	(void) p;
 
 	if (ADAPTIVE_PERIOD_SKIP())
 		return;
 
 	CSURL(cs);
-	if (!cs) return;
-
-	ADAPTIVE_PERIOD_ONSUCCESS(lb_downstream_delay);
-	gchar **tt = NULL;
-	NSINFO_READ(tt = g_strdupv_inline(srvtypes));
-
-	if (tt) {
-		for (gchar **t=tt; *t ;++t)
-			_reload_srvtype (*t);
-		g_free (tt);
+	if (cs && lb_cache_reload()) {
+		ADAPTIVE_PERIOD_ONSUCCESS(lb_downstream_delay);
 	}
-
-	grid_lbpool_reconfigure(lbpool, &nsinfo);
 }
 
 static void
-_task_reload_csurl (gpointer p)
+_task_reload_csurl (gpointer p UNUSED)
 {
 	ADAPTIVE_PERIOD_DECLARE ();
-	(void) p;
 
 	if (ADAPTIVE_PERIOD_SKIP())
 		return;
 
-	gchar *s = oio_cfg_get_value(nsname, OIO_CFG_CONSCIENCE);
+	gchar *s = oio_cfg_get_value(ns_name, OIO_CFG_CONSCIENCE);
 	STRING_STACKIFY(s);
 	if (s) {
 		ADAPTIVE_PERIOD_ONSUCCESS(csurl_refresh_delay);
@@ -466,14 +548,13 @@ _task_reload_csurl (gpointer p)
 }
 
 static void
-_task_reload_nsinfo (gpointer p)
+_task_reload_nsinfo (gpointer p UNUSED)
 {
 	ADAPTIVE_PERIOD_DECLARE ();
-	(void) p;
 
 	CSURL(cs);
 	if (!cs)
-		return;
+		return; /* not ready */
 
 	if (ADAPTIVE_PERIOD_SKIP())
 		return;
@@ -482,7 +563,7 @@ _task_reload_nsinfo (gpointer p)
 	GError *err = conscience_remote_get_namespace (cs, &ni);
 	if (err) {
 		GRID_WARN ("NSINFO reload error [%s] from [%s]: (%d) %s",
-			nsname, cs, err->code, err->message);
+			ns_name, cs, err->code, err->message);
 		g_clear_error (&err);
 	} else {
 		ADAPTIVE_PERIOD_ONSUCCESS(nsinfo_refresh_delay);
@@ -493,10 +574,9 @@ _task_reload_nsinfo (gpointer p)
 }
 
 static void
-_task_reload_srvtypes (gpointer p)
+_task_reload_srvtypes (gpointer p UNUSED)
 {
 	ADAPTIVE_PERIOD_DECLARE ();
-	(void) p;
 
 	CSURL(cs);
 	if (!cs)
@@ -505,47 +585,30 @@ _task_reload_srvtypes (gpointer p)
 	if (ADAPTIVE_PERIOD_SKIP ())
 		return;
 
-	GSList *list = NULL;
-	GError *err = conscience_remote_get_types (cs, &list);
+	gchar **types = NULL;
+	GError *err = conscience_remote_get_types (cs, &types);
+	EXTRA_ASSERT((err != NULL) ^ (types != NULL));
+
 	if (err != NULL) {
 		GRID_WARN ("SRVTYPES reload error [%s] from [%s] : (%d) %s",
-			nsname, cs, err->code, err->message);
+			ns_name, cs, err->code, err->message);
 		g_clear_error (&err);
 		return;
 	} else {
 		ADAPTIVE_PERIOD_ONSUCCESS(srvtypes_refresh_delay);
 		GRID_DEBUG("SRVTYPES reloaded %u for [%s]",
-				g_slist_length(list), nsname);
+				g_strv_length(types), ns_name);
 	}
 
-	gchar **newset = (gchar **) metautils_list_to_array (list);
-	g_slist_free (list);
-	list = NULL;
+	NSINFO_WRITE(gchar **tmp = srvtypes; srvtypes = types; types = tmp);
 
-	NSINFO_WRITE(gchar **tmp = srvtypes;
-	srvtypes = newset;
-	newset = tmp);
-
-	if (newset)
-		g_strfreev (newset);
+	if (types)
+		g_strfreev (types);
 }
 
 static void
-_task_expire_local (gpointer p)
+_task_push (gpointer p UNUSED)
 {
-	(void) p;
-	const gint64 now = oio_ext_monotonic_time ();
-	const gint64 oldest = OLDEST(now,cs_expire_local_services);
-	guint count;
-	PUSH_DO(count = lru_tree_remove_older (srv_registered, oldest));
-	if (count)
-		GRID_INFO("%u local services", count);
-}
-
-static void
-_task_push (gpointer p)
-{
-	(void) p;
 	struct lru_tree_s *lru = NULL;
 	GSList *tmp = NULL;
 	gboolean _list (gpointer k, gpointer v, gpointer u) {
@@ -554,7 +617,7 @@ _task_push (gpointer p)
 		return FALSE;
 	}
 
-	PUSH_DO(lru = push_queue; push_queue = _push_queue_create());
+	PUSH_WRITE(lru = push_queue; push_queue = _push_queue_create());
 	lru_tree_foreach(lru, _list, NULL);
 
 	if (!tmp) {
@@ -652,8 +715,8 @@ grid_main_get_options (void)
 			"synchronously and no cache is kept on a GET."},
 
 		{"LocalScores", OT_BOOL, {.b = &flag_local_scores},
-			"Enables the updates of the list of local services, so that\n"
-			"their score is displayed. This gives a pretty output but\n"
+			"Enables the updates of the list of local services, so that "
+			"their score is displayed. This gives a pretty output but "
 			"requires CPU spent in critical sections."},
 
 		{"DirLowTtl", OT_UINT, {.u = &dir_low_ttl},
@@ -664,6 +727,18 @@ grid_main_get_options (void)
 			"Directory 'high' (cs+meta0) TTL for cache elements"},
 		{"DirHighMax", OT_UINT, {.u = &dir_high_max},
 			"Directory 'high' (cs+meta0) MAX cached elements"},
+
+		{"PreferMaster", OT_BOOL, {.b = &flag_prefer_master_for_read},
+		        "Prefer to join the Master, even foor read-only"},
+		{"PreferSlave", OT_BOOL, {.b = &flag_prefer_slave_for_read},
+		        "Prefer to join a Slave, for read-only"},
+		{"PreferMasterForWrites", OT_BOOL, {.b = &flag_prefer_master_for_write},
+		        "Prefer to join a Master, for write operations"},
+		{"ForceMaster", OT_BOOL, {.b = &flag_force_master},
+		        "Force the service to redirect if not master "
+					"(even on read-only requests). Only effective on meta2 "
+					"requests."},
+
 		{NULL, 0, {.i = 0}, NULL}
 	};
 
@@ -706,9 +781,12 @@ grid_main_specific_fini (void)
 		path_parser_clean (path_parser);
 		path_parser = NULL;
 	}
-	if (lbpool) {
-		grid_lbpool_destroy (lbpool);
-		lbpool = NULL;
+	if (lb) {
+		oio_lb__clear(&lb);
+	}
+	if (lb_world) {
+		oio_lb_world__destroy(lb_world);
+		lb_world = NULL;
 	}
 	if (resolver) {
 		hc_resolver_destroy (resolver);
@@ -722,6 +800,10 @@ grid_main_specific_fini (void)
 		lru_tree_destroy (srv_known);
 		srv_known = NULL;
 	}
+	if (srv_master) {
+		lru_tree_destroy (srv_master);
+		srv_master = NULL;
+	}
 	if (push_queue) {
 		lru_tree_destroy (push_queue);
 		push_queue = NULL;
@@ -743,12 +825,14 @@ grid_main_specific_fini (void)
 		srv_registered = NULL;
 	}
 	namespace_info_clear (&nsinfo);
-	oio_str_clean (&nsname);
+	oio_str_clean (&ns_name);
 	g_rw_lock_clear(&nsinfo_rwlock);
-	g_mutex_clear(&push_mutex);
+	g_rw_lock_clear(&reg_rwlock);
+	g_rw_lock_clear(&push_rwlock);
 	g_rw_lock_clear(&csurl_rwlock);
 	g_rw_lock_clear(&srv_rwlock);
 	g_rw_lock_clear(&wanted_rwlock);
+	g_rw_lock_clear(&master_rwlock);
 
 	if (csurl) g_strfreev (csurl);
 	csurl = NULL;
@@ -777,10 +861,12 @@ configure_request_handlers (void)
 	SET("/cache/max/low/$COUNT/#POST", action_cache_set_max_low);
 	SET("/cache/max/high/$COUNT/#POST", action_cache_set_max_high);
 
-    // New routes
+	// New routes
 
 	// Load Balancing
+	SET("/$NS/lb/reload/#POST", action_lb_reload);
 	SET("/$NS/lb/choose/#GET", action_lb_choose);
+	SET("/$NS/lb/poll/#POST", action_lb_poll);
 
 	// Local services
 	SET ("/$NS/local/list/#GET", action_local_list);
@@ -794,49 +880,54 @@ configure_request_handlers (void)
 	SET("/$NS/conscience/lock/#POST", action_conscience_lock);
 	SET("/$NS/conscience/unlock/#POST", action_conscience_unlock);
 
-    // Directory
-    SET("/$NS/reference/create/#POST", action_ref_create);
-    SET("/$NS/reference/destroy/#POST", action_ref_destroy);
-    SET("/$NS/reference/show/#GET", action_ref_show);
-    SET("/$NS/reference/get_properties/#POST", action_ref_prop_get);
-    SET("/$NS/reference/set_properties/#POST", action_ref_prop_set);
-    SET("/$NS/reference/del_properties/#POST", action_ref_prop_del);
-    SET("/$NS/reference/link/#POST", action_ref_link);
-    SET("/$NS/reference/unlink/#POST", action_ref_unlink);
-    SET("/$NS/reference/force/#POST", action_ref_force);
-    SET("/$NS/reference/renew/#POST", action_ref_renew);
+	// Directory
+	SET("/$NS/reference/create/#POST", action_ref_create);
+	SET("/$NS/reference/destroy/#POST", action_ref_destroy);
+	SET("/$NS/reference/show/#GET", action_ref_show);
+	SET("/$NS/reference/get_properties/#POST", action_ref_prop_get);
+	SET("/$NS/reference/set_properties/#POST", action_ref_prop_set);
+	SET("/$NS/reference/del_properties/#POST", action_ref_prop_del);
+	SET("/$NS/reference/link/#POST", action_ref_link);
+	SET("/$NS/reference/unlink/#POST", action_ref_unlink);
+	SET("/$NS/reference/force/#POST", action_ref_force);
+	SET("/$NS/reference/renew/#POST", action_ref_renew);
 
-    // Meta2
-    // Container
-    SET("/$NS/container/create/#POST", action_container_create);
-    SET("/$NS/container/destroy/#POST", action_container_destroy);
-    SET("/$NS/container/show/#GET", action_container_show);
-    SET("/$NS/container/list/#GET", action_container_list);
-    SET("/$NS/container/get_properties/#POST", action_container_prop_get);
-    SET("/$NS/container/set_properties/#POST", action_container_prop_set);
-    SET("/$NS/container/del_properties/#POST", action_container_prop_del);
-    SET("/$NS/container/touch/#POST", action_container_touch);
-    SET("/$NS/container/dedup/#POST", action_container_dedup);
-    SET("/$NS/container/purge/#POST", action_container_purge);
-    SET("/$NS/container/flush/#POST", action_container_flush);
-    SET("/$NS/container/raw_insert/#POST", action_container_raw_insert);
-    SET("/$NS/container/raw_update/#POST", action_container_raw_update);
-    SET("/$NS/container/raw_delete/#POST", action_container_raw_delete);
+	// Meta2
+	// Container
+	SET("/$NS/container/create/#POST", action_container_create);
+	SET("/$NS/container/create_many/#POST", action_container_create_many);
+	SET("/$NS/container/destroy/#POST", action_container_destroy);
+	SET("/$NS/container/show/#GET", action_container_show);
+	SET("/$NS/container/list/#GET", action_container_list);
+	SET("/$NS/container/get_properties/#POST", action_container_prop_get);
+	SET("/$NS/container/set_properties/#POST", action_container_prop_set);
+	SET("/$NS/container/del_properties/#POST", action_container_prop_del);
+	SET("/$NS/container/touch/#POST", action_container_touch);
+	SET("/$NS/container/dedup/#POST", action_container_dedup);
+	SET("/$NS/container/purge/#POST", action_container_purge);
+	SET("/$NS/container/flush/#POST", action_container_flush);
+	SET("/$NS/container/raw_insert/#POST", action_container_raw_insert);
+	SET("/$NS/container/raw_update/#POST", action_container_raw_update);
+	SET("/$NS/container/raw_delete/#POST", action_container_raw_delete);
 
-    // Content
-    SET("/$NS/content/create/#POST", action_content_put);
-    SET("/$NS/content/link/#POST", action_content_link);
-    SET("/$NS/content/delete/#POST", action_content_delete);
-    SET("/$NS/content/show/#GET", action_content_show);
-    SET("/$NS/content/prepare/#POST", action_content_prepare);
-    SET("/$NS/content/get_properties/#POST", action_content_prop_get);
-    SET("/$NS/content/set_properties/#POST", action_content_prop_set);
-    SET("/$NS/content/del_properties/#POST", action_content_prop_del);
-    SET("/$NS/content/touch/#POST", action_content_touch);
-    SET("/$NS/content/spare/#POST", action_content_spare);
-    SET("/$NS/content/copy/#POST", action_content_copy);
+	// Content
+	SET("/$NS/content/create/#POST", action_content_put);
+	SET("/$NS/content/link/#POST", action_content_link);
+	SET("/$NS/content/delete/#POST", action_content_delete);
+	SET("/$NS/content/delete_many/#POST", action_content_delete_many);
+	SET("/$NS/content/show/#GET", action_content_show);
+	SET("/$NS/content/locate/#GET", action_content_show);
+	SET("/$NS/content/prepare/#POST", action_content_prepare);
+	SET("/$NS/content/get_properties/#POST", action_content_prop_get);
+	SET("/$NS/content/set_properties/#POST", action_content_prop_set);
+	SET("/$NS/content/del_properties/#POST", action_content_prop_del);
+	SET("/$NS/content/touch/#POST", action_content_touch);
+	SET("/$NS/content/spare/#POST", action_content_spare);
+	SET("/$NS/content/copy/#POST", action_content_copy);
+	SET("/$NS/content/update/#POST", action_content_update);
+	SET("/$NS/content/truncate/#POST", action_content_truncate);
 
-    // Admin
+	// Admin
 	SET("/$NS/admin/ping/#POST", action_admin_ping);
 	SET("/$NS/admin/info/#POST", action_admin_info);
 	SET("/$NS/admin/status/#POST", action_admin_status);
@@ -851,6 +942,9 @@ configure_request_handlers (void)
 	SET("/$NS/admin/freeze/#POST", action_admin_freeze);
 	SET("/$NS/admin/enable/#POST", action_admin_enable);
 	SET("/$NS/admin/disable/#POST", action_admin_disable);
+
+	SET("/$NS/admin/meta0_force/#POST", action_admin_meta0_force);
+	SET("/$NS/admin/meta0_list/#GET", action_admin_meta0_list);
 }
 
 static gboolean
@@ -865,12 +959,14 @@ grid_main_configure (int argc, char **argv)
 	const char *cfg_namespace = argv[1];
 
 	g_rw_lock_init (&csurl_rwlock);
-	g_mutex_init (&push_mutex);
+	g_rw_lock_init (&push_rwlock);
+	g_rw_lock_init (&reg_rwlock);
 	g_rw_lock_init (&nsinfo_rwlock);
 	g_rw_lock_init (&srv_rwlock);
 	g_rw_lock_init (&wanted_rwlock);
+	g_rw_lock_init (&master_rwlock);
 
-	nsname = g_strdup (cfg_namespace);
+	ns_name = g_strdup (cfg_namespace);
 	g_strlcpy (nsinfo.name, cfg_namespace, sizeof (nsinfo.name));
 	nsinfo.chunk_size = 1;
 
@@ -883,6 +979,13 @@ grid_main_configure (int argc, char **argv)
 	path_parser = path_parser_init ();
 	configure_request_handlers ();
 
+	/* Make the proxy avoid meta services with known problems */
+	oio_cache_avoid_on_error = oio_cfg_get_bool (
+			ns_name, OIO_CFG_AVOID_BADSRV, FALSE);
+	if (oio_cache_avoid_on_error)
+		GRID_NOTICE("Faulty peers avoidance: ENABLED");
+
+	/* init the networking capability of the processus */
 	server = network_server_init ();
 
 	/* ensure each Route as a pair of count/time stats */
@@ -895,9 +998,11 @@ grid_main_configure (int argc, char **argv)
 			gq_count_all, 0, gq_count_unexpected, 0,
 			gq_time_all, 0, gq_time_unexpected, 0);
 
-	lbpool = grid_lbpool_create (nsname);
+	lb_world = oio_lb_local__create_world();
+	lb = oio_lb__create();
 	srv_down = lru_tree_create((GCompareFunc)g_strcmp0, g_free, NULL, LTO_NOATIME);
 	srv_known = lru_tree_create((GCompareFunc)g_strcmp0, g_free, NULL, LTO_NOATIME);
+	srv_master = lru_tree_create((GCompareFunc)g_strcmp0, g_free, g_free, LTO_NOATIME);
 
 	resolver = hc_resolver_create ();
 	enum hc_resolver_flags_e f = 0;
@@ -930,7 +1035,7 @@ grid_main_configure (int argc, char **argv)
 	downstream_gtq = grid_task_queue_create ("downstream");
 
 	grid_task_queue_register (downstream_gtq, (guint) 1,
-		(GDestroyNotify) _task_reload_lbpool, NULL, NULL);
+		(GDestroyNotify) _task_reload_lb, NULL, NULL);
 
 	// Now prepare a queue for administrative tasks, such as cache expiration,
 	// configuration reloadings, etc.
@@ -938,9 +1043,10 @@ grid_main_configure (int argc, char **argv)
 
 	grid_task_queue_register (admin_gtq, 1,
 		(GDestroyNotify) _task_expire_services_known, NULL, NULL);
-
 	grid_task_queue_register (admin_gtq, 1,
 		(GDestroyNotify) _task_expire_services_down, NULL, NULL);
+	grid_task_queue_register (admin_gtq, 1,
+		(GDestroyNotify) _task_expire_services_master, NULL, NULL);
 
 	grid_task_queue_register (admin_gtq, 1,
 		(GDestroyNotify) _task_expire_resolver, NULL, NULL);
