@@ -85,6 +85,7 @@ dav_rawx_create_server_config(apr_pool_t *p, server_rec *s)
 	conf->hash_width = 3;
 	conf->fsync_on_close = FSYNC_ON_CHUNK_DIR;
 	conf->fallocate = 1;
+	conf->checksum_mode = CHECKSUM_ALWAYS;
 
 	return conf;
 }
@@ -106,6 +107,7 @@ dav_rawx_merge_server_config(apr_pool_t *p, void *base, void *overrides)
 	newconf->hash_width = child->hash_width;
 	newconf->fsync_on_close = child->fsync_on_close;
 	newconf->fallocate = child->fallocate;
+	newconf->checksum_mode = child->checksum_mode;
 	memcpy(newconf->docroot, child->docroot, sizeof(newconf->docroot));
 	memcpy(newconf->ns_name, child->ns_name, sizeof(newconf->ns_name));
 	update_rawx_conf(p, &(newconf->rawx_conf), newconf->ns_name);
@@ -151,7 +153,6 @@ dav_rawx_cmd_gridconfig_docroot(cmd_parms *cmd, void *config, const char *arg1)
 	dav_rawx_server_conf *conf;
 
 	(void) config;
-	DAV_XDEBUG_POOL(cmd->pool, 0, "%s()", __FUNCTION__);
 
 	/* Check the directory exists */
 	do {
@@ -187,10 +188,6 @@ _cleanup (void *p)
 		namespace_info_free (conf->ni);
 		conf->ni = NULL;
 	}
-	if (NULL != conf->sp) {
-		storage_policy_clean (conf->sp);
-		conf->sp = NULL;
-	}
 	return APR_SUCCESS;
 }
 
@@ -221,15 +218,6 @@ dav_rawx_cmd_gridconfig_namespace(cmd_parms *cmd, void *config, const char *arg1
 	conf->rawx_conf = apr_palloc(cmd->pool, sizeof(rawx_conf_t));
 	apr_pool_cleanup_register (cmd->pool, conf->rawx_conf, _cleanup, _cleanup);
 
-	gchar * stgpol = namespace_storage_policy(ns_info, ns_info->name);
-	if (NULL != stgpol) {
-		conf->rawx_conf->sp = storage_policy_init(ns_info, stgpol);
-		oio_str_clean (&stgpol);
-	} else {
-		conf->rawx_conf->sp = NULL;
-	}
-	g_assert (stgpol == NULL);
-
 	conf->rawx_conf->ni = ns_info;
 	conf->rawx_conf->acl = _get_acl(cmd->pool, ns_info);
 	conf->rawx_conf->last_update = oio_ext_monotonic_seconds();
@@ -239,19 +227,6 @@ dav_rawx_cmd_gridconfig_namespace(cmd_parms *cmd, void *config, const char *arg1
 	return NULL;
 }
 
-static int
-_str_to_boolean(const char *arg)
-{
-	int n_arg = 0;
-	n_arg |= (0 == apr_strnatcasecmp(arg, "on"));
-	n_arg |= (0 == apr_strnatcasecmp(arg, "true"));
-	n_arg |= (0 == apr_strnatcasecmp(arg, "yes"));
-	n_arg |= (0 == apr_strnatcasecmp(arg, "enabled"));
-	n_arg |= (0 == apr_strnatcasecmp(arg, "1"));
-
-	return n_arg;
-}
-
 static const char *
 dav_rawx_cmd_gridconfig_fsync(cmd_parms *cmd, void *config, const char *arg1)
 {
@@ -261,7 +236,7 @@ dav_rawx_cmd_gridconfig_fsync(cmd_parms *cmd, void *config, const char *arg1)
 	DAV_XDEBUG_POOL(cmd->pool, 0, "%s()", __FUNCTION__);
 
 	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
-	if (_str_to_boolean(arg1))
+	if (oio_str_parse_bool(arg1, FALSE))
 		conf->fsync_on_close |= FSYNC_ON_CHUNK;
 	else
 		conf->fsync_on_close &= ~FSYNC_ON_CHUNK;
@@ -278,7 +253,7 @@ dav_rawx_cmd_gridconfig_fsync_dir(cmd_parms *cmd, void *config, const char *arg1
 	DAV_XDEBUG_POOL(cmd->pool, 0, "%s()", __FUNCTION__);
 
 	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
-	if (_str_to_boolean(arg1))
+	if (oio_str_parse_bool(arg1, FALSE))
 		conf->fsync_on_close |= FSYNC_ON_CHUNK_DIR;
 	else
 		conf->fsync_on_close &= ~FSYNC_ON_CHUNK_DIR;
@@ -295,7 +270,7 @@ dav_rawx_cmd_gridconfig_fallocate(cmd_parms *cmd, void *config, const char *arg1
 	DAV_XDEBUG_POOL(cmd->pool, 0, "%s()", __FUNCTION__);
 
 	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
-	conf->fallocate = _str_to_boolean(arg1);
+	conf->fallocate = oio_str_parse_bool(arg1, FALSE);
 
 	return NULL;
 }
@@ -337,6 +312,46 @@ dav_rawx_cmd_gridconfig_acl(cmd_parms *cmd, void *config, const char *arg1)
 	return NULL;
 }
 
+static const char *
+dav_rawx_cmd_gridconfig_checksum(cmd_parms *cmd, void *config, const char *arg1)
+{
+	dav_rawx_server_conf *conf;
+	(void) config;
+
+	DAV_DEBUG_POOL(cmd->pool, 0, "%s(%s)", __FUNCTION__, arg1);
+
+	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
+	if (!oio_str_is_set(arg1)) {
+		conf->checksum_mode = CHECKSUM_ALWAYS;
+	} else if (0 == apr_strnatcasecmp(arg1, "smart")) {
+		conf->checksum_mode = CHECKSUM_SMART;
+	} else if (oio_str_parse_bool(arg1, TRUE)) {
+		conf->checksum_mode = CHECKSUM_ALWAYS;
+	} else {
+		conf->checksum_mode = CHECKSUM_NEVER;
+	}
+
+	return NULL;
+}
+
+static const char *
+dav_rawx_cmd_gridconfig_compression(cmd_parms *cmd, void *config, const char *arg1)
+{
+	dav_rawx_server_conf *conf;
+	(void) config;
+
+	DAV_XDEBUG_POOL(cmd->pool, 0, "%s()", __FUNCTION__);
+
+	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
+	if (apr_strnatcasecmp(arg1, "off") &&
+			apr_strnatcasecmp(arg1, "disabled") &&
+			apr_strnatcasecmp(arg1, "false") &&
+			apr_strnatcasecmp(arg1, "0"))
+		strncpy(conf->compression_algo, arg1, sizeof(conf->compression_algo));
+
+	return NULL;
+}
+
 static void
 rawx_hook_child_init(apr_pool_t *pchild, server_rec *s)
 {
@@ -353,14 +368,16 @@ rawx_hook_child_init(apr_pool_t *pchild, server_rec *s)
 
 	conf->cleanup = _cleanup_child;
 
-	gchar *event_agent_addr = gridcluster_get_eventagent(conf->ns_name);
-	GError *err = rawx_event_init(event_agent_addr);
-	if (NULL != err) {
-		DAV_ERROR_POOL(pchild, 0, "Failed to initialize event context: (%d) %s",
-				err->code, err->message);
-		g_clear_error (&err);
+	if (oio_cfg_get_bool(conf->ns_name, OIO_CFG_RAWX_EVENTS, TRUE)) {
+		gchar *event_agent_addr = oio_cfg_get_eventagent(conf->ns_name);
+		GError *err = rawx_event_init(event_agent_addr);
+		if (NULL != err) {
+			DAV_ERROR_POOL(pchild, 0, "Failed to initialize event context: (%d) %s",
+					err->code, err->message);
+			g_clear_error (&err);
+		}
+		g_free(event_agent_addr);
 	}
-	g_free(event_agent_addr);
 
 	oio_log_to_syslog ();
 }
@@ -542,6 +559,8 @@ static const command_rec dav_rawx_cmds[] =
     AP_INIT_TAKE1("grid_fsync_dir",   dav_rawx_cmd_gridconfig_fsync_dir,   NULL, RSRC_CONF, "do fsync on chunk direcory after renaming .pending"),
     AP_INIT_TAKE1("grid_fallocate",   dav_rawx_cmd_gridconfig_fallocate,   NULL, RSRC_CONF, "call fallocate when receiving a chunk"),
     AP_INIT_TAKE1("grid_acl",         dav_rawx_cmd_gridconfig_acl,         NULL, RSRC_CONF, "enabled acl"),
+    AP_INIT_TAKE1("grid_compression", dav_rawx_cmd_gridconfig_compression, NULL, RSRC_CONF, "enable compression ('zlib' or 'lzo')'"),
+    AP_INIT_TAKE1("grid_checksum",    dav_rawx_cmd_gridconfig_checksum,    NULL, RSRC_CONF, "enable checksuming the body of PUT ('yes', 'no', 'smart')'"),
     AP_INIT_TAKE1(NULL,  NULL,  NULL, RSRC_CONF, NULL)
 };
 
