@@ -1,7 +1,7 @@
 /*
 OpenIO SDS server
 Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO, modified as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -31,7 +31,9 @@ License along with this library.
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/resource.h>
+#include <poll.h>
 #include <sys/epoll.h>
+#include <sys/eventfd.h>
 #include <sys/un.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
@@ -40,7 +42,6 @@ License along with this library.
 #include <metautils/lib/metautils.h>
 
 #include "internals.h"
-#include "stats_holder.h"
 #include "network_server.h"
 #include "resolv.h"
 #include "slab.h"
@@ -55,7 +56,7 @@ static gboolean _endpoint_is_INET6 (struct endpoint_s *u);
 static gboolean _endpoint_is_INET4 (struct endpoint_s *u);
 static gboolean _endpoint_is_INET (struct endpoint_s *u);
 
-static GError * _endpoint_open (struct endpoint_s *u);
+static GError * _endpoint_open (struct endpoint_s *u, gboolean udp_allowed);
 static void _endpoint_close (struct endpoint_s *u);
 
 static struct network_client_s* _endpoint_accept_one(
@@ -82,6 +83,9 @@ static void _cb_worker(struct network_client_s *clt,
 static void _cb_stats(struct server_stat_msg_s *msg,
 		struct network_server_s *srv);
 
+static void _cb_ping(struct network_client_s *clt,
+		struct network_server_s *srv);
+
 /* Returns the number of max file descriptors for this process */
 static guint _server_get_maxfd(void);
 
@@ -206,14 +210,12 @@ network_server_stat_getall (struct network_server_s *srv)
 struct network_server_s *
 network_server_init(void)
 {
-	int wakeup[2] = {-1, -1};
-	if (0 > pipe(wakeup)) {
-		GRID_ERROR("PIPE creation failure : (%d) %s", errno, strerror(errno));
+	int efd = -1;
+	if ((efd = eventfd(0, EFD_NONBLOCK)) < 0) {
+		GRID_ERROR("eventfd creation failure: (%d) %s",
+				errno, strerror(errno));
 		return NULL;
 	}
-	shutdown(wakeup[0], SHUT_WR);
-	shutdown(wakeup[1], SHUT_RD);
-	fcntl(wakeup[0], F_SETFL, O_NONBLOCK|fcntl(wakeup[0], F_GETFL));
 
 	guint maxfd = _server_get_maxfd();
 
@@ -230,8 +232,7 @@ network_server_init(void)
 
 	result->cnx_max_sys = maxfd;
 	result->cnx_max = (result->cnx_max_sys * 99) / 100;
-	result->wakeup[0] = wakeup[0];
-	result->wakeup[1] = wakeup[1];
+	result->eventfd = efd;
 	result->epollfd = epoll_create(1024);
 
 	result->atexit_max_open_never_input = SERVER_DEFAULT_CNX_INACTIVE;
@@ -245,31 +246,52 @@ network_server_init(void)
 	result->gq_counter_cnx_accept = g_quark_from_static_string ("counter cnx.accept");
 	result->gq_counter_cnx_close =  g_quark_from_static_string ("counter cnx.close");
 
-	result->pool_stats =
-		g_thread_pool_new ((GFunc)_cb_stats, result, 1, TRUE, NULL);
-	result->pool_workers = g_thread_pool_new ((GFunc)_cb_worker, result,
+	result->pool_stats = g_thread_pool_new ((GFunc)_cb_stats, result,
+			1, TRUE, NULL);
+	result->pool_tcp = g_thread_pool_new ((GFunc)_cb_worker, result,
 			SERVER_DEFAULT_THP_MAXWORKERS, FALSE, NULL);
+	/* Even if UDP is eventually not allowed, a pool with shared threads
+	 * doesn't consume much resources. */
+	result->pool_udp = g_thread_pool_new ((GFunc)_cb_ping, result,
+			8, FALSE, NULL);
 	g_thread_pool_set_max_unused_threads (SERVER_DEFAULT_THP_MAXUNUSED);
 	g_thread_pool_set_max_idle_time (SERVER_DEFAULT_THP_IDLE);
 
-	GRID_DEBUG("SERVER ready with epollfd[%d] pipe[%d,%d]",
-			result->epollfd, result->wakeup[0], result->wakeup[1]);
+	GRID_DEBUG("SERVER ready with epollfd[%d] eventfd[%d]",
+			result->epollfd, result->eventfd);
 
 	return result;
 }
 
+void
+network_server_allow_udp(struct network_server_s *srv)
+{
+	g_assert(srv != NULL);
+
+	for (struct endpoint_s **pe = srv->endpointv; srv->endpointv && *pe; ++pe) {
+		GRID_ERROR("BUG: Can't call %s when servers are already open",
+				__FUNCTION__);
+		g_assert((*pe)->fd < 0);
+	}
+	srv->udp_allowed = TRUE;
+}
+
 static void
 _stop_pools (struct network_server_s *srv)
 {
 	g_thread_pool_stop_unused_threads ();
 
+	if (srv->pool_udp) {
+		g_thread_pool_free (srv->pool_udp, FALSE, TRUE);
+		srv->pool_udp = NULL;
+	}
 	if (srv->pool_stats) {
 		g_thread_pool_free (srv->pool_stats, FALSE, TRUE);
 		srv->pool_stats = NULL;
 	}
-	if (srv->pool_workers) {
-		g_thread_pool_free (srv->pool_workers, FALSE, TRUE);
-		srv->pool_workers = NULL;
+	if (srv->pool_tcp) {
+		g_thread_pool_free (srv->pool_tcp, FALSE, TRUE);
+		srv->pool_tcp = NULL;
 	}
 }
 
@@ -280,8 +302,10 @@ network_server_clean(struct network_server_s *srv)
 		return;
 
 	_stop_pools (srv);
-	if (srv->thread_events != NULL)
-		g_error("EventThread not joined!");
+	if (srv->thread_tcp != NULL)
+		g_error("Event thread not joined: %s", "tcp");
+	if (srv->thread_udp != NULL)
+		g_error("Event thread not joined: %s", "udp");
 
 	g_mutex_clear(&srv->lock_stats);
 	g_mutex_clear(&srv->lock_threads);
@@ -297,8 +321,7 @@ network_server_clean(struct network_server_s *srv)
 	if (srv->stats)
 		g_array_free (srv->stats, TRUE);
 
-	metautils_pclose(&(srv->wakeup[0]));
-	metautils_pclose(&(srv->wakeup[1]));
+	metautils_pclose(&(srv->eventfd));
 
 	if (srv->queue_monitor) {
 		g_async_queue_unref(srv->queue_monitor);
@@ -329,6 +352,7 @@ _srv_bind_host(struct network_server_s *srv, const gchar *url, gpointer u,
 	struct endpoint_s *e = g_malloc0(sizeof(*e) + 1 + len);
 	e->magic = MAGIC_ENDPOINT;
 	e->fd = -1;
+	e->fd_udp = -1;
 	e->flags = flags;
 	e->factory_udata = u;
 	e->factory_hook = factory;
@@ -370,6 +394,21 @@ network_server_bind_host_throughput(struct network_server_s *srv, const gchar *u
 	_srv_bind_host(srv, url, u, factory, NETSERVER_THROUGHPUT);
 }
 
+gchar**
+network_server_endpoints (struct network_server_s *srv)
+{
+	g_assert_nonnull(srv);
+	GPtrArray *tmp = g_ptr_array_new();
+	for (struct endpoint_s **pe = srv->endpointv; srv->endpointv && *pe; ++pe) {
+		if ((*pe)->fd < 0)
+			continue;
+		g_ptr_array_add(tmp,
+				g_strdup_printf("%s:%d", (*pe)->url, (*pe)->port_real));
+	}
+	g_ptr_array_add (tmp, NULL);
+	return (gchar**) g_ptr_array_free(tmp, FALSE);
+}
+
 void
 network_server_close_servers(struct network_server_s *srv)
 {
@@ -381,20 +420,18 @@ network_server_close_servers(struct network_server_s *srv)
 GError *
 network_server_open_servers(struct network_server_s *srv)
 {
-	struct endpoint_s **u;
+	g_assert(srv != NULL);
 
-	EXTRA_ASSERT(srv != NULL);
-
-	for (u=srv->endpointv; u && *u ;u++) {
+	for (struct endpoint_s **u = srv->endpointv; srv->endpointv && *u; u++) {
 		GError *err;
-		if (NULL != (err = _endpoint_open(*u))) {
+		if (NULL != (err = _endpoint_open(*u, srv->udp_allowed))) {
 			g_prefix_error(&err, "url open error : ");
 			network_server_close_servers(srv);
 			return err;
 		}
 	}
 
-	for (u=srv->endpointv; u && *u ;u++) {
+	for (struct endpoint_s **u = srv->endpointv; srv->endpointv && *u; u++) {
 		GRID_DEBUG("fd=%d port=%d endpoint=%s ready", (*u)->fd,
 				(*u)->port_real, (*u)->url);
 	}
@@ -403,10 +440,16 @@ network_server_open_servers(struct network_server_s *srv)
 }
 
 static void
-_drain(int fd)
+_drain_eventfd(int fd)
 {
-	char buff[2048];
-	while (0 < read(fd, buff, sizeof(buff))) {}
+	guint64 event_count = 0u;
+	int rc = metautils_syscall_read(fd, &event_count, 8);
+	if (rc < 0 && errno != EAGAIN) {  // EAGAIN -> counter is 0
+		GRID_WARN("Failed to reset event counter: (%d) %s",
+				errno, strerror(errno));
+	} else if (event_count > 100u) {
+		GRID_INFO("Burst of %"G_GUINT64_FORMAT" events", event_count);
+	}
 }
 
 static const char *
@@ -428,13 +471,13 @@ static void
 ARM_WAKER(struct network_server_s *srv, int how)
 {
 	struct epoll_event ev;
-	ev.data.ptr = srv->wakeup;
+	ev.data.ptr = &(srv->eventfd);
 	ev.events = EPOLLIN|EPOLLET|EPOLLONESHOT;
 
-	if (0 == epoll_ctl(srv->epollfd, how, srv->wakeup[0], &ev))
+	if (0 == epoll_ctl(srv->epollfd, how, srv->eventfd, &ev))
 		return;
 	GRID_DEBUG("WUP epoll_ctl(%d,%d,%s) = (%d) %s", srv->epollfd,
-			srv->wakeup[0], epoll2str(how), errno, strerror(errno));
+			srv->eventfd, epoll2str(how), errno, strerror(errno));
 }
 
 static void
@@ -488,7 +531,7 @@ _manage_client_event(struct network_server_s *srv,
 
 	if (clt->events & CLT_ERROR)
 		ARM_CLIENT(srv, clt, EPOLL_CTL_DEL);
-	g_thread_pool_push(srv->pool_workers, clt, NULL);
+	g_thread_pool_push(srv->pool_tcp, clt, NULL);
 }
 
 static void
@@ -516,7 +559,7 @@ _manage_events(struct network_server_s *srv)
 	if (erc > 0) {
 		while (erc-- > 0) {
 			pev = allev+erc;
-			if (pev->data.ptr == srv->wakeup)
+			if (pev->data.ptr == &(srv->eventfd))
 				continue;
 			if (MAGIC_ENDPOINT == *((unsigned int*)(pev->data.ptr)))
 				_manage_endpoint_event (srv, pev->data.ptr);
@@ -525,7 +568,7 @@ _manage_events(struct network_server_s *srv)
 		}
 	}
 
-	_drain(srv->wakeup[0]);
+	_drain_eventfd(srv->eventfd);
 	ARM_WAKER(srv, EPOLL_CTL_MOD);
 	struct network_client_s *clt;
 	while (NULL != (clt = g_async_queue_try_pop(srv->queue_monitor))) {
@@ -582,7 +625,7 @@ _thread_cb_events(gpointer d)
 		}
 	}
 
-	/* XXX the server connections are being closed in the main thread that
+	/* the server connections are being closed in the main thread that
 	 * received the exit signal. They will be removed automatically from
 	 * the epoll pool.*/
 
@@ -603,6 +646,163 @@ _thread_cb_events(gpointer d)
 	return d;
 }
 
+static gsize
+_endpoint_count_all (struct endpoint_s **pu)
+{
+	gsize count = 0;
+	for (; *pu ;++pu) { count ++; }
+	return count;
+}
+
+static gsize
+_endpoint_count_udp (struct endpoint_s **pu)
+{
+	gsize count = 0;
+	for (; *pu ;++pu) { if ((*pu)->fd_udp > 0) { count ++; } }
+	return count;
+}
+
+static void
+_endpoint_monitor_udp (struct endpoint_s **pu, struct pollfd *pfd)
+{
+	for (gint i=0; pu[i] ;++i) {
+		pfd[i].fd = pu[i]->fd_udp;
+		pfd[i].events = pu[i]->fd_udp > 0 ? POLLIN : 0;
+		pfd[i].revents = 0;
+	}
+}
+
+static void
+_cb_ping(struct network_client_s *clt, struct network_server_s *srv)
+{
+	EXTRA_ASSERT(clt != NULL);
+	EXTRA_ASSERT(clt->server == srv);
+
+	if (!srv->flag_continue) {
+		GRID_TRACE("PING %s -> %s discarded (server stopping)",
+				clt->peer_name, clt->local_name);
+	} else {
+
+		const gint64 now = oio_ext_monotonic_time();
+
+		/* OIO_SERVER_UDP_QUEUE_MAXAGE is arbitrary but it avoids managing
+		 * pings that have probably been retried by the emitter. */
+		if (now - clt->time.evt_in > OIO_SERVER_UDP_QUEUE_MAXAGE) {
+			GRID_DEBUG("PING %s -> %s queued for too long",
+					clt->peer_name, clt->local_name);
+		} else {
+			int rc = clt->transport.notify_input(clt);
+			if (rc != RC_PROCESSED) {
+				GRID_DEBUG("PING %s -> %s processing error",
+						clt->peer_name, clt->local_name);
+			}
+		}
+	}
+	_client_clean(srv, clt);
+}
+
+static void
+_manage_ping_event(struct network_server_s *srv, struct endpoint_s *e,
+		struct pollfd *pfd)
+{
+	/* destined for little notifications, there is currently no clue ping
+	 * are bigger than few 100's of bytes. 1k is enough. */
+	guint8 buf[1024];
+
+	/* consume several messages, but not indefinitely to avoid starvations
+	 * with other ping sockets */
+	for (gint i=0; i<8 ;++i) {
+		struct sockaddr_storage ss;
+		socklen_t ss_len = sizeof(ss);
+		ssize_t r = recvfrom(pfd->fd, buf, sizeof(buf), 0,
+				(struct sockaddr*)&ss, &ss_len);
+		if (r <= 0)
+			break;
+
+		/* fake a client, the transport needs it */
+		struct network_client_s *clt = SLICE_NEW0(struct network_client_s);
+		clt->server = srv;
+		clt->fd = -1;
+		clt->events = CLT_READ;
+		clt->time.cnx = oio_ext_monotonic_time ();
+		clt->time.evt_in = clt->time.cnx;
+		grid_sockaddr_to_string((struct sockaddr*)&ss,
+				clt->peer_name, sizeof(clt->peer_name));
+		g_snprintf(clt->local_name, sizeof(clt->local_name), "%s:%d",
+				e->url, e->port_real);
+
+		if (e->factory_hook)
+			e->factory_hook(e->factory_udata, clt);
+
+		/* Insert a slab in the input queue */
+		data_slab_sequence_append(&clt->input,
+				data_slab_make_buffer(g_memdup(buf, r), r));
+
+		/* notify the transport layer, and manage this in another thread */
+		if (NULL != clt->transport.notify_input) {
+			GError *err = NULL;
+
+			/* OIO_SERVER_UDP_QUEUE_MAXLEN is arbitrary, but it is only used to
+			 * avoid a memory leak */
+			const guint unprocessed = g_thread_pool_unprocessed(srv->pool_udp);
+			if (unprocessed > OIO_SERVER_UDP_QUEUE_MAXLEN) {
+				GRID_DEBUG("PING %s -> %s (%"G_GSIZE_FORMAT") dropped",
+						clt->peer_name, clt->local_name,
+						data_slab_sequence_size(&clt->input));
+			} else {
+				if (!g_thread_pool_push(srv->pool_udp, clt, &err)) {
+					GRID_WARN("PING %s -> %s (%"G_GSIZE_FORMAT") discarded: (%d) %s",
+							clt->peer_name, clt->local_name,
+							data_slab_sequence_size(&clt->input),
+							err->code, err->message);
+					_client_clean(srv, clt);
+				} else {
+					GRID_TRACE("PING %s -> %s (%"G_GSIZE_FORMAT") defered",
+							clt->peer_name, clt->local_name,
+							data_slab_sequence_size(&clt->input));
+				}
+			}
+		} else {
+			GRID_DEBUG("PING %s -> %s (%"G_GSIZE_FORMAT") discarded: %s",
+					clt->peer_name, clt->local_name,
+					data_slab_sequence_size(&clt->input),
+					"no transport");
+			_client_clean(srv, clt);
+		}
+	}
+}
+
+static gpointer
+_thread_cb_ping(gpointer d)
+{
+	metautils_ignore_signals();
+
+	struct network_server_s *srv = d;
+
+	/* no server open, no need to continue */
+	if (_endpoint_count_udp(srv->endpointv) <= 0)
+		return d;
+
+	const gsize count_structs = _endpoint_count_all(srv->endpointv);
+	struct pollfd pfd[count_structs];
+	_endpoint_monitor_udp(srv->endpointv, pfd);
+
+	while (srv->flag_continue) {
+		int rc = metautils_syscall_poll(pfd, count_structs, 1000);
+		if (rc < 0) {
+			GRID_WARN("PING poll error (%d) %s", errno, strerror(errno));
+		} else if (rc > 0) {
+			for (guint i=0; i<count_structs ;++i) {
+				if (pfd[i].revents & POLLIN)
+					_manage_ping_event(srv, srv->endpointv[i], pfd+i);
+				pfd[i].revents = 0;
+			}
+		}
+	}
+
+	return d;
+}
+
 GError *
 network_server_run(struct network_server_s *srv)
 {
@@ -627,7 +827,9 @@ network_server_run(struct network_server_s *srv)
 		ARM_ENDPOINT(srv, u, EPOLL_CTL_ADD);
 	ARM_WAKER(srv, EPOLL_CTL_ADD);
 
-	srv->thread_events = g_thread_new("events", _thread_cb_events, srv);
+	if (srv->udp_allowed)
+		srv->thread_udp = g_thread_new("udp", _thread_cb_ping, srv);
+	srv->thread_tcp = g_thread_new("tcp", _thread_cb_events, srv);
 
 	network_server_stat_push2 (srv, FALSE,
 			srv->gq_gauge_cnx_max, srv->cnx_max,
@@ -636,7 +838,7 @@ network_server_run(struct network_server_s *srv)
 	while (srv->flag_continue) {
 		g_usleep(1 * G_TIME_SPAN_SECOND);
 		network_server_stat_push4 (srv, FALSE,
-				srv->gq_gauge_threads, (guint64) g_thread_pool_get_num_threads(srv->pool_workers),
+				srv->gq_gauge_threads, (guint64) g_thread_pool_get_num_threads(srv->pool_tcp),
 				srv->gq_gauge_cnx_current, srv->cnx_clients,
 				srv->gq_counter_cnx_accept, srv->cnx_accept,
 				srv->gq_counter_cnx_close, srv->cnx_close);
@@ -645,10 +847,14 @@ network_server_run(struct network_server_s *srv)
 	network_server_close_servers(srv);
 	GRID_DEBUG("Server %p waiting for its threads", srv);
 
-	/* wait for the first event thread */
-	if (srv->thread_events) {
-		g_thread_join(srv->thread_events);
-		srv->thread_events = NULL;
+	/* wait for the event threads */
+	if (srv->thread_tcp) {
+		g_thread_join(srv->thread_tcp);
+		srv->thread_tcp = NULL;
+	}
+	if (srv->thread_udp) {
+		g_thread_join(srv->thread_udp);
+		srv->thread_udp = NULL;
 	}
 
 	/* XXX(jfs): seems legit but requires exit critical path to be reviewed.
@@ -690,17 +896,18 @@ _endpoint_close (struct endpoint_s *u)
 			(void) unlink (u->url);
 		metautils_pclose(&(u->fd));
 	}
+	if (u->fd_udp >= 0)
+		metautils_pclose(&(u->fd_udp));
 	u->port_real = 0;
 }
 
 static GError *
-_endpoint_open(struct endpoint_s *u)
+_endpoint_open(struct endpoint_s *u, gboolean udp_allowed)
 {
 	EXTRA_ASSERT(u != NULL);
 
-	struct sockaddr_storage ss;
+	struct sockaddr_storage ss = {0};
 	socklen_t ss_len = sizeof(ss);
-	memset(&ss, 0, sizeof(ss));
 
 	/* patch some socket preferences that make sense only for INET sockets */
 	if (_endpoint_is_UNIX(u)) {
@@ -711,16 +918,28 @@ _endpoint_open(struct endpoint_s *u)
 
 	/* Get a socket of the right type */
 	if (_endpoint_is_UNIX(u))
-		u->fd = socket(AF_UNIX, SOCK_STREAM, 0);
-	else if (_endpoint_is_INET6(u))
-		u->fd = socket(AF_INET6, SOCK_STREAM, 0);
-	else
-		u->fd = socket(AF_INET, SOCK_STREAM, 0);
+		u->fd = socket_nonblock(AF_UNIX, SOCK_STREAM, 0);
+	else {
+		if (_endpoint_is_INET6(u)) {
+			u->fd = socket_nonblock(AF_INET6, SOCK_STREAM, 0);
+			if (udp_allowed)
+				u->fd_udp = socket_nonblock(AF_INET6, SOCK_DGRAM, 0);
+		} else {
+			u->fd = socket_nonblock(AF_INET, SOCK_STREAM, 0);
+			if (udp_allowed)
+				u->fd_udp = socket_nonblock(AF_INET, SOCK_DGRAM, 0);
+		}
+		if (udp_allowed && u->fd_udp < 0)
+			return NEWERROR(errno, "socket(udp) = '%s'", strerror(errno));
+	}
 	if (u->fd < 0)
-		return NEWERROR(errno, "socket() = '%s'", strerror(errno));
+		return NEWERROR(errno, "socket(tcp) = '%s'", strerror(errno));
 
-	if (_endpoint_is_INET(u))
+	if (_endpoint_is_INET(u)) {
 		sock_set_reuseaddr (u->fd, TRUE);
+		if (u->fd_udp >= 0)
+			sock_set_reuseaddr (u->fd_udp, TRUE);
+	}
 
 	/* Bind the socket the right way according to its type */
 	if (_endpoint_is_UNIX(u)) {
@@ -741,12 +960,17 @@ _endpoint_open(struct endpoint_s *u)
 		s4->sin_port = htons(u->port_cfg);
 		inet_pton(AF_INET, u->url, &(s4->sin_addr));
 	}
+
 	if (0 > bind(u->fd, (struct sockaddr*)&ss, ss_len)) {
 		int errsave = errno;
 		u->port_real = 0;
 		if (_endpoint_is_UNIX(u))
 			metautils_pclose (&u->fd);
-		return NEWERROR(errsave, "bind(%s) = '%s'", u->url, strerror(errsave));
+		return NEWERROR(errsave, "bind(tcp,%s) = '%s'", u->url, strerror(errsave));
+	}
+	if (u->fd_udp >= 0 && 0 > bind(u->fd_udp, (struct sockaddr*)&ss, ss_len)) {
+		int errsave = errno;
+		return NEWERROR(errsave, "bind(udp,%s) = '%s'", u->url, strerror(errsave));
 	}
 
 	/* for INET sockets, get the port really used */
@@ -760,9 +984,6 @@ _endpoint_open(struct endpoint_s *u)
 			u->port_real = ntohs(((struct sockaddr_in6*)&ss)->sin6_port);
 	}
 
-	/* And finally set the mandatory fags. */
-	sock_set_non_blocking(u->fd, TRUE);
-
 	if (0 > listen(u->fd, 32768))
 		return NEWERROR(errno, "listen() = '%s'", strerror(errno));
 
@@ -813,9 +1034,11 @@ retry:
 		case EXCESS_NONE:
 			break;
 		case EXCESS_HARD:
+			SLICE_FREE(struct network_client_s, clt);
 			metautils_pclose(&fd);
 			_cnx_notify_close(srv);
-			GRID_WARN("Too many clients!");
+			GRID_WARN("Too many inbound connections! (cnx_max=%u)",
+					srv->cnx_max);
 			return NULL;
 	}
 
@@ -841,9 +1064,9 @@ void
 network_server_set_max_workers(struct network_server_s *srv, guint max)
 {
 	EXTRA_ASSERT(srv != NULL);
-	if (!grid_main_is_running() || !srv->pool_workers)
+	if (!grid_main_is_running() || !srv->pool_tcp)
 		return;
-	g_thread_pool_set_max_threads (srv->pool_workers, CLAMP(max, 1, G_MAXUINT16), NULL);
+	g_thread_pool_set_max_threads (srv->pool_tcp, CLAMP(max, 1, G_MAXUINT16), NULL);
 }
 
 void
@@ -912,9 +1135,11 @@ _cb_worker(struct network_client_s *clt, struct network_server_s *srv)
 	}
 	else {
 		g_async_queue_push(srv->queue_monitor, clt);
-		ssize_t w = write(srv->wakeup[1], "", 1);
-		if (w != 1) {
-			GRID_DEBUG("Server: Event thread notification failed");
+		guint64 evt_count = 1u;
+		ssize_t w = write(srv->eventfd, &evt_count, 8);
+		if (w != 8) {
+			GRID_WARN("event thread notification failed: (%d) %s",
+					errno, strerror(errno));
 		}
 	}
 }
@@ -925,7 +1150,7 @@ _server_get_maxfd(void)
 	struct rlimit limit;
 
 	if (0 != getrlimit(RLIMIT_NOFILE, &limit)) {
-		GRID_WARN("getrlimit() error : (%d) %s", errno, strerror(errno));
+		GRID_WARN("getrlimit() error: (%d) %s", errno, strerror(errno));
 		return 512;
 	}
 	else {
@@ -963,6 +1188,7 @@ _client_add_to_monitored(struct network_server_s *srv,
 	EXTRA_ASSERT(clt->server == srv);
 	EXTRA_ASSERT(clt->prev == NULL);
 	EXTRA_ASSERT(clt->next == NULL);
+	EXTRA_ASSERT(clt->fd >= 0);
 
 	if (NULL != (clt->next = srv->first))
 		clt->next->prev = clt;
@@ -989,18 +1215,17 @@ _client_ready_for_output(struct network_client_s *clt)
 static void
 _client_clean(struct network_server_s *srv, struct network_client_s *clt)
 {
-	(void) srv;
-
 	/* Notifies the upper layer the client is being exiting. */
 	if (clt->transport.notify_error)
 		clt->transport.notify_error(clt);
 
 	EXTRA_ASSERT(clt->prev == NULL);
 	EXTRA_ASSERT(clt->next == NULL);
-	EXTRA_ASSERT(clt->fd >= 0);
 
-	metautils_pclose(&(clt->fd));
-	_cnx_notify_close(srv);
+	if (clt->fd >= 0) {
+		metautils_pclose(&(clt->fd));
+		_cnx_notify_close(srv);
+	}
 
 	clt->flags = clt->events = 0;
 	memset(&(clt->time), 0, sizeof(clt->time));
@@ -1160,6 +1385,7 @@ _client_manage_event(struct network_client_s *clt, int events)
 					break;
 				case RC_NODATA:
 					clt->flags |= NETCLIENT_IN_CLOSED;
+					// FALLTHROUGH
 				case RC_NOTREADY:
 					/* no need to loop again */
 					rcI = 0;
@@ -1183,8 +1409,9 @@ network_client_send_slab(struct network_client_s *client, struct data_slab_s *ds
 	client->time.evt_out = oio_ext_monotonic_time ();
 
 	if (!_client_ready_for_output(client)) {
-		register int type = ds->type;
-		GRID_DEBUG("fd=%d Discarding data, output closed", client->fd);
+		const int type = ds->type;
+		GRID_TRACE("fd=%d/%s discarding data, output closed",
+				client->fd, client->peer_name);
 		data_slab_free(ds);
 		return MACRO_COND(type == STYPE_EOF, 0, -1);
 	}
@@ -1234,7 +1461,6 @@ void
 network_client_allow_input(struct network_client_s *clt, gboolean v)
 {
 	EXTRA_ASSERT(clt != NULL);
-	EXTRA_ASSERT(clt->fd >= 0);
 
 	if (!clt || clt->fd < 0)
 		return;
@@ -1249,3 +1475,14 @@ network_client_allow_input(struct network_client_s *clt, gboolean v)
 	}
 }
 
+int
+network_server_first_udp (struct network_server_s *srv)
+{
+	if (!srv || !srv->udp_allowed || !srv->endpointv)
+		return -1;
+	for (struct endpoint_s **pe=srv->endpointv; *pe ;++pe) {
+		if ((*pe)->fd_udp >= 0)
+			return (*pe)->fd_udp;
+	}
+	return -1;
+}
