@@ -24,10 +24,9 @@ License along with this library.
 #include <malloc.h>
 
 #include <metautils/lib/metautils.h>
-#include <metautils/lib/metacomm.h>
+#include <core/internals.h>
 
 #include "internals.h"
-#include "stats_holder.h"
 #include "network_server.h"
 #include "transport_gridd.h"
 
@@ -121,7 +120,7 @@ transport_gridd_dispatcher_add_requests(
 	if (!descr)
 		return NEWERROR(EINVAL, "Invalid request descriptor");
 
-	for (d=descr; d && d->name && d->handler ;d++) {
+	for (d = descr; descr && d->name && d->handler; d++) {
 		struct hashstr_s *hname;
 		struct gridd_request_handler_s *handler;
 
@@ -177,7 +176,6 @@ transport_gridd_factory0(struct gridd_request_dispatcher_s *dispatcher,
 {
 	EXTRA_ASSERT(dispatcher != NULL);
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->fd >= 0);
 
 	struct transport_client_context_s *transport_context = g_malloc0(sizeof(*transport_context));
 	transport_context->dispatcher = dispatcher;
@@ -240,7 +238,7 @@ network_client_log_access(struct log_item_s *item)
 		g_string_append(gstr, ensure(r->subject));
 	}
 
-	g_log("access", GRID_LOGLVL_INFO, "%s", gstr->str);
+	INCOMING("%s", gstr->str);
 	g_string_free(gstr, TRUE);
 }
 
@@ -435,7 +433,6 @@ transport_gridd_notify_input(struct network_client_s *clt)
 	struct transport_client_context_s *ctx;
 
 	EXTRA_ASSERT(clt != NULL);
-	EXTRA_ASSERT(clt->fd >= 0);
 
 	ctx = clt->transport.client_context;
 	/* read the data */
@@ -889,6 +886,17 @@ dispatch_KILL(struct gridd_reply_ctx_s *reply,
 	return TRUE;
 }
 
+static gboolean
+dispatch_REDIRECT(struct gridd_reply_ctx_s *reply,
+		gpointer gdata, gpointer hdata)
+{
+	(void) gdata, (void) hdata;
+	gchar **endpoints = network_server_endpoints(reply->client->server);
+	reply->send_error(0, NEWERROR(CODE_REDIRECT, "%s", endpoints[0]));
+	g_strfreev(endpoints);
+	return TRUE;
+}
+
 #define VOLPREFIX "config volume="
 
 static gboolean
@@ -927,7 +935,7 @@ dispatch_VERSION(struct gridd_reply_ctx_s *reply,
 {
 	(void) gdata, (void) hdata;
 	reply->no_access();
-	reply->add_body(metautils_gba_from_string(OIOSDS_API_VERSION));
+	reply->add_body(metautils_gba_from_string(OIOSDS_PROJECT_VERSION));
 	reply->send_reply(CODE_FINAL_OK, "OK");
 	return TRUE;
 }
@@ -942,6 +950,7 @@ gridd_get_common_requests(void)
 		{"REQ_VERSION",   dispatch_VERSION,       NULL},
 		{"REQ_HANDLERS",  dispatch_LISTHANDLERS,  NULL},
 		{"REQ_KILL",      dispatch_KILL,          NULL},
+		{"REQ_REDIRECT",  dispatch_REDIRECT,      NULL},
 		{NULL, NULL, NULL}
 	};
 
