@@ -218,14 +218,14 @@ load_table_row(sqlite3 *db, const hashstr_t *name, gint64 rowid, Row_t *row,
 {
 	int rc;
 	sqlite3_stmt *stmt = NULL;
-	gchar *sql;
+	gchar sql[128] = {0};
 
 	GRID_TRACE2("%s(%p,%s,%"G_GINT64_FORMAT",%p,%p)", __FUNCTION__,
 			db, hashstr_str(name), rowid, row, table);
 
-	sql = g_strdup_printf("SELECT * FROM %s WHERE ROWID = ?", hashstr_str(name));
+	g_snprintf(sql, sizeof(sql), "SELECT * FROM %s WHERE ROWID = ?",
+			hashstr_str(name));
 	sqlite3_prepare_debug(rc, db, sql, -1, &stmt, NULL);
-	g_free(sql);
 
 	sqlite3_bind_int64(stmt, 1, rowid);
 	while (SQLITE_ROW == (rc = sqlite3_step(stmt)))
@@ -240,7 +240,7 @@ context_pending_inc_versions(struct sqlx_repctx_s *ctx)
 	gboolean _on_table(hashstr_t *k, GTree *rows, gpointer u0) {
 		(void) u0;
 		if (g_tree_nnodes(rows) > 0) {
-			if (0 != g_ascii_strcasecmp(hashstr_str(k), "main.admin")) {
+			if (0 != strcmp(hashstr_str(k), "main.admin")) {
 				gsize max = sizeof("version:")+hashstr_len(k);
 				gchar buf[max];
 				g_snprintf(buf, max, "version:%s", hashstr_str(k));
@@ -313,12 +313,13 @@ _replicate_on_peers(gchar **peers, struct sqlx_repctx_s *ctx)
 		gridd_client_create_many(peers, encoded, NULL, NULL);
 	g_byte_array_unref(encoded);
 
+	gridd_clients_set_timeout_cnx(clients, SQLX_REPLI_TIMEOUT_CNX);
 	gridd_clients_set_timeout(clients, SQLX_REPLI_TIMEOUT);
 
 	gridd_clients_start(clients);
 	GError *err = gridd_clients_loop(clients);
 	if (!err) {
-		for (struct gridd_client_s **pc=clients; pc && *pc ;pc++) {
+		for (struct gridd_client_s **pc = clients; clients && *pc; pc++) {
 			GError *e = gridd_client_error(*pc);
 			if (!e)
 				++ count_success;
@@ -373,7 +374,7 @@ _defer_synchronous_RESYNC(struct sqlx_repctx_s *ctx)
 
 	if (err != NULL) {
 		GRID_WARN("Replicated transaction started but peers not found "
-				"[%s][%s] : (%d) %s", ctx->sq3->name.base, ctx->sq3->name.type,
+				"[%s][%s]: (%d) %s", ctx->sq3->name.base, ctx->sq3->name.type,
 				err->code, err->message);
 		g_clear_error(&err);
 		return;
@@ -396,7 +397,7 @@ _perform_REPLICATE(struct sqlx_repctx_s *ctx)
 
 	if (err != NULL) {
 		GRID_WARN("Replicated transaction started but peers not found "
-				"[%s][%s] : (%d) %s", ctx->sq3->name.base,
+				"[%s][%s]: (%d) %s", ctx->sq3->name.base,
 				ctx->sq3->name.type, err->code, err->message);
 		g_clear_error(&err);
 		return 1;
@@ -414,7 +415,7 @@ _perform_REPLICATE(struct sqlx_repctx_s *ctx)
 	if (!err)
 		return 0;
 
-	GRID_WARN("%s(%p) FAILED : (%d) %s", __FUNCTION__, ctx, err->code,
+	GRID_WARN("%s(%p) FAILED: (%d) %s", __FUNCTION__, ctx, err->code,
 			err->message);
 	g_error_free(err);
 	ctx->any_change = 0;
@@ -490,7 +491,7 @@ sqlx_synchronous_resync(struct sqlx_repctx_s *ctx, gchar **peers)
 	// Generate the DUMP
 	err = sqlx_repository_dump_base_gba(ctx->sq3, &dump);
 	if (NULL != err) {
-		GRID_WARN("[%s][%s] Synchronous COMMIT not possible : (%d) %s",
+		GRID_WARN("[%s][%s] Synchronous COMMIT not possible: (%d) %s",
 				ctx->sq3->name.base, ctx->sq3->name.type,
 				err->code, err->message);
 		g_clear_error(&err);
@@ -499,7 +500,8 @@ sqlx_synchronous_resync(struct sqlx_repctx_s *ctx, gchar **peers)
 
 	// Now send it to the SLAVES
 	peers_restore(peers, sqlx_name_mutable_to_const(&ctx->sq3->name), dump);
-	GRID_INFO("RESTORED on SLAVES [%s][%s]", ctx->sq3->name.base, ctx->sq3->name.type);
+	GRID_INFO("RESTORED on SLAVES [%s][%s]", ctx->sq3->name.base,
+			ctx->sq3->name.type);
 }
 
 static void
@@ -676,7 +678,8 @@ sqlx_transaction_end(struct sqlx_repctx_s *ctx, GError *err)
 		}
 		else {
 			if (ctx->errors->len > 0) {
-				GRID_WARN("COMMIT errors on [%s.%s]:%s", ctx->sq3->name.base, ctx->sq3->name.type, ctx->errors->str);
+				GRID_WARN("COMMIT errors on [%s.%s]:%s", ctx->sq3->name.base,
+						ctx->sq3->name.type, ctx->errors->str);
 			}
 			if (ctx->resync_todo && ctx->resync_todo->len) {
 				// Detected the need of an explicit RESYNC on some SLAVES.
