@@ -1,7 +1,7 @@
 /*
 OpenIO SDS sqlx
 Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -30,7 +30,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <events/oio_events_queue_zmq.h>
 #include <events/oio_events_queue_beanstalkd.h>
 #include <server/network_server.h>
-#include <server/stats_holder.h>
 #include <server/internals.h>
 #include <server/transport_gridd.h>
 #include <sqliterepo/sqlx_macros.h>
@@ -43,6 +42,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <sqliterepo/internals.h>
 #include <resolver/hc_resolver.h>
 
+#include <core/oiolb.h>
+
 #include <glib.h>
 
 #include "sqlx_service.h"
@@ -51,6 +52,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 # define SQLX_MAX_TIMER_PER_ROUND 100
 #endif
 
+#ifndef SQLX_SHUTDOWN_TIMEOUT
+# define SQLX_SHUTDOWN_TIMEOUT (10 * G_TIME_SPAN_SECOND)
+#endif
+
+static volatile gboolean udp_allowed = FALSE;
+
 // common_main hooks
 static struct grid_main_option_s * sqlx_service_get_options(void);
 static const char * sqlx_service_usage(void);
@@ -74,7 +81,8 @@ static gpointer _worker_clients (gpointer p);
 
 static const struct gridd_request_descr_s * _get_service_requests (void);
 
-static GError* _reload_lbpool(struct grid_lbpool_s *glp, gboolean flush);
+static GError* _reload_lb_world(
+		struct oio_lb_world_s *lbw, struct oio_lb_s *lb);
 
 // Static variables
 static struct sqlx_service_s SRV = {{0}};
@@ -114,8 +122,15 @@ static struct grid_main_option_s common_options[] =
 			"(milliseconds). -1 means wait forever, 0 return "
 			"immediately." },
 
-	{"MaxBases", OT_UINT, {.u = &SRV.cfg_max_bases},
-		"Limits the number of concurrent open bases (0=automatic)" },
+	{"MaxBaseWaiters", OT_UINT, {.u = &SRV.cfg_max_waiters},
+		"Limits the number of threads waiting on a single base" },
+
+	{"MaxBasesHard", OT_UINT, {.u = &SRV.cfg_max_bases_hard},
+		"Absolute max number of cached bases. Won't ever be increased." },
+	{"MaxBases", OT_UINT, {.u = &SRV.cfg_max_bases_soft},
+		"Limits the number of concurrent open bases (0=automatic), bounded "
+			"to MaxBasesHard naturally"	},
+
 	{"MaxPassive", OT_UINT, {.u = &SRV.cfg_max_passive},
 		"Limits the number of concurrent passive connections (0=automatic)" },
 	{"MaxActive", OT_UINT, {.u = &SRV.cfg_max_active},
@@ -195,11 +210,8 @@ _configure_with_arguments(struct sqlx_service_s *ss, int argc, char **argv)
 	}
 	GRID_DEBUG("NS configured to [%s]", ss->ns_name);
 
-	ss->lb = grid_lbpool_create (ss->ns_name);
-	if (!ss->lb) {
-		GRID_WARN("LB allocation failure");
-		return FALSE;
-	}
+	ss->lb_world = oio_lb_local__create_world();
+	ss->lb = oio_lb__create();
 
 	s = g_strlcpy(ss->volume, argv[1], sizeof(ss->volume));
 	if (s >= sizeof(ss->volume)) {
@@ -209,7 +221,37 @@ _configure_with_arguments(struct sqlx_service_s *ss, int argc, char **argv)
 	}
 	GRID_DEBUG("Volume configured to [%s]", ss->volume);
 
-	ss->zk_url = gridcluster_get_zookeeper(ss->ns_name);
+	/* Load the default ZK url */
+	ss->zk_url = oio_cfg_get_value(ss->ns_name, OIO_CFG_ZOOKEEPER);
+
+	/* if any, use a specific ZK url for the current service type */
+	do {
+		gchar k[sizeof(OIO_CFG_ZOOKEEPER)+2+LIMIT_LENGTH_SRVTYPE];
+		g_snprintf(k, sizeof(k), "%s.%s", OIO_CFG_ZOOKEEPER, ss->service_config->srvtype);
+		gchar *str = oio_cfg_get_value(ss->ns_name, k);
+		if (str)
+			GRID_NOTICE("ZK [%s] <- [%s] (at %s)", ss->zk_url, str, k);
+		else
+			GRID_DEBUG("ZK [%s] (nothing at %s)", ss->zk_url, k);
+		if (str) oio_str_reuse(&ss->zk_url, str);
+	} while (0);
+
+	/* Check if UDP is allowed for servers in the /etc/oio/sds.conf files */
+	gchar *str_udp_allowed = oio_cfg_get_value (ss->ns_name, OIO_CFG_UDP_ALLOWED);
+	if (str_udp_allowed) {
+		udp_allowed = oio_str_parse_bool(str_udp_allowed, FALSE);
+		GRID_NOTICE("UDP %s", udp_allowed ? "allowed" : "forbidden");
+		g_free(str_udp_allowed);
+	}
+
+	/* Check if the logging of outgoing requests has been activated */
+	gchar *str_log_out = oio_cfg_get_value(ss->ns_name, OIO_CFG_LOG_OUTGOING);
+	if (str_log_out) {
+		oio_log_outgoing = oio_str_parse_bool(str_log_out, FALSE);
+		g_free(str_log_out);
+	}
+	if (oio_log_outgoing)
+		GRID_NOTICE("Outgoing requests log [ON]");
 
 	return TRUE;
 }
@@ -239,21 +281,30 @@ _configure_limits(struct sqlx_service_s *ss)
 	}
 
 	// We keep 20 FDs for unexpected cases (sqlite sometimes uses
-	// temporary files, event when we ask for memory journals).
+	// temporary files, even when we ask for memory journals).
 	total = (limit.rlim_cur - 20);
 	// If user sets outstanding values for the first 2 parameters,
 	// there is still 2% available for the 3rd.
 	max = total * 49 / 100;
-	// This is totally arbitrary.
-	min = total / 100;
+	// Hardcoded in sqlx_repository_configure_maxbases()
+	min = 4;
 
 	available = total;
-	// max_bases cannot be changed at runtime, so we set it first and
-	// clamp the other limits accordingly.
+
+	/* max_bases cannot be changed at runtime, so we set it first and
+	 * clamp the other limits accordingly.
+	 * For backward compatibility purposes, we configure the SOFT limit
+	 * instead of the hard one, when only "MaxBases" is set, and we deduce
+	 * the HARD limit. This is the limit the users will see in action, but
+	 * with a room for expansion though.
+	 */
 	CONFIGURE_LIMIT(
-			(ss->cfg_max_bases > 0?
-				ss->cfg_max_bases : SQLX_MAX_BASES_PERCENT(total)),
-			ss->max_bases);
+			(ss->cfg_max_bases_soft > 0?
+				ss->cfg_max_bases_soft : SQLX_MAX_BASES_PERCENT(total)),
+			ss->max_bases_soft);
+	ss->max_bases_hard = ss->cfg_max_bases_hard > 0 ?
+				ss->cfg_max_bases_hard : SQLX_MAX_BASES;
+
 	// max_passive > max_active permits answering to clients while
 	// managing internal procedures (elections, replications...).
 	CONFIGURE_LIMIT(
@@ -265,11 +316,12 @@ _configure_limits(struct sqlx_service_s *ss)
 				ss->cfg_max_active : SQLX_MAX_ACTIVE_PERCENT(total)),
 			ss->max_active);
 
-	GRID_INFO("Limits set to ACTIVES[%u] PASSIVES[%u] BASES[%u] "
+	GRID_INFO("Limits set to ACTIVES[%u] PASSIVES[%u] BASES[%u/%u] "
 			"fd=%u/%u page_size=%u",
-			ss->max_active, ss->max_passive, ss->max_bases,
-			ss->max_active + ss->max_passive + ss->max_bases, (guint)limit.rlim_cur,
-			ss->cfg_page_size);
+			ss->max_active, ss->max_passive,
+			ss->max_bases_soft, ss->max_bases_hard,
+			ss->max_active + ss->max_passive + ss->max_bases_soft,
+			(guint)limit.rlim_cur, ss->cfg_page_size);
 
 	return TRUE;
 #undef CONFIGURE_LIMIT
@@ -308,11 +360,13 @@ _configure_synchronism(struct sqlx_service_s *ss)
 		return TRUE;
 	}
 
-	ss->sync = sqlx_sync_create(ss->zk_url);
+	gboolean shuffle = oio_cfg_get_bool(ss->ns_name, OIO_CFG_ZK_SHUFFLED, TRUE);
+	GRID_INFO("ZK cnx string shuffling [%s]", shuffle ? "ON" : "OFF");
+	ss->sync = sqlx_sync_create(ss->zk_url, shuffle);
 	if (!ss->sync)
 		return FALSE;
 
-	gchar *realprefix = g_strdup_printf("/hc/ns/%s/%s", SRV.ns_name,
+	gchar *realprefix = g_strdup_printf("/hc/ns/%s/%s", ss->ns_name,
 			ss->service_config->zk_prefix);
 	sqlx_sync_set_prefix(ss->sync, realprefix);
 	g_free(realprefix);
@@ -369,6 +423,8 @@ _configure_backend(struct sqlx_service_s *ss)
 	if (ss->cfg_page_size >= 512)
 		repository_config.page_size = ss->cfg_page_size;
 
+	repository_config.max_bases = ss->max_bases_hard;
+
 	GError *err = sqlx_repository_init(ss->volume, &repository_config,
 			&ss->repository);
 	if (err) {
@@ -378,6 +434,10 @@ _configure_backend(struct sqlx_service_s *ss)
 		return FALSE;
 	}
 
+	sqlx_repository_configure_maxbases(SRV.repository, SRV.max_bases_soft);
+	sqlx_cache_set_max_waiting(sqlx_repository_get_cache(SRV.repository),
+			SRV.cfg_max_waiters);
+
 	err = sqlx_repository_configure_type(ss->repository,
 			ss->service_config->srvtype, ss->service_config->schema);
 
@@ -421,7 +481,7 @@ _configure_events_queue (struct sqlx_service_s *ss)
 		return TRUE;
 	}
 
-	gchar *url =  gridcluster_get_eventagent (SRV.ns_name);
+	gchar *url =  oio_cfg_get_eventagent (SRV.ns_name);
 	STRING_STACKIFY (url);
 
 	if (!url) {
@@ -467,6 +527,9 @@ sqlx_service_action(void)
 {
 	GError *err = NULL;
 
+	if (oio_cache_avoid_on_error)
+		GRID_NOTICE("Faulty peers avoidance: ENABLED");
+
 	if (!SRV.flag_nolock) {
 		err = volume_service_lock (SRV.volume, SRV.service_config->srvtype,
 				SRV.announce->str, SRV.ns_name);
@@ -478,7 +541,6 @@ sqlx_service_action(void)
 
 	gridd_client_pool_set_max(SRV.clients_pool, SRV.max_active);
 	network_server_set_maxcnx(SRV.server, SRV.max_passive);
-	sqlx_repository_configure_maxbases(SRV.repository, SRV.max_bases);
 
 	election_manager_set_peering(SRV.election_manager, SRV.peering);
 	if (SRV.sync)
@@ -508,9 +570,10 @@ sqlx_service_action(void)
 			return _action_report_error(err, "Failed to start the QUEUE thread");
 	}
 
-	/* SERVER/GRIDD main run loop */
+	/* open all the sockets */
 	if (!grid_main_is_running())
 		return;
+	network_server_allow_udp(SRV.server);
 	grid_daemon_bind_host(SRV.server, SRV.url->str, SRV.dispatcher);
 	err = network_server_open_servers(SRV.server);
 	if (NULL != err)
@@ -518,6 +581,13 @@ sqlx_service_action(void)
 	if (!grid_main_is_running())
 		return;
 
+	if (udp_allowed) {
+		int fd_udp = network_server_first_udp(SRV.server);
+		GRID_DEBUG("UDP socket fd=%d", fd_udp);
+		sqlx_peering_direct__set_udp(SRV.peering, fd_udp);
+	}
+
+	/* SERVER/GRIDD main run loop */
 	if (NULL != (err = network_server_run(SRV.server)))
 		return _action_report_error(err, "GRIDD run failure");
 }
@@ -538,15 +608,16 @@ static gboolean
 sqlx_service_configure(int argc, char **argv)
 {
 	return _configure_limits(&SRV)
-	    && _init_configless_structures(&SRV)
-	    && _configure_with_arguments(&SRV, argc, argv)
+		&& _init_configless_structures(&SRV)
+		&& _configure_with_arguments(&SRV, argc, argv)
+		/* NS now known! */
 		&& _configure_synchronism(&SRV)
-	    && _configure_peering(&SRV)
-	    && _configure_replication(&SRV)
-	    && _configure_backend(&SRV)
-	    && _configure_tasks(&SRV)
-	    && _configure_network(&SRV)
-	    && _configure_events_queue(&SRV)
+		&& _configure_peering(&SRV)
+		&& _configure_replication(&SRV)
+		&& _configure_backend(&SRV)
+		&& _configure_tasks(&SRV)
+		&& _configure_network(&SRV)
+		&& _configure_events_queue(&SRV)
 		&& (!SRV.service_config->post_config
 				|| SRV.service_config->post_config(&SRV));
 }
@@ -557,7 +628,9 @@ sqlx_service_set_defaults(void)
 	SRV.max_elections_timers_per_round = SQLX_MAX_TIMER_PER_ROUND;
 	SRV.open_timeout = DEFAULT_CACHE_OPEN_TIMEOUT / G_TIME_SPAN_MILLISECOND;
 
-	SRV.cfg_max_bases = 0;
+	SRV.cfg_max_waiters = SQLX_MAX_WAITING;
+	SRV.cfg_max_bases_soft = 0;
+	SRV.cfg_max_bases_hard = SQLX_MAX_BASES;
 	SRV.cfg_max_passive = 0;
 	SRV.cfg_max_active = 0;
 	SRV.cfg_max_workers = 200;
@@ -585,7 +658,6 @@ sqlx_service_specific_fini(void)
 
 	if (SRV.server) {
 		network_server_close_servers(SRV.server);
-		network_server_stop(SRV.server);
 	}
 
 	if (SRV.thread_reload)
@@ -603,13 +675,18 @@ sqlx_service_specific_fini(void)
 		if (cache)
 			sqlx_cache_expire(cache, G_MAXUINT, 0);
 	}
-	if (SRV.election_manager)
-		election_manager_exit_all(SRV.election_manager, 0, TRUE);
+	if (election_manager_is_operational(SRV.election_manager))
+		election_manager_exit_all(SRV.election_manager,
+				SQLX_SHUTDOWN_TIMEOUT, TRUE);
 	if (SRV.sync)
 		sqlx_sync_close(SRV.sync);
 	if (SRV.peering)
 		sqlx_peering__destroy(SRV.peering);
 
+	// Stop the server AFTER cleaning all elections
+	if (SRV.server)
+		network_server_stop(SRV.server);
+
 	// Cleanup
 	if (SRV.gtq_admin) {
 		grid_task_queue_destroy(SRV.gtq_admin);
@@ -632,10 +709,6 @@ sqlx_service_specific_fini(void)
 		sqlx_repository_clean(SRV.repository);
 		SRV.repository = NULL;
 	}
-	if (SRV.sync) {
-		sqlx_sync_clear(SRV.sync);
-		SRV.sync = NULL;
-	}
 	if (SRV.resolver) {
 		hc_resolver_destroy(SRV.resolver);
 		SRV.resolver = NULL;
@@ -657,15 +730,27 @@ sqlx_service_specific_fini(void)
 		SRV.clients_pool = NULL;
 	}
 
+	if (SRV.clients_factory) {
+		gridd_client_factory_clean(SRV.clients_factory);
+		SRV.clients_factory = NULL;
+	}
+
 	// Must be freed after SRV.clients_pool
 	if (SRV.election_manager) {
 		election_manager_clean(SRV.election_manager);
 		SRV.election_manager = NULL;
 	}
+	if (SRV.sync) {
+		sqlx_sync_clear(SRV.sync);
+		SRV.sync = NULL;
+	}
 
-	if (SRV.lb) {
-		grid_lbpool_destroy (SRV.lb);
-		SRV.lb = NULL;
+	if (SRV.lb)
+		oio_lb__clear(&SRV.lb);
+
+	if (SRV.lb_world) {
+		oio_lb_world__destroy(SRV.lb_world);
+		SRV.lb_world = NULL;
 	}
 
 	if (SRV.events_queue) {
@@ -687,7 +772,9 @@ static guint
 _count_options (struct grid_main_option_s const * tab)
 {
 	guint count = 0;
-	for (; tab && tab->name ;++tab,++count) {}
+	if (tab)
+		for (; tab->name; ++tab, ++count)
+			;
 	return count;
 }
 
@@ -886,6 +973,126 @@ _task_reconfigure_events (gpointer p)
 	}
 }
 
+static void
+_reload_lb_service_types(struct oio_lb_world_s *lbw, struct oio_lb_s *lb,
+		gchar **srvtypes, GPtrArray *tabsrv, GPtrArray *taberr)
+{
+	struct service_update_policies_s *pols = service_update_policies_create();
+	gchar *pols_cfg = gridcluster_get_service_update_policy(SRV.nsinfo);
+	service_update_reconfigure(pols, pols_cfg);
+	g_free(pols_cfg);
+
+	for (guint i=0; srvtypes[i] ;++i) {
+		const char * srvtype = srvtypes[i];
+		if (!oio_lb__has_pool(lb, srvtype)) {
+			GRID_DEBUG("Creating pool for service type [%s]", srvtype);
+			oio_lb__force_pool(lb,
+					oio_lb_pool__from_service_policy( lbw, srvtype, pols));
+		}
+
+		if (!taberr->pdata[i])
+			oio_lb_world__feed_service_info_list(lbw, tabsrv->pdata[i]);
+	}
+
+	service_update_policies_destroy(pols);
+}
+
+static void
+_free_list_of_services(gpointer p)
+{
+	if (!p)
+		return;
+	g_slist_free_full((GSList*)p, (GDestroyNotify)service_info_clean);
+}
+
+static void
+_free_error(gpointer p)
+{
+	if (!p)
+		return;
+	g_error_free((GError*)p);
+}
+
+static GError*
+_reload_lb_world(struct oio_lb_world_s *lbw, struct oio_lb_s *lb)
+{
+	gchar **srvtypes = NULL;
+	GPtrArray *tabsrv = NULL, *taberr = NULL;
+	gboolean any_loading_error = FALSE;
+
+	/* Load the list of service types */
+	if (SRV.srvtypes[0] && SRV.srvtypes[0] != '!') {
+		srvtypes = g_strsplit(SRV.srvtypes, ",", -1);
+	} else {
+		GSList *list_srvtypes = NULL;
+		GError *err = conscience_get_types(SRV.ns_name, &list_srvtypes);
+		if (err) {
+			g_prefix_error(&err, "LB pool reload error: ");
+			return err;
+		}
+		if (!SRV.srvtypes[0])
+			srvtypes = (gchar**) metautils_list_to_array(list_srvtypes);
+		else {
+			/* the application gives an exclusion list */
+			EXTRA_ASSERT(SRV.srvtypes[0] == '!');
+			srvtypes = g_malloc0(sizeof(void*) *
+					(1 + g_slist_length(list_srvtypes)));
+			guint i = 0;
+			for (GSList *l=list_srvtypes; l ;l=l->next) {
+				const char * srvtype = l->data;
+				if (!g_strstr_len(SRV.srvtypes+1, -1, srvtype)) {
+					/* service type not excluded */
+					srvtypes[i++] = l->data;
+				} else {
+					g_free(l->data);
+				}
+			}
+		}
+		g_slist_free(list_srvtypes);
+	}
+
+	EXTRA_ASSERT(srvtypes != NULL);
+	if (!*srvtypes) {
+		g_strfreev(srvtypes);
+		return NULL;
+	}
+
+	/* Now preload all the service of these types */
+	tabsrv = g_ptr_array_new_full(8, _free_list_of_services);
+	taberr = g_ptr_array_new_full(8, _free_error);
+	for (char **pst=srvtypes; *pst ;++pst) {
+		const char * srvtype = *pst;
+		GSList *srv = NULL;
+		GError *e = conscience_get_services(SRV.ns_name, srvtype, FALSE, &srv);
+		if (e) {
+			GRID_WARN("Failed to load the list of [%s] in NS=%s", SRV.ns_name, srvtype);
+			any_loading_error = TRUE;
+		}
+		g_ptr_array_add(tabsrv, srv);
+		g_ptr_array_add(taberr, e);
+	}
+
+	/* Now refresh the service pools. We don't trigger any purge if we
+	 * encountered any error while loading the list of services, because
+	 * the world exposes a global generation number to manage expirations,
+	 * and because without service, we have no way (yet) to find *all* the
+	 * slots concerned by any service of a given type. */
+	if (*srvtypes) {
+		if (!any_loading_error)
+			oio_lb_world__increment_generation(lbw);
+		oio_lb_world__reload_pools(lbw, lb, SRV.nsinfo);
+		_reload_lb_service_types(lbw, lb, srvtypes, tabsrv, taberr);
+		oio_lb_world__reload_storage_policies(lbw, lb, SRV.nsinfo);
+		if (!any_loading_error)
+			oio_lb_world__purge_old_generations(lbw);
+	}
+
+	if (taberr) g_ptr_array_free(taberr, TRUE);
+	if (tabsrv) g_ptr_array_free(tabsrv, TRUE);
+	g_strfreev(srvtypes);
+	return NULL;
+}
+
 void
 sqlx_task_reload_lb (struct sqlx_service_s *ss)
 {
@@ -893,78 +1100,19 @@ sqlx_task_reload_lb (struct sqlx_service_s *ss)
 	ADAPTIVE_PERIOD_DECLARE();
 
 	if (!grid_main_is_running ())
-		return;
-	if (!ss->lb || !ss->nsinfo)
-		return;
+		return;  /* stopped */
+	if (!SRV.nsinfo || !SRV.nsinfo)
+		return;  /* not ready */
 	if (ADAPTIVE_PERIOD_SKIP())
 		return;
 
-	GError *err = _reload_lbpool(ss->lb, FALSE);
-	if (!err) {
-		ADAPTIVE_PERIOD_ONSUCCESS(10);
-	} else {
-		GRID_WARN("Failed to reload the LB pool services: (%d) %s",
-				err->code, err->message);
+	GError *err = _reload_lb_world(ss->lb_world, ss->lb);
+	if (err) {
+		GRID_WARN("Failed to reload LB world: %s", err->message);
 		g_clear_error(&err);
+	} else {
+		ADAPTIVE_PERIOD_ONSUCCESS(10);
 	}
-
-	grid_lbpool_reconfigure(ss->lb, ss->nsinfo);
-}
-
-static GError*
-_reload_lbpool(struct grid_lbpool_s *glp, gboolean flush)
-{
-	gboolean _reload_srvtype(const gchar *ns, const gchar *srvtype) {
-		GSList *list_srv = NULL;
-		GError *err = conscience_get_services (ns, srvtype, FALSE, &list_srv);
-		if (err) {
-			GRID_WARN("Gridagent/conscience error: Failed to list the services"
-					" of type [%s]: code=%d %s", srvtype, err->code,
-					err->message);
-			g_clear_error(&err);
-			return FALSE;
-		}
-
-		if (list_srv || flush) {
-			GSList *l = list_srv;
-
-			gboolean provide(struct service_info_s **p_si) {
-				if (!l)
-					return FALSE;
-				*p_si = l->data;
-				l->data = NULL;
-				l = l->next;
-				return TRUE;
-			}
-			grid_lbpool_reload(glp, srvtype, provide);
-			g_slist_free(list_srv);
-		}
-
-		return TRUE;
-	}
-
-	GSList *list_srvtypes = NULL;
-	GError *err = conscience_get_types (grid_lbpool_namespace(glp), &list_srvtypes);
-	if (err)
-		g_prefix_error(&err, "LB pool reload error: ");
-	else {
-		guint errors = 0;
-		const gchar *ns = grid_lbpool_namespace(glp);
-
-		for (GSList *l=list_srvtypes; l ;l=l->next) {
-			if (!l->data)
-				continue;
-			if (!_reload_srvtype(ns, l->data))
-				++ errors;
-		}
-
-		if (errors)
-			GRID_DEBUG("Reloaded %u service types, with %u errors",
-					g_slist_length(list_srvtypes), errors);
-	}
-
-	g_slist_free_full (list_srvtypes, g_free);
-	return err;
 }
 
 /* Specific requests handlers ----------------------------------------------- */
@@ -976,7 +1124,9 @@ _dispatch_RELOAD (struct gridd_reply_ctx_s *reply, gpointer pss, gpointer i)
 	struct sqlx_service_s *ss = pss;
 	g_assert (ss != NULL);
 
-	GError *err = _reload_lbpool(ss->lb, TRUE);
+	oio_lb_world__flush(ss->lb_world);
+
+	GError *err = _reload_lb_world(ss->lb_world, ss->lb);
 	if (err)
 		reply->send_error (0, err);
 	else
@@ -990,7 +1140,6 @@ _dispatch_FLUSH (struct gridd_reply_ctx_s *reply, gpointer pss, gpointer i)
 	(void) i;
 	struct sqlx_service_s *ss = pss;
 	g_assert(ss != NULL);
-	/* if (ss->lb) grid_lbpool_flush (ss->lb); */
 	if (ss->resolver) {
 		hc_resolver_flush_csm0 (ss->resolver);
 		hc_resolver_flush_services (ss->resolver);
