@@ -38,17 +38,6 @@ def gen_names():
             yield i, '{0}/{1}/plop'.format(c0, c1)
 
 
-def gen_props(n, v):
-    """Generates 'n' properties tuples, whose value is prefixed with 'v'"""
-    for i in range(n):
-        yield "user.k{0}".format(i), v+str(i)
-
-
-def user_props(b):
-    """Filters user's properties from the set of the container properties."""
-    return dict((k, v) for k, v in b.items() if k.startswith("user."))
-
-
 class TestMeta2Containers(BaseTestCase):
 
     def setUp(self):
@@ -68,10 +57,14 @@ class TestMeta2Containers(BaseTestCase):
         except:
             pass
 
-    def _create(self, params, code):
-        headers = {'X-oio-action-mode': 'autocreate'}
-        resp = self.session.post(self.url_container('create'),
-                                 params=params, headers=headers)
+    def _create(self, params, code, autocreate=True):
+        headers = {}
+        if autocreate:
+            headers['x-oio-action-mode'] = 'autocreate'
+        data = json.dumps({'properties': {}})
+        resp = self.session.post(
+            self.url_container('create'), params=params, data=data,
+            headers=headers)
         self.assertEqual(resp.status_code, code)
 
     def _delete(self, params):
@@ -83,30 +76,6 @@ class TestMeta2Containers(BaseTestCase):
                                  headers={'X-oio-action-mode': 'force'})
         self.assertEqual(resp.status_code, 204)
 
-    def test_cycle_container(self):
-        params = self.param_ref(self.ref)
-
-        resp = self.session.get(self.url_container('show'), params=params)
-        self.assertEqual(resp.status_code, 404)
-
-        resp = self.session.post(self.url_container('create'),
-                                 params=params)
-        self.assertEqual(resp.status_code, 403)
-        self._create(params, 204)
-
-        resp = self.session.get(self.url_container('show'), params=params)
-        self.assertEqual(resp.status_code, 204)
-        # TODO check the headers
-
-        self._create(params, 201)
-
-        self._delete(params)
-        resp = self.session.post(self.url_container('destroy'), params=params)
-        self.assertEqual(resp.status_code, 404)
-
-        resp = self.session.get(self.url_container('show'), params=params)
-        self.assertEqual(resp.status_code, 404)
-
     def check_list_output(self, body, nbobj, nbpref):
         self.assertIsInstance(body, dict)
         self.assertIn('prefixes', body)
@@ -116,6 +85,79 @@ class TestMeta2Containers(BaseTestCase):
         self.assertIsInstance(body['objects'], list)
         self.assertEqual(len(body['objects']), nbobj)
 
+    def test_create_many(self):
+        params = self.param_ref(self.ref)
+        headers = {}
+        headers['x-oio-action-mode'] = 'autocreate'
+        headers['Content-Type'] = 'application/json'
+
+        # Create different uploads
+        data = ('{"containers":' +
+                '[{"name":"test1","properties":{},"system":{}},' +
+                '{"name":"test2","properties":{},"system":{}}]}')
+        resp = self.session.post(
+            self.url_container('create_many'), params=params, data=data,
+            headers=headers)
+        self.assertEqual(resp.status_code, 200)
+        data = json.loads(resp.content)["containers"]
+        self.assertEqual(data[0]["status"], 201)
+        self.assertEqual(data[1]["status"], 201)
+        self._delete(self.param_ref("test1"))
+        self._delete(self.param_ref("test2"))
+
+        # Create same upload
+        data = ('{"containers":' +
+                '[{"name":"test1","properties":{},"system":{}},' +
+                '{"name":"test1","properties":{},"system":{}}]}')
+        resp = self.session.post(
+            self.url_container('create_many'), params=params, data=data,
+            headers=headers)
+        self.assertEqual(resp.status_code, 200)
+        data = json.loads(resp.content)["containers"]
+        self.assertEqual(data[0]["status"], 201)
+        self.assertEqual(data[1]["status"], 433)
+        self._delete(self.param_ref("test1"))
+
+        # Empty body should be answered with an error
+        resp = self.session.post(
+            self.url_container('create_many'), params=params,
+            headers=headers)
+        self.assertEqual(resp.status_code, 400)
+
+        # Create with  missing name
+        data = ('{"containers":' +
+                '[{"properties":{},"system":{}}' +
+                ']}')
+        resp = self.session.post(
+            self.url_container('create_many'), params=params, data=data,
+            headers=headers)
+        self.assertEqual(resp.status_code, 400)
+
+        # Send a non conform json (missing '{')
+        data = ('{"containers":' +
+                '["name":"test","properties":{},"system":{}}' +
+                ']}')
+        resp = self.session.post(
+            self.url_container('create_many'), params=params, data=data,
+            headers=headers)
+        self.assertEqual(resp.status_code, 400)
+
+        # Don't send account
+        data = ('{"containers":' +
+                '[{"name":"test1","properties":{},"system":{}}' +
+                ']}')
+        resp = self.session.post(
+            self.url_container('create_many'), data=data,
+            headers=headers)
+        self.assertEqual(resp.status_code, 400)
+
+        # Send empty array
+        data = ('{"containers":[]}')
+        resp = self.session.post(
+            self.url_container('create_many'), data=data,
+            headers=headers)
+        self.assertEqual(resp.status_code, 400)
+
     def test_list(self):
         params = self.param_ref(self.ref)
         self._create(params, 204)
@@ -131,11 +173,11 @@ class TestMeta2Containers(BaseTestCase):
             p = "X-oio-content-meta-"
             headers = {p+"policy": "NONE",
                        p+"id": h,
-                       p+"version": 0,
+                       p+"version": "0",
                        p+"hash": "0"*32,
                        p+"length": "0",
                        p+"mime-type": "application/octet-stream",
-                       p+"chunk-method": "plain/bytes"}
+                       p+"chunk-method": "plain/nb_copy=3"}
             p = self.param_content(self.ref, name)
             body = json.dumps([chunk, ])
             resp = self.session.post(self.url_content('create'),
@@ -183,120 +225,6 @@ class TestMeta2Containers(BaseTestCase):
         self.check_list_output(resp.json(), 8, 0)
         del params['marker_end']
 
-    def check_prop_output(self, body, ref):
-        self.assertIsInstance(body, dict)
-        self.assertDictEqual(user_props(body), ref)
-
-    def test_properties_noent(self):
-        params = self.param_ref(self.ref)
-
-        resp = self.session.post(self.url_container('get_properties'),
-                                 params=params)
-        self.assertEqual(resp.status_code, 404)
-        resp = self.session.post(self.url_container('set_properties'),
-                                 params=params)
-        self.assertEqual(resp.status_code, 404)
-        resp = self.session.post(self.url_container('del_properties'),
-                                 params=params, data=json.dumps([]))
-        self.assertEqual(resp.status_code, 404)
-
-    def test_properties_none(self):
-        params = self.param_ref(self.ref)
-        self._create(params, 204)
-
-        # chek no props after creation
-        resp = self.session.post(self.url_container('get_properties'),
-                                 params=params)
-        self.assertEqual(resp.status_code, 200)
-        body = resp.json()
-        self.assertIsInstance(body, dict)
-        self.assertGreater(len(body), 0)
-        self.assertDictEqual(user_props(body), {})
-
-        # check set/del works on no set
-        resp = self.session.post(self.url_container('set_properties'),
-                                 params=params)
-        self.assertEqual(resp.status_code, 200)
-
-        resp = self.session.post(self.url_container('del_properties'),
-                                 params=params, data=json.dumps([]))
-        self.assertEqual(resp.status_code, 200)
-
-    def test_properties(self):
-        params = self.param_ref(self.ref)
-        self._create(params, 204)
-
-        # Check the simple SET works
-        data = dict(gen_props(256, 'val'))
-        resp = self.session.post(self.url_container('set_properties'),
-                                 params=params, data=json.dumps(data))
-        self.assertEqual(resp.status_code, 200)
-
-        resp = self.session.post(self.url_container('get_properties'),
-                                 params=params)
-        self.assertEqual(resp.status_code, 200)
-        self.check_prop_output(resp.json(), data)
-
-        # check SET overriding works
-        change = {"user.k0": "XXX"}
-        resp = self.session.post(self.url_container('set_properties'),
-                                 params=params, data=json.dumps(change))
-        self.assertEqual(resp.status_code, 200)
-
-        data.update(change)
-        resp = self.session.post(self.url_container('get_properties'),
-                                 params=params)
-        self.assertEqual(resp.status_code, 200)
-        self.check_prop_output(resp.json(), data)
-
-        # check the FLUSH/REPLACE works
-        data = dict(gen_props(16, 'XXXX'))
-        params1 = dict(params)
-        params1['flush'] = '1'
-        resp = self.session.post(self.url_container('set_properties'),
-                                 params=params1, data=json.dumps(data))
-        self.assertEqual(resp.status_code, 200)
-
-        resp = self.session.post(self.url_container('get_properties'),
-                                 params=params)
-        self.assertEqual(resp.status_code, 200)
-        self.check_prop_output(resp.json(), data)
-
-        # check the simple delete works
-        resp = self.session.post(self.url_container('del_properties'),
-                                 params=params, data=json.dumps(["user.k0"]))
-        self.assertEqual(resp.status_code, 200)
-
-        del data["user.k0"]
-        resp = self.session.post(self.url_container('get_properties'),
-                                 params=params)
-        self.assertEqual(resp.status_code, 200)
-        self.check_prop_output(resp.json(), data)
-
-        # check the DELETE(all) works
-        resp = self.session.post(self.url_container('del_properties'),
-                                 params=params, data=json.dumps([]))
-        self.assertEqual(resp.status_code, 200)
-
-        data = dict()
-        resp = self.session.post(self.url_container('get_properties'),
-                                 params=params)
-        self.assertEqual(resp.status_code, 200)
-        self.check_prop_output(resp.json(), data)
-
-        # check the FLUSH works
-        data = dict(gen_props(32, "kjlkqjlxqjs"))
-        resp = self.session.post(self.url_container('set_properties'),
-                                 params=params, data=json.dumps(data))
-        self.assertEqual(resp.status_code, 200)
-
-        params1 = dict(params)
-        params1['flush'] = '1'
-        resp = self.session.post(self.url_container('set_properties'),
-                                 params=params1, data=json.dumps({}))
-        self.assertEqual(resp.status_code, 200)
-        self.check_prop_output(resp.json(), {})
-
     def test_touch(self):
         params = self.param_ref(self.ref)
         resp = self.session.post(self.url_container('touch'), params=params)
@@ -389,6 +317,7 @@ class TestMeta2Contents(BaseTestCase):
                                  data=json.dumps({'size': 1024}))
         self.assertError(resp, 404, 406)
         resp = self.session.post(self.url_content('prepare'),
+
                                  params=params,
                                  data=json.dumps({'size': 1024}),
                                  headers=headers)
@@ -413,6 +342,126 @@ class TestMeta2Contents(BaseTestCase):
 
         # TODO check SPARE requests reaching the meta2 server
 
+    def _create_content(self, name):
+        headers = {'X-oio-action-mode': 'autocreate'}
+        params = self.param_content(self.ref, name)
+        resp = self.session.post(self.url_content('prepare'),
+                                 data=json.dumps({'size': '1024'}),
+                                 params=params,
+                                 headers=headers)
+        self.assertEqual(resp.status_code, 200)
+        chunks = resp.json()
+
+        headers = {'x-oio-action-mode': 'autocreate',
+                   'x-oio-content-meta-length': '1024'}
+        resp = self.session.post(self.url_content('create'),
+                                 params=params,
+                                 headers=headers,
+                                 data=json.dumps(chunks))
+        self.assertEqual(resp.status_code, 204)
+
+    def test_delete_many(self):
+        # Send no account
+        params = self.param_ref(self.ref)
+        params['acct'] = ""
+        resp = self.session.post(self.url_content('delete_many'),
+                                 params=params)
+        self.assertError(resp, 400, 400)
+
+        # Send no container
+        params = self.param_ref("")
+        resp = self.session.post(self.url_content('delete_many'),
+                                 params=params)
+        self.assertError(resp, 400, 400)
+
+        # Send empty body
+        params = self.param_ref(self.ref)
+        resp = self.session.post(self.url_content('delete_many'),
+                                 params=params)
+        self.assertError(resp, 400, 400)
+
+        # Send empty content
+        data = ('{"contents"}')
+        resp = self.session.post(self.url_content('delete_many'),
+                                 params=params, data=data)
+        self.assertError(resp, 400, 400)
+
+        # Send empty array
+        data = ('{"contents":[]}')
+        resp = self.session.post(self.url_content('delete_many'),
+                                 params=params, data=data)
+        self.assertError(resp, 400, 400)
+
+        # Send one existent
+        self._create_content('should_exist')
+        data = ('{"contents":[{"name":"should_exist"}]}')
+        resp = self.session.post(self.url_content('delete_many'),
+                                 params=params, data=data)
+        json_data = resp.json()
+        self.assertEqual(resp.status_code, 200)
+        self.assertEqual(json_data['contents'][0]['status'], 204)
+
+        # Send one nonexistent
+        data = ('{"contents":[{"name":"should_not_exist"}]}')
+        resp = self.session.post(self.url_content('delete_many'),
+                                 params=params, data=data)
+        json_data = resp.json()
+        self.assertEqual(json_data['contents'][0]['status'], 420)
+        # Send one existent and one nonexistent
+        self._create_content('should_exist')
+        data = ('{"contents":[{"name":"should_exist"},'
+                + '{"name":"should_not_exist"}]}')
+        resp = self.session.post(self.url_content('delete_many'),
+                                 params=params, data=data)
+        json_data = resp.json()
+        self.assertEqual(resp.status_code, 200)
+        self.assertEqual(json_data['contents'][0]['status'], 204)
+        self.assertEqual(json_data['contents'][1]['status'], 420)
+        # Send 2 nonexistents
+        data = ('{"contents":[{"name":"should_not_exist"},'
+                + '{"name":"should_also_not_exist"}]}')
+        resp = self.session.post(self.url_content('delete_many'),
+                                 params=params, data=data)
+        json_data = resp.json()
+        self.assertEqual(json_data['contents'][0]['status'], 420)
+        self.assertEqual(json_data['contents'][1]['status'], 420)
+        # Send 2 existents
+        self._create_content('should_exist')
+        self._create_content('should_also_exist')
+        data = ('{"contents":[{"name":"should_exist"},'
+                + '{"name":"should_also_exist"}]}')
+        resp = self.session.post(self.url_content('delete_many'),
+                                 params=params, data=data)
+        json_data = resp.json()
+        self.assertEqual(resp.status_code, 200)
+        self.assertEqual(json_data['contents'][0]['status'], 204)
+        self.assertEqual(json_data['contents'][1]['status'], 204)
+
+        strange_paths = [
+            "Annual report.txt",
+            "foo+bar=foobar.txt",
+            "100%_bug_free.c",
+            "forward/slash/allowed",
+            "I\\put\\backslashes\\and$dollar$signs$in$file$names",
+            "Je suis tombé sur la tête, mais ça va bien.",
+            "%s%f%u%d%%",
+            "carriage\rreturn",
+            "line\nfeed",
+            "ta\tbu\tla\ttion",
+            "controlchars",
+        ]
+        contents = []
+        for name in strange_paths:
+            self._create_content(name)
+            contents.append({"name": name})
+        data = json.dumps({"contents": contents})
+        resp = self.session.post(self.url_content('delete_many'),
+                                 params=params, data=data)
+        json_data = resp.json()
+        self.assertEqual(resp.status_code, 200)
+        for r in json_data['contents']:
+            self.assertEqual(r['status'], 204)
+
     def test_copy(self):
         path = random_content()
         to = '{0}/{1}/{2}//{3}'.format(self.ns, self.account, self.ref,
@@ -436,8 +485,10 @@ class TestMeta2Contents(BaseTestCase):
         self.assertError(resp, 403, 406)
 
         # No content
+        data = json.dumps({'properties': {}})
         resp = self.session.post(self.url_container('create'),
-                                 params=params, headers=headers)
+                                 params=params, headers=headers,
+                                 data=data)
         self.assertEqual(resp.status_code, 204)
         resp = self.session.post(self.url_content('copy'),
                                  headers=headers, params=params)
@@ -454,7 +505,8 @@ class TestMeta2Contents(BaseTestCase):
             self.assertEqual(resp.status_code, 200)
             body = resp.json()
             self.assertIsInstance(body, dict)
-            self.assertDictEqual(body, expected)
+            self.assertIsInstance(body.get('properties'), dict)
+            self.assertDictEqual(expected, body['properties'])
 
         def del_ok(keys):
             resp = self.session.post(self.url_content('del_properties'),
@@ -463,7 +515,8 @@ class TestMeta2Contents(BaseTestCase):
 
         def set_ok(kv):
             resp = self.session.post(self.url_content('set_properties'),
-                                     params=params, data=json.dumps(kv))
+                                     params=params,
+                                     data=json.dumps({'properties': kv}))
             self.assertEqual(resp.status_code, 204)
 
         # GetProperties on no content
@@ -480,7 +533,7 @@ class TestMeta2Contents(BaseTestCase):
         chunks = resp.json()
 
         headers = {'X-oio-action-mode': 'autocreate',
-                   'X-oio-content-meta-length': 1024}
+                   'X-oio-content-meta-length': '1024'}
         resp = self.session.post(self.url_content('create'),
                                  params=params,
                                  headers=headers,
@@ -501,7 +554,7 @@ class TestMeta2Contents(BaseTestCase):
 
     def test_cycle_content(self):
         path = random_content()
-        headers = {'X-oio-action-mode': 'autocreate'}
+        headers = {'x-oio-action-mode': 'autocreate'}
         params = self.param_content(self.ref, path)
 
         resp = self.session.get(self.url_content('show'), params=params)
@@ -511,14 +564,14 @@ class TestMeta2Contents(BaseTestCase):
         self.assertError(resp, 403, 406)
 
         resp = self.session.post(self.url_content('prepare'),
-                                 data=json.dumps({'size': 1024}),
+                                 data=json.dumps({'size': '1024'}),
                                  params=params,
                                  headers=headers)
         self.assertEqual(resp.status_code, 200)
         chunks = resp.json()
 
-        headers = {'X-oio-action-mode': 'autocreate',
-                   'X-oio-content-meta-length': 1024}
+        headers = {'x-oio-action-mode': 'autocreate',
+                   'x-oio-content-meta-length': '1024'}
         resp = self.session.post(self.url_content('create'),
                                  params=params,
                                  headers=headers,
