@@ -39,10 +39,6 @@ while getopts "s:n:l" opt ; do
 	esac
 done
 
-if [ -z "$NS" ] && [ -n "$LOCAL" ] ; then
-	NS=$(${PREFIX}-test-config.py -n)
-fi
-
 if [ -z "$NS" ] ; then
 	echo "No namespace configured"
 	exit 1
